/*
  Provádí kontrolu, zda při mazání dokladu či položky dokladu 
  nezůstávají ve vazební tabulce nějaké vazby. V takovém případě
  trigger selže.

  Autor: Pavel Cvrček
*/

CREATE OR REPLACE FUNCTION dzpoldoklcheckvazby()
  RETURNS trigger AS $$
declare
  _idPolDokl integer;
  _notInDVazebTab boolean;
begin
  if (OLD.modul in( 'FAV','FAP','PHL','ZAV')) then
    _idPolDokl:= OLD.idPolFak;
  else
    if (OLD.modul in( 'SKL')) then
      _idPolDokl:= OLD.idPolSklad;
    else
      if (OLD.modul in( 'BAN','POK','INT')) then
        _idPolDokl:= OLD.idPolInt;
      else
        if (OLD.modul in ('PPP', 'NAV', 'OBP', 'PPV', 'NAP', 'OBV')) then
          _idPolDokl := OLD.idPolObch;
        else
          raise exception 'dPolDoklCheckVazby(): Neznámý modul: %', OLD.modul;
        end if;
      end if;
    end if;
  end if;
  
  _notInDVazebTab:= (select dZkontrolujVazebniTab (_idPolDokl, false, OLD.modul));
  if (_notInDVazebTab = false) then
    raise exception '0003:dDoklCheckVazby(): Ve vazební tabulce zůstávají vazby na položku dokladu.';
  end if;
  
  return OLD;
end;
$$ LANGUAGE plpgsql;

drop trigger if exists dzPolFakCheckVazby on dpolfak;
drop trigger if exists dzPolIntCheckVazby on dpolint;
drop trigger if exists dzPolSkladCheckVazby on dpolsklad;

create trigger dzPolFakCheckVazby before delete on dpolfak 
for each row execute procedure dzPolDoklCheckVazby();

create trigger dzPolIntCheckVazby before delete on dpolint 
for each row execute procedure dzPolDoklCheckVazby();

create trigger dzPolSkladCheckVazby before delete on dpolsklad 
for each row execute procedure dzPolDoklCheckVazby();

CREATE TRIGGER dzpolobchcheckvazby BEFORE DELETE ON dpolobch
  FOR EACH ROW EXECUTE PROCEDURE dzpoldoklcheckvazby();