------------------------------------------------------------------------------
-- Bug 45255 - Přepočet stavu účtů nerespektuje nově přidané pole do události 
------------------------------------------------------------------------------

CREATE OR REPLACE FUNCTION mudalostiinsertucto() RETURNS trigger
    AS $$
begin
  -- odpisy majetku jsou vedeny vzdy v mistni mene, proto za menu
  -- dosazujeme -1 a za sumZklMen 0.0
  execute createOrUpdateObraty(NEW.Ucetni, false, NEW.DatUcto, NEW.idZklMdUcet, NEW.idZklDalUcet,
        -1, NEW.idStred, NEW.sumZkl, 0.0);

  -- u udalosti castecneho nebo uplneho vyrazeni musime zauctovat take zustatek udalosti
  if (NEW.typudalostik in ('typUdal.uplVyr', 'typUdal.casVyr')) then
    -- účetní částka má při uvádění do obratů přednost (známe-li ji, tak použijeme tu)
    if ((select nahrucetodpk from mmajetek as M where M.idmajetku = NEW.idmajetku) = 'nahrUcet.ne') then 
        execute createOrUpdateObraty(NEW.Ucetni, false, NEW.DatUcto, NEW.idDphMdUcet, NEW.idDphDalUcet, 
            -1, NEW.idStred, NEW.sumdphdanove, 0.0); 
    else 
        execute createOrUpdateObraty(NEW.Ucetni, false, NEW.DatUcto, NEW.idDphMdUcet, NEW.idDphDalUcet, 
            -1, NEW.idStred, NEW.sumdph, 0.0); 
    end if;
  end if;

  return NEW;
end;
$$ LANGUAGE plpgsql;
ALTER FUNCTION public.mudalostiinsertucto() OWNER TO %firmaRole%;


CREATE OR REPLACE FUNCTION mudalostiupdateucto() RETURNS trigger
    AS $$
begin
  -- odpisy majetku jsou vedeny vzdy v mistni mene, proto za menu
  -- dosazujeme -1 a za sumZklMen 0.0
  execute createOrUpdateObraty(OLD.Ucetni, false, OLD.DatUcto, OLD.idZklMdUcet, OLD.idZklDalUcet,
       -1, OLD.idStred, -OLD.sumZkl, 0.0);

  -- pokud byla drive udalost castecne nebo uplne vyrazeni musime ji odebrat z obratu
  if (OLD.typUdalostiK in ('typUdal.uplVyr', 'typUdal.casVyr')) then
    -- účetní částka má při uvádění do obratů přednost (známe-li ji, tak použijeme tu)
    if  ((select nahrucetodpk from mmajetek as M where M.idmajetku = OLD.idmajetku) <> 'nahrUcet.ne') then--(OLD.sumdph <> 0.0) then 
        execute createOrUpdateObraty(OLD.Ucetni, false, OLD.DatUcto, OLD.idDphMdUcet, OLD.idDphDalUcet, 
            -1, OLD.idStred, -OLD.sumdph, 0.0); 
    else  
        execute createOrUpdateObraty(OLD.Ucetni, false, OLD.DatUcto, OLD.idDphMdUcet, OLD.idDphDalUcet, 
            -1, OLD.idStred, -OLD.sumdphdanove, 0.0); 
    end if;
  end if;

  -- odpisy majetku jsou vedeny vzdy v mistni mene, proto za menu
  -- dosazujeme null a za sumZklMen 0.0
  execute createOrUpdateObraty(NEW.Ucetni, false, NEW.DatUcto, NEW.idZklMdUcet, NEW.idZklDalUcet,
       -1, NEW.idStred, NEW.sumZKl, 0.0);

  -- pokud je udalost castecne nebo uplne vyrazeni musime zauctovat jeji zustatkovou cenu
  if (NEW.typUdalostiK in ('typUdal.uplVyr', 'typUdal.casVyr')) then
    -- účetní částka má při uvádění do obratů přednost
    if ((select nahrucetodpk from mmajetek as M where M.idmajetku = NEW.idmajetku) = 'nahrUcet.ne') then--(NEW.sumdph <> 0.0) then 
        execute createOrUpdateObraty(NEW.Ucetni, false, NEW.DatUcto, NEW.idDphMdUcet, NEW.idDphDalUcet, 
            -1, NEW.idStred, NEW.sumdphdanove, 0.0); 
    else 
        execute createOrUpdateObraty(NEW.Ucetni, false, NEW.DatUcto, NEW.idDphMdUcet, NEW.idDphDalUcet, 
            -1, NEW.idStred, NEW.sumdph, 0.0); 
    end if;
  end if;

  return NEW;
end;
$$ LANGUAGE plpgsql;
ALTER FUNCTION public.mudalostiupdateucto() OWNER TO %firmaRole%;

CREATE OR REPLACE FUNCTION mudalostideleteucto() RETURNS trigger
    AS $$
begin
  -- odpisy majetku jsou vedeny vzdy v mistni mene, proto za menu
  -- dosazujeme -1 a za sumZklMen 0.0
  execute createOrUpdateObraty(OLD.Ucetni, false, OLD.DatUcto, OLD.idZklMdUcet, OLD.idZklDalUcet,
       -1, OLD.idStred, -OLD.sumZkl, 0.0);

  -- pokud byla drive udalost castecne nebo uplne vyrazeni musime ji odebrat z obratu
  if (OLD.typUdalostiK in ('typUdal.uplVyr', 'typUdal.casVyr')) then
    -- účetní částka má při uvádění do obratů přednost
    if ((select nahrucetodpk from mmajetek as M where M.idmajetku = OLD.idmajetku) = 'nahrUcet.ne') then 
        execute createOrUpdateObraty(OLD.Ucetni, false, OLD.DatUcto, OLD.idDphMdUcet, OLD.idDphDalUcet, 
            -1, OLD.idStred, -OLD.sumdphdanove, 0.0); 
    else 
        execute createOrUpdateObraty(OLD.Ucetni, false, OLD.DatUcto, OLD.idDphMdUcet, OLD.idDphDalUcet, 
            -1, OLD.idStred, -OLD.sumdph, 0.0); 
    end if;
  end if;

  return OLD;
end;
$$ LANGUAGE plpgsql;
ALTER FUNCTION public.mudalostideleteucto() OWNER TO %firmaRole%;


CREATE OR REPLACE FUNCTION prepoctiobraty(iducetobd integer) RETURNS void
    AS $$ 
declare 
  _platiOd date; 
  _platiDo date; 
begin 
  select platioddata, platidodata into _platiOd, _platiDo from nucetobdobi where iducetobdobi = idUcetObd; 
 
  if (_platiOd is not null and _platiDo is not null) then 
    delete from uobraty where obdobi > 0 and (iducetobdobi = idUcetObd or 
      (obdobi < 99 and date(rok || '-' || obdobi || '-02') between _platiOd and _platiDo)); 
     
    -- zauctovani polozek faktur 
     perform createOrUpdateObraty(P.ucetni, P.storno, P.datucto, P.idZklMdUcet, P.idZklDalUcet, P.idMeny, P.idStred, 
         P.idCinnost, P.sumZkl, P.sumZklMen) from dpolfak as P  
         where P.ucetni = true and P.storno = false and P.datUcto between _platiOd and _platiDo; 
      
     perform createOrUpdateObraty(P.ucetni, P.storno, P.datucto, P.idDphMdUcet, P.idDphDalUcet, P.idMeny, P.idStred, 
         P.idCinnost, P.sumDph, P.sumDphMen) from dpolfak as P  
         where P.ucetni = true and P.storno = false and P.datUcto between _platiOd and _platiDo; 
        
     -- zauctovani polozek internich dokladu 
     perform createOrUpdateObraty(P.ucetni, P.storno, P.datucto, P.idZklMdUcet, P.idZklDalUcet, P.idMeny, P.idStred, 
         P.idCinnost, P.sumZkl, P.sumZklMen) from dpolint as P  
         where P.ucetni = true and P.storno = false and P.datUcto between _platiOd and _platiDo; 
        
     perform createOrUpdateObraty(P.ucetni, P.storno, P.datucto, P.idDphMdUcet, P.idDphDalUcet, P.idMeny, P.idStred, 
         P.idCinnost, P.sumDph, P.sumDphMen) from dpolint as P  
         where P.ucetni = true and P.storno = false and P.datUcto between _platiOd and _platiDo; 
                                 
     -- zauctovani polozek skladovych dokladu                      
     perform createOrUpdateObraty(P.ucetni, P.storno, P.datucto, P.idZklMdUcet, P.idZklDalUcet, P.idMeny, P.idStred, 
         P.idCinnost, P.sumZkl, P.sumZklMen) from dpolsklad as P  
         where P.ucetni = true and P.storno = false and P.datUcto between _platiOd and _platiDo; 
        
     perform createOrUpdateObraty(P.ucetni, P.storno, P.datucto, P.idDphMdUcet, P.idDphDalUcet, P.idMeny, P.idStred, 
         P.idCinnost, P.sumDph, P.sumDphMen) from dpolsklad as P  
         where P.ucetni = true and P.storno = false and P.datUcto between _platiOd and _platiDo; 
     
     -- danove naklady 
     perform createOrUpdateObraty(P.ucetni, false, P.datUcto, P.idZklMdUcet, P.idZklDalUcet, L.idMeny, P.idStred,  
          P.sumZkl, P.sumZklMen) from mdannakl as P left outer join mleasing as L on L.idleasing = P.idleasing  
          where P.ucetni = true and P.datUcto between _platiOd and _platiDo; 
           
     -- danove odpisy 
     -- odpisy majetku jsou vedeny vzdy v mistni mene, proto za menu dosazujeme -1 a za sumZklMen 0.0 
     perform createOrUpdateObraty(P.ucetni, false, P.datUcto, P.idZklMdUcet, P.idZklDalUcet, -1, P.idStred,  
          P.sumZkl, 0.0) from mdanodpisy as P 
          where P.ucetni = true and P.datUcto between _platiOd and _platiDo; 
      
     -- splatkovy kalendar 
     perform createOrUpdateObraty(P.ucetni, false, P.datUcto, P.idZklMdUcet, P.idZklDalUcet, L.idMeny, P.idStred,  
          P.sumZkl, P.sumZklMen) from msplatkal as P left outer join mleasing as L on L.idleasing = P.idleasing  
          where P.ucetni = true and P.datUcto between _platiOd and _platiDo; 
           
     perform createOrUpdateObraty(P.ucetni, false, P.datUcto, P.idDphMdUcet, P.idDphDalUcet, L.idMeny, P.idStred,  
          P.sumDph, P.sumDphMen) from msplatkal as P left outer join mleasing as L on L.idleasing = P.idleasing  
          where P.ucetni = true and P.datUcto between _platiOd and _platiDo; 
      
     -- ucetni odpisy 
     -- odpisy majetku jsou vedeny vzdy v mistni mene, proto za menu dosazujeme -1 a za sumZklMen 0.0 
     perform createOrUpdateObraty(P.ucetni, false, P.datUcto, P.idZklMdUcet, P.idZklDalUcet, -1, P.idStred,  
          P.sumZkl, 0.0) from mucetodpisy as P 
          where P.ucetni = true and P.datUcto between _platiOd and _platiDo; 
      
     -- udalosti 
     -- odpisy majetku jsou vedeny vzdy v mistni mene, proto za menu dosazujeme -1 a za sumZklMen 0.0 
     perform createOrUpdateObraty(P.ucetni, false, P.datUcto, P.idZklMdUcet, P.idZklDalUcet, -1, P.idStred,  
          P.sumZkl, 0.0) from mudalosti as P 
          where P.ucetni = true and P.datUcto between _platiOd and _platiDo; 
        
          --udalosti
     --doplneno Steklem 14.4.2016 - prepocet stavu uctu ignoroval nove pole "sumdphdanove"
     perform createOrUpdateObraty(P.ucetni, false, P.datUcto, P.idDphMdUcet, P.idDphDalUcet, -1, P.idStred,   
          case when 
         (select nahrucetodpk from mmajetek as M where M.idmajetku = P.idmajetku ) = 'nahrUcet.ne' 
          then  P.sumdphdanove else P.sumDph end , 0.0) from mudalosti as P  
          where P.ucetni = true and P.datUcto between _platiOd and _platiDo; 
        
  end if; 
end;  
$$
    LANGUAGE plpgsql;


ALTER FUNCTION public.prepoctiobraty(iducetobd integer) OWNER TO %firmaRole%;


-- zvyseni verze databaze
insert into dbversion(iddbversion, dbversion, needreimport, appversion, upgradetime)
values (nextval('seq_dbversion'), 1004, 't', %appVersion%, clock_timestamp());