----------------------------------------------------------------
-- Bug 45432 - Přijatá objednávka - rezervace - vydaná faktura 
----------------------------------------------------------------

CREATE OR REPLACE FUNCTION daapolskladinsertmnozmj()
  RETURNS trigger AS
$BODY$ 
declare 
  _stavSkladu numeric(19,6);
  _rezervovano numeric(19,6);
  _firmaId integer;
  _pozadavkyMj numeric(19,6);
  _stavMjSPozadavky numeric(19,6);
  _typPohybuK character varying;
  _sklGenerovatPozadavky boolean;
begin
  -- tento trigger tu musi zustat protoze jinak na obycejnych vydejkach a prijemkach nebude vyplneno mnozMjPlan
  select sklgenerovatpozadavky into _sklGenerovatPozadavky from nNastav where NEW.datVyst >= PlatiOdData order by PlatiOdData desc limit 1; 
  if (_sklGenerovatPozadavky is null) then 
        select sklgenerovatpozadavky into _sklGenerovatPozadavky from nNastav where PlatiOdData is null limit 1;  
  end if; 

  NEW.mnozMjPlan = NEW.mnozMj;

  if (_sklGenerovatPozadavky) then
    select typPohybuK, idFirmy into _typPohybuK, _firmaId from ddoklsklad where iddoklsklad = NEW.idDoklSklad;
    select stavMj, pozadavkyMj, stavMjSPozadavky into _stavSkladu, _pozadavkyMj, _stavMjSPozadavky from skarty where idKarty = NEW.idKarty;
    select dGetRezervovano(NEW.idcenik, NEW.idbspskl, _firmaId) into _rezervovano;
    
    if(_typPohybuK = 'typPohybu.vydej' and NEW.mnozMj > (_stavSkladu - _rezervovano)) then 
      update skarty set 
          pozadavkyMj = _pozadavkyMj + NEW.mnozMj, 
          stavMjSPozadavky = _stavMjSPozadavky - NEW.mnozMj
        where idkarty = NEW.idKarty; 
      NEW.mnozMj = 0.0; 
    end if; 
  end if;
  
  return NEW;
end; 
$BODY$
  LANGUAGE 'plpgsql' VOLATILE
  COST 100;
ALTER FUNCTION daapolskladinsertmnozmj() OWNER TO %firmaRole%;

CREATE OR REPLACE FUNCTION skladrealizujpozadavky(i_idpolsklad integer, i_limit integer)
  RETURNS integer AS
$BODY$ 
declare
  _polSklad record;
  _polSkladDoklSklad record;
  _stavSkladu numeric(19,6);
  _rezervovano numeric(19,6);
  _firmaId integer; 
  _cenaMj numeric(19,6);
  _pocetPozadavku integer;
  _pocet integer;
  _resultId integer;
  _pocetPolSklad integer;
begin
  _pocet = 0;
  _resultId = null;
  for _polSklad in (
    select p.*, d.typPohybuK, d.stavSkladK from dpolsklad p join ddoklsklad d on p.idDoklSklad = d.idDoklSklad
    where typPohybuK = 'typPohybu.vydej' and mnozMj = 0 and mnozMjPlan != 0 and p.storno = false and idPolSklad > i_idpolsklad
    order by idPolSklad) loop
      /*behem aktualizace pozadavku na vydej muze dojit ke smazani nektere z polozek
      a diky tomu se pak muze rozhodit suma pozadavku na skladove karte*/
      select 1 into _pocetPolSklad from dPolSklad ps where ps.idPolSklad = _polSklad.idPolSklad limit 1;
      continue when _pocetPolSklad is null;
      
      _pocet = _pocet + 1;
      exit when _pocet > i_limit;
      _resultId = _polSklad.idPolSklad;
      select idFirmy into _firmaId from ddoklsklad where iddoklsklad = _polSklad.iddoklsklad;
      select dGetRezervovano(_polSklad.idcenik, _polSklad.idbspskl, _firmaId) into _rezervovano;
      --select dgetstavnasklade(_polSklad.idkarty, _polSklad.datSklad, /*_polSklad.idpolsklad*/ -1, _polSklad.sarze, _polSklad.expirace, _polSklad.typPohybuK) into _stavSkladu;
      select stavMj, prumCenaTuz into _stavSkladu, _cenaMj from skarty where idkarty = _polSklad.idKarty;
      
      if((_stavSkladu - _rezervovano) >= _polSklad.mnozMjPlan) then
        update skarty set
          pozadavkyMj = pozadavkyMj - _polSklad.mnozMjPlan,
          stavMjSPozadavky = stavMjSPozadavky + _polSklad.mnozMjPlan
        where idkarty = _polSklad.idKarty;
        
        update dpolsklad set mnozMj = mnozMjPlan, cenaMj = _cenaMj, sumZkl = mnozMjPlan * _cenaMj, datSklad = LOCALTIMESTAMP, lastupdate = LOCALTIMESTAMP
          where idpolsklad = _polSklad.idPolSklad;
        perform sprepoctisumydokladu(_polSklad.iddoklsklad);
        
        select count(*) into _pocetPozadavku from dpolSklad where idDoklSklad = _polSklad.idDoklSklad and mnozMj = 0.0 and mnozMjPlan != 0.0;
        if(_pocetPozadavku = 0) then
          if(compareVarchar(_polSklad.stavSkladK, 'stavSklad.vygenFaktNeuplny')) then
            update ddoklsklad set stavSkladK = 'stavSklad.vygenFakt' where idDoklSklad = _polSklad.idDoklSklad;
          end if;
          if(compareVarchar(_polSklad.stavSkladK, 'stavSklad.vygenPoklNeuplny')) then
            update ddoklsklad set stavSkladK = 'stavSklad.vygenPokl' where idDoklSklad = _polSklad.idDoklSklad;
          end if;
        end if;
      end if;
  end loop;
  return _resultId;
end;    
$BODY$
  LANGUAGE 'plpgsql' VOLATILE
  COST 100;
ALTER FUNCTION skladrealizujpozadavky(integer, integer) OWNER TO %firmaRole%;

  -- zvyseni verze databaze
insert into dbversion(iddbversion, dbversion, needreimport, appversion, upgradetime)
values (nextval('seq_dbversion'), 1018, 't', %appVersion%, clock_timestamp());