-----------------------------------------------------------------------------
-- Bug 45719 - Založení faktury s položkami a adresářem trvá 7s (profizoo) -- 
-----------------------------------------------------------------------------

CREATE OR REPLACE FUNCTION daapolskladinsertmnozmj()
  RETURNS trigger AS
$BODY$ 
declare 
  _stavSkladu numeric(19,6);
  _rezervovano numeric(19,6);
  _firmaId integer;
  _pozadavkyMj numeric(19,6);
  _stavMjSPozadavky numeric(19,6);
  _typPohybuK character varying;
  _sklGenerovatPozadavky boolean;
begin
  -- tento trigger tu musi zustat protoze jinak na obycejnych vydejkach a prijemkach nebude vyplneno mnozMjPlan
  select sklgenerovatpozadavky into _sklGenerovatPozadavky from nNastav where NEW.datVyst >= PlatiOdData order by PlatiOdData desc limit 1; 
  if (_sklGenerovatPozadavky is null) then 
        select sklgenerovatpozadavky into _sklGenerovatPozadavky from nNastav where PlatiOdData is null limit 1;  
  end if; 

  NEW.mnozMjPlan = NEW.mnozMj;
  -- realny cas, now() vraci cas startu transakce (pro vice polozek pak byl stejny cas)
  NEW.datSklad = timeofday();

  if (_sklGenerovatPozadavky) then
    select typPohybuK, idFirmy into _typPohybuK, _firmaId from ddoklsklad where iddoklsklad = NEW.idDoklSklad;
    select stavMj, pozadavkyMj, stavMjSPozadavky into _stavSkladu, _pozadavkyMj, _stavMjSPozadavky from skarty where idKarty = NEW.idKarty;
    select dGetRezervovano(NEW.idcenik, NEW.idbspskl, _firmaId) into _rezervovano;
    
    if(_typPohybuK = 'typPohybu.vydej' and NEW.mnozMj > (_stavSkladu - _rezervovano)) then 
      update skarty set 
          pozadavkyMj = _pozadavkyMj + NEW.mnozMj, 
          stavMjSPozadavky = _stavMjSPozadavky - NEW.mnozMj 
        where idkarty = NEW.idKarty; 
      NEW.mnozMj = 0.0; 
    end if;
  end if;
  
  return NEW;
end; 
$BODY$
  LANGUAGE 'plpgsql' VOLATILE
  COST 100;
ALTER FUNCTION daapolskladinsertmnozmj() OWNER TO %firmaRole%;

CREATE OR REPLACE FUNCTION dpolskladinsert()
  RETURNS trigger AS
$BODY$
declare
  _rozsirenaPolozka record;
  _typPohybuk character varying;
  _mazatTmpTable boolean;
  _idPolSklad integer;
  _probihaPrepocetSkladu character varying;
  _polSklad record; 
  _pocetMenenychVydejek integer;
  _odsunutaVydejka record;
  _pomocnaVydejka record;
begin
  if(NEW.storno = true) then
    return NEW;
  end if; 
  
  select getglotxvariable('probihaPrepocetSkladu') into _probihaPrepocetSkladu;
  if(_probihaPrepocetSkladu = 'true') then
  else
    -- tento select zjisti kolik je vydejek, ktere by se meli prepocitavat
    select count(idpolsklad) into _pocetMenenychVydejek from dpolsklad ps join ddoklsklad ds 
      on ps.datsklad > NEW.datSklad and ps.cenyRucne = false and ps.iddoklsklad = ds.iddoklsklad and ds.typpohybuk = 'typPohybu.vydej';
    if(_pocetMenenychVydejek = 0 and NEW.cenyRucne = true) then
      execute sprepoctiskladkartu(NEW.mnozMj, NEW.cenaMj, NEW.sumZkl, NEW.datVyst, NEW.idKarty, NEW.idDoklSklad, NEW.idBspSkl, 1); 
      return NEW;
    end if;

    -- v pripade ze neprobiha prepocet skladu
    -- generuji rozsirene polozky
    select skladGenerujRozsirenePolozky(NEW.datSklad, NEW.idPolSklad, NEW.idKarty) into _mazatTmpTable;

    execute putglotxvariable('zurnalTxName', 'prepocetSkladu');

    for _rozsirenaPolozka in (select * from tmp_RozsirenePolozky order by datum, skupina, iddoklskladMaster, typpohybuk desc, idpolsklad) loop
      if(_rozsirenaPolozka.idPolSklad <> NEW.idPolSklad) then
        if(_rozsirenaPolozka.cenyRucne = true) then
          -- pokud maji byt ceny zadany rucne a nemaji se nikterak menit tak jen zmenime skladovou kartu a jedeme dal
          if(_rozsirenaPolozka.storno = false) then 
            select * into _polSklad from dpolsklad where idpolsklad = _rozsirenaPolozka.idPolSklad;
            execute sprepoctiskladkartu(_polSklad.mnozMj, _polSklad.cenaMj, _polSklad.sumZkl, _polSklad.datVyst, _polSklad.idKarty, _polSklad.idDoklSklad, _polSklad.idBspSkl, -1); 
          end if;
        else
          execute skladOdpoctiRozsirenouPolozku(_rozsirenaPolozka.idPolSklad);
        end if;
      end if;
    end loop;

    for _rozsirenaPolozka in (select * from tmp_RozsirenePolozky order by datum, skupina, iddoklskladMaster, typpohybuk desc, idpolsklad) loop
      if(_rozsirenaPolozka.cenyRucne = true) then
        -- pokud maji byt ceny zadany rucne a nemaji se nikterak menit tak jen zmenime skladovou kartu a jedeme dal
        if(_rozsirenaPolozka.storno = false) then 
          select * into _polSklad from dpolsklad where idpolsklad = _rozsirenaPolozka.idPolSklad;
          execute sprepoctiskladkartu(_polSklad.mnozMj, _polSklad.cenaMj, _polSklad.sumZkl, _polSklad.datVyst, _polSklad.idKarty, _polSklad.idDoklSklad, _polSklad.idBspSkl, 1); 
        end if;
      else
        execute skladZapoctiRozsirenouPolozku(_rozsirenaPolozka.idPolSklad);
      end if;
    end loop;

    if(_mazatTmpTable) then
      drop table tmp_RozsirenePolozky;
      drop table tmp_SkladoveKarty;
      execute removeglotxvariable('zurnalTxName');
    end if;
  end if;

  select typpohybuk into _typPohybuk from ddoklsklad where iddoklsklad = NEW.iddoklsklad;
  --pokud výdej, tak fifuju vazby
  if (_typPohybuk = 'typPohybu.vydej' or (_typPohybuK = 'typPohybu.prijem' and NEW.mnozMj < 0.0)) then
     execute sfifoDobropis(NEW.idpoldobropis, NEW.idPolSklad, NEW.idCenik, NEW.idBspSkl, 
              NEW.idKarty, NEW.mnozmj, NEW.cenyrucne);
    
    --execute sPrepoctiSumyDokladu(NEW.idDoklSklad);
  end if;

  if(_probihaPrepocetSkladu = 'true') then
    if(NEW.idPolSklad < 0) then
      _idPolSklad = -NEW.idPolSklad;
    else
      _idPolSklad = NEW.idPolSklad;
    end if;

    execute skladinsertrozsirenoupolozku(_idPolSklad);
  end if;

  execute sprepoctisumydokladu(NEW.idDoklSklad);

  return NEW;
end;
$BODY$
  LANGUAGE 'plpgsql' VOLATILE
  COST 100;
ALTER FUNCTION dpolskladinsert() OWNER TO %firmaRole%;

CREATE OR REPLACE FUNCTION ccenikupdatedodavatel()
  RETURNS trigger AS $$
declare
  dodOld integer;
  _iddod integer;
begin
  dodOld = null;
  if (tg_op = 'UPDATE') then
    dodOld = OLD.iddodavatel;
  end if;
  
  if not compareid(NEW.iddodavatel, dodOld) then
    if (getglotxvariable('dodavatelDisableTrigger') is not null) then 
      return NEW; 
    end if; 
    perform putglotxvariable('dodavatelDisableTrigger', '1'); 

    delete from cdodavatele where automgener = true and idcenik = NEW.idcenik;  -- smazeme stare automaticky generovane dodavatele
    update cdodavatele set primarni = false where idcenik = NEW.idcenik;        -- ostatnim nastavime ze nejsou primarni
    
    if (NEW.iddodavatel is not null) then                                       -- pokud je zvolen nejaky novy dodavatel
      select iddodavatel into _iddod from cdodavatele where idfirmy = NEW.iddodavatel and idcenik = NEW.idcenik;
      
      if (_iddod is null) then
        insert into cdodavatele (primarni, automgener, idcenik, idfirmy)
          values (true, true, NEW.idcenik, NEW.iddodavatel);                    -- neexistuje tak ho vytvorime
      else
        update cdodavatele set primarni = true where iddodavatel = _iddod;      -- zvolenemu existujicimu nastavime primarni
      end if;
    end if;
  
    perform removeglotxvariable('dodavatelDisableTrigger');
  end if;
  
  return NEW;
end;
$$ LANGUAGE plpgsql;
ALTER FUNCTION public.ccenikupdatedodavatel() OWNER TO %firmaRole%;

-- zvyseni verze databaze
insert into dbversion(iddbversion, dbversion, needreimport, appversion, upgradetime)
  values (nextval('seq_dbversion'), 1036, 't', %appVersion%, clock_timestamp());