-------------------------------------------------------------------------------------
-- Bug 14983 – [WinStrom] Některá zmigrovaná data se tváří jako nezaúčtovaná
-------------------------------------------------------------------------------------

CREATE OR REPLACE FUNCTION isDokladZuctovan(idDokl integer, doklPostfix varchar(100))
  RETURNS boolean AS $$
declare
  _modul varchar(100);
  _podvojucto boolean;
  _ucetni boolean;
  _storno boolean;
  _bezpol boolean;
  _datUcto date;
  _doklOk integer;
  _polozek integer;
  _polOk integer;
  idDoklColumn varchar(100);
  doklTable varchar(100);
  polTable varchar(100);
begin
  idDoklColumn := 'idDokl' || doklPostfix;
  doklTable := 'dDokl' || doklPostfix;
  polTable := 'dPol' || doklPostfix;

  execute 'select modul, datUcto, ucetni, bezpolozek, storno from ' || doklTable || ' where ' || idDoklColumn || ' = ' || idDokl
   into _modul, _datUcto, _ucetni, _bezpol, _storno;
  execute 'select count(*) from ' || polTable || ' where ' || idDoklColumn || ' = ' || idDokl into _polozek;
  _doklOk := 0;

  if (_ucetni = true and _storno = false) then
    -- zjistime zda firma vede ucetnictvi
    select podvojucto into _podvojucto from nNastav where PlatiOdData <= _datUcto order by PlatiOdData desc limit 1;
    if (_podvojucto is null) then
      select podvojucto into _podvojucto from nNastav where PlatiOdData is null limit 1;
    end if;

    if (_podvojucto = true) then
      -- ucetnictvi
      if (_bezpol = true or _polozek = 0) then
	if (_modul = 'INT') then
          execute
            'select count(*) from ' || doklTable || ' where ' || idDoklColumn || ' = ' || idDokl || ' and
              idstred is not null and datucto is not null and
              ((sumosv = 0 and sumzklsniz = 0 and sumzklzakl = 0) or idprimucet is not null) and
              ((sumosv = 0 and sumzklsniz = 0 and sumzklzakl = 0) or idprotiucet is not null) and
              ((sumdphsniz = 0 and sumdphzakl = 0) or (iddphsnizucet is not null and iddphzaklucet is not null))'
            into _doklOk;
	else
          execute
            'select count(*) from ' || doklTable || ' where ' || idDoklColumn || ' = ' || idDokl || ' and
              idstred is not null and datucto is not null and
              ((sumcelkem = 0) or idprimucet is not null) and
              ((sumosv = 0 and sumzklsniz = 0 and sumzklzakl = 0) or idprotiucet is not null) and
              (sumdphsniz = 0 or iddphsnizucet is not null) and
              (sumdphzakl = 0 or iddphzaklucet is not null)'
            into _doklOk;
	end if;
      else
        execute
          'select count(*) from ' || polTable || ' where ' || idDoklColumn || ' = ' || idDokl || ' and
            (ucetni = false or
            (idstred is not null and datucto is not null and
              (sumZkl = 0 or (idzkldalucet is not null and idzklmducet is not null)) and
              (modul = ''SKL'' or typszbdphk is null or typszbdphk = ''typSzbDph.dphOsv'' or sumDph = 0 or (
                iddphmducet is not null and iddphdalucet is not null))))'
          into _polOk;
        if (_polOk = _polozek) then
          _doklOk := 1;
        end if;
      end if;
    else
      -- danova evidence
      if (doklTable = 'dDoklFak' or doklTable = 'dDoklInt') then
        if (_bezpol = true or _polozek = 0) then
          execute 'select count(*) from ' || doklTable || ' where ' || idDoklColumn || ' = ' || idDokl || '
            and datucto is not null and danevidk is not null' into _doklOk;
        else
          execute 'select count(*) from ' || polTable || ' where ' || idDoklColumn || ' = ' || idDokl || '
            and (ucetni = false or (datucto is not null and danevidk is not null))' into _polOk;
          if (_polOk = _polozek) then
            _doklOk := 1;
          end if;
        end if;
      end if;
    end if;
  end if;

  return (_doklOk = 1);
end;
$$ LANGUAGE plpgsql;
ALTER FUNCTION public.isDokladZuctovan(integer, varchar(100)) OWNER TO %firmaRole%;


-- aktualizace stavu zaúčtování i při insertech
DROP TRIGGER ddoklfakupdatezauctovano ON ddoklfak;
DROP TRIGGER ddoklintupdatezauctovano ON ddoklint;
DROP TRIGGER ddoklskladupdatezauctovano ON ddoklsklad;
DROP TRIGGER dpolfakupdatezauctovano ON dpolfak;
DROP TRIGGER dpolintupdatezauctovano ON dpolint;
DROP TRIGGER dpolskladupdatezauctovano ON dpolsklad;


CREATE TRIGGER ddoklfakupdatezauctovano AFTER INSERT OR UPDATE ON ddoklfak
  FOR EACH ROW EXECUTE PROCEDURE ddoklfakupdatezauctovano();
  
CREATE TRIGGER ddoklintupdatezauctovano AFTER INSERT OR UPDATE ON ddoklint
  FOR EACH ROW EXECUTE PROCEDURE ddoklintupdatezauctovano();

CREATE TRIGGER ddoklskladupdatezauctovano AFTER INSERT OR UPDATE ON ddoklsklad
  FOR EACH ROW EXECUTE PROCEDURE ddoklskladupdatezauctovano();


CREATE TRIGGER dpolfakupdatezauctovano AFTER INSERT OR UPDATE ON dpolfak
  FOR EACH ROW EXECUTE PROCEDURE dpolfakupdatezauctovano();

CREATE TRIGGER dpolintupdatezauctovano AFTER INSERT OR UPDATE ON dpolint
  FOR EACH ROW EXECUTE PROCEDURE dpolintupdatezauctovano();

CREATE TRIGGER dpolskladupdatezauctovano AFTER INSERT OR UPDATE ON dpolsklad
  FOR EACH ROW EXECUTE PROCEDURE dpolskladupdatezauctovano();


-- oprava stávajících dokladů
update ddoklfak set zuctovano = isDokladZuctovan(idDoklFak, 'Fak') where zuctovano <> isDokladZuctovan(idDoklFak, 'Fak');  
update ddoklint set zuctovano = isDokladZuctovan(idDoklInt, 'Int') where zuctovano <> isDokladZuctovan(idDoklInt, 'Int');
update ddoklsklad set zuctovano = isDokladZuctovan(idDoklSklad, 'Sklad') where zuctovano <> isDokladZuctovan(idDoklSklad, 'Sklad');

-- doplneni sloupce uuid do dokladu
alter table ddoklfak add column uuid character varying(50);
alter table ddoklint add column uuid character varying(50);
alter table ddoklsklad add column uuid character varying(50);
alter table ddoklobch add column uuid character varying(50);

-- zvyseni verze databaze
insert into dbversion(iddbversion, dbversion, needreimport)
  values (nextval('seq_dbversion'), 108, 't');