--------------------------------------------------------------------------------
-- Bug 47071 - Změna přílohy reportu nebo razítka v nastavení neovlivňuje tisky (problém kešování)
--------------------------------------------------------------------------------

alter table dpolsklad alter column datSklad set default clock_timestamp();

CREATE OR REPLACE FUNCTION daapolskladinsertmnozmj()
  RETURNS trigger AS
$BODY$ 
declare 
  _stavSkladu numeric(19,6);
  _rezervovano numeric(19,6);
  _firmaId integer;
  _pozadavkyMj numeric(19,6);
  _stavMjSPozadavky numeric(19,6);
  _typPohybuK character varying;
  _sklGenerovatPozadavky boolean;
begin
  -- tento trigger tu musi zustat protoze jinak na obycejnych vydejkach a prijemkach nebude vyplneno mnozMjPlan
  select sklgenerovatpozadavky into _sklGenerovatPozadavky from nNastav where NEW.datVyst >= PlatiOdData order by PlatiOdData desc limit 1; 
  if (_sklGenerovatPozadavky is null) then 
        select sklgenerovatpozadavky into _sklGenerovatPozadavky from nNastav where PlatiOdData is null limit 1;  
  end if; 

  NEW.mnozMjPlan = NEW.mnozMj;
  -- realny cas, now() vraci cas startu transakce (pro vice polozek pak byl stejny cas)
  -- pokud neni vyplneno resi defaultni hodnota na datSklad
  --NEW.datSklad = timeofday();

  if (_sklGenerovatPozadavky) then
    select typPohybuK, idFirmy into _typPohybuK, _firmaId from ddoklsklad where iddoklsklad = NEW.idDoklSklad;
    select stavMj, pozadavkyMj, stavMjSPozadavky into _stavSkladu, _pozadavkyMj, _stavMjSPozadavky from skarty where idKarty = NEW.idKarty;
    select dGetRezervovano(NEW.idcenik, NEW.idbspskl, _firmaId) into _rezervovano;
    
    if(_typPohybuK = 'typPohybu.vydej' and NEW.mnozMj > (_stavSkladu - _rezervovano)) then 
      update skarty set 
          pozadavkyMj = _pozadavkyMj + NEW.mnozMj, 
          stavMjSPozadavky = _stavMjSPozadavky - NEW.mnozMj 
        where idkarty = NEW.idKarty; 
      NEW.mnozMj = 0.0; 
    end if;
  end if;
  
  return NEW;
end; 
$BODY$
  LANGUAGE 'plpgsql' VOLATILE
  COST 100;
ALTER FUNCTION daapolskladinsertmnozmj() OWNER TO %firmaRole%;

CREATE OR REPLACE FUNCTION createPolSkladSada(_idPolDokl integer, _modul character varying)
  RETURNS void AS
$BODY$ 
declare
  _polDokl record;
  _doklad record;
  _polozkaSady record;
  _skladKarta record;
  _cenik record;
  _sklad record;
  _idDoklSklad integer;
  _idDoklad integer;
  _idMeny integer;
  _idUcetObdobi integer;
  _idPolSklad integer;
  _idZklMdUcet integer;
  _idZklDalUcet integer;
  _i integer;
  _cisRad integer;
  _typDoklK character varying;
  _mnozMj numeric(19,6);
  _sumZkl numeric(19,6);
  _sumZklMen numeric(19,6);
  _cenaMj numeric(19,6);
  _kurz numeric(19,6);
  _isCiziMena boolean;
  _nasobic integer;
  _idSkladu integer;
  _kopZklMdUcet boolean;
  _kopZklDalUcet boolean;
  _idTypUcOp integer;
  _kopStred boolean;
  _kopCinnost boolean;
  _kopZakazku boolean;
  _idStredDoklSklad integer;
  _idCinnostDoklSklad integer;
  _idZakazkyDoklSklad integer;
  _typPohybuSkladKVydej character varying;
  _skupZboz record;
  _datUcto date; 
  _datPosl date;
begin

  if(_modul like 'FAV' or _modul like 'FAP') then 
    select * into _polDokl from dPolFak where idPolFak = _idPolDokl; 
    _idDoklad = _polDokl.idDoklFak;
    select * into _doklad from dDoklFak where idDoklFak = _idDoklad;
    select typDoklK into _typDoklK from dTypDokl where idTypDokl = (select idTypDokl from dDoklFak where idDoklFak = _polDokl.idDoklFak); 
  else 
    select * into _polDokl from dPolInt where idPolInt = _idPolDokl; 
    _idDoklad = _polDokl.idDoklInt;
    select * into _doklad from dDoklInt where idDoklInt = _idDoklad;
    _typDoklK = null; -- pro pokladnu dobropisy nejsou 
  end if;
 
  if(_polDokl.datVyst >= '2014-01-01') then
    _datUcto = _polDokl.datUcto;
    if(_datUcto is null) then
      _datUcto = _polDokl.datVyst;
    end if;
  else
    _datUcto = _polDokl.datVyst;
  end if;


  select * into _sklad from dBsp where idBsp = _polDokl.idBspSkl;

  _i = 0;

  select findIdUcetniObdobiKDatu(_datUcto) into _iducetobdobi;

  FOR _polozkaSady IN
    select * from cSady where idCenikSada = _polDokl.idCenik
  LOOP
    select * into _cenik from cCenik where idCenik = _polozkaSady.idCenik;

    if(_cenik.skladove) then
      _i = _i + 1;
      _cisRad = 100 * _polDokl.cisRad + _i;

      _idSkladu = _polDokl.idBspSkl;

      select * into _skladKarta from skarty 
        where idSkladu = _idSkladu and idCenik = _polozkaSady.idCenik and idUcetObdobi = _idUcetObdobi;

      -- pokud neexistuje skladova karta pokusim se ji najit pomoci mapovani skladu
      if(_skladKarta is null) then
        select getPrefIdSkladu(_polDokl.idStred, _cenik.idSkupZboz, _cenik.idCenik) into _idSkladu;
        select * into _skladKarta from skarty
          where idSkladu = _idSkladu and idCenik = _cenik.idCenik and idUcetObdobi = _idUcetObdobi;
        if(_skladKarta is null) then
          -- pokud ani podle mapovani skladu se skladova karta nenajde tak vyhazuju vyjimku
          raise exception '0007:Skladová karta neexistuje pro ceník % a sklad % a nebo je špatně nastaveno mapovaní skladu.', _cenik.nazev, _sklad.nazev;
        end if;
      else
        _idSkladu = _skladKarta.idSkladu;
      end if;

      select findparovanydoklsklad(_iddoklad, _modul, 'typVazbyDokl.hlavaSkl', true, _idSkladu) into _idDoklSklad;  
   
      if(_idDoklSklad is null) then  
        select createDoklSklad(_idPolDokl, _idDoklad, _modul) into _idDoklSklad;  
        if(_idDoklSklad is null) then 
          return; 
        end if; 
      end if; 
 
      select getkurzfromdoklad(_idDoklSklad, _polDokl.datVyst) into _kurz; 
      select isDoklSkladMenaCizi(_idDoklSklad, _polDokl.datVyst) into _isCiziMena;  
 
      select idMeny, idTypUcOp, idstred, idcinnost, idZakazky 
        into _idMeny, _idTypUcOp, _idStredDoklSklad, _idCinnostDoklSklad, _idZakazkyDoklSklad  
        from dDoklSklad where idDoklSklad = _idDoklSklad;  
 
      _nasobic = 1;
      if(_typDoklK like 'typDokladu.dobropis') then
        _nasobic = -1;
      end if;
      _mnozMj = _polozkaSady.mnozMj * _polDokl.mnozMj * _nasobic;
      _cenaMj = _skladKarta.prumCenaTuz / _kurz;
      _sumZkl = _mnozMj * _skladKarta.prumCenaTuz;
      if(_isCiziMena) then 
        _sumZklMen = _mnozMj * _cenaMj; 
      else 
        _sumZklMen = 0.0; 
      end if;
    
      -- nastaveni uctu pro polSklad vydej 
      _kopZklMdUcet = false;
      _kopZklDalUcet = false;
      if(_typDoklK like 'typDokladu.dobropis') then
        -- dal ucet je ze skupiny zbozi nebo z hlavicky pokud neni ve skupine zbozi nastaven
        select idProtiUcetSklV into _idZklDalUcet from cSkupZboz where idSkupZboz = _cenik.idSkupZboz;
        if(_idZklDalUcet is null) then
          select idProtiUcet into _idZklDalUcet from dDoklSklad where idDoklSklad = _idDoklSklad;
          --pokud je z hlavicky tak se bude kopirovat
          _kopZklDalUcet = true;
        end if;
        -- md ucet musi byt primarni ucet skladu ze ktereho se kousky sady vydavaji
        select idPrimUcet into _idZklMdUcet from dBsp where idBsp = _idSkladu;
      else
        -- nastaveni uctu pro polSklad vydej 
        select typPohybuSkladK into _typPohybuSkladKVydej from ddoklsklad where iddoklsklad = _idDoklSklad;
        select * into _skupZboz from cskupzboz where idskupzboz = (select idskupzboz from ccenik where idcenik = _polDokl.idCenik); 
    
        if(_skupzboz.idskupzboz is not null) then
          -- Úpravy dle Bug 32848 – [WinStrom] - nerespektování upřesnění typu pohybu při tvorbě výdejky
          if(_typPohybuSkladKVydej is null) then
            _idZklMdUcet = _skupZboz.idprotiucetsklv;
          -- typPohybuSklad.vydejHoly -  Holý" výdej  - idprotiucetsklholyv
          elsif(_typPohybuSkladKVydej like 'typPohybuSklad.vydejHoly') then
            _idZklMdUcet = _skupZboz.idprotiucetsklholyv;
          -- typPohybuSklad.vydejPoFak - Výdej po faktuře  - idprotiucetsklv
          elsif(_typPohybuSkladKVydej like 'typPohybuSklad.vydejPoFak') then
            _idZklMdUcet = _skupZboz.idprotiucetsklv;
          -- typPohybuSklad.vydejPolot - Výdej polotovaru - idprotiucetsklholyv
          elsif(_typPohybuSkladKVydej like 'typPohybuSklad.vydejPolot') then
            _idZklMdUcet = _skupZboz.idprotiucetsklholyv;
          -- typPohybuSklad.vydejPoObch - Výdej po objednávce  -  idprotiucetsklv
          elsif(_typPohybuSkladKVydej like 'typPohybuSklad.vydejPoObch') then
            _idZklMdUcet = _skupZboz.idprotiucetsklv;
          -- typPohybuSklad.vydejPrevod -  Výdej pro převodku  - idprotiucetsklprevv
          elsif(_typPohybuSkladKVydej like 'typPohybuSklad.vydejPrevod') then
            _idZklMdUcet = _skupZboz.idprotiucetsklprevv;
          -- typPohybuSklad.vydejProFakCenik - Výdej pro fakturaci (ceny z ceníku) - idprotiucetsklv
          elsif(_typPohybuSkladKVydej like 'typPohybuSklad.vydejProFakCenik') then
            _idZklMdUcet = _skupZboz.idprotiucetsklv;
          -- typPohybuSklad.vydejProFakRucne - Výdej pro fakturaci (ceny ručně) - idprotiucetsklv
          elsif(_typPohybuSkladKVydej like 'typPohybuSklad.vydejProFakRucne') then
            _idZklMdUcet = _skupZboz.idprotiucetsklv; 
          --typPohybuSklad.vydejVyrob - Výdej výrobku  - idprotiucetsklholyv
          elsif(_typPohybuSkladKVydej like 'typPohybuSklad.vydejVyrob') then
            _idZklMdUcet = _skupZboz.idprotiucetsklholyv; 
          end if;
        end if; 
    
        if(_idZklMdUcet is null) then
          -- pokud neni ve skupine zbozi nastaven protiucet tak vemu protiucet z hlavicky a dam kopirovat na true 
          select idProtiUcet into _idZklMdUcet from dDoklSklad where idDoklSklad = _idDoklSklad;
          --pokud je z hlavicky tak se bude kopirovat
          _kopZklMdUcet = true;
        end if;
        -- dal ucet musi byt primarni ucet skladu ze ktereho se kousky sady vydavaji
        select idPrimUcet into _idZklDalUcet from dBsp where idBsp = _idSkladu;
      end if; 
 
      _kopStred = true;
      if(not compareId(_idStredDoklSklad, _polDokl.idStred)) then
        _kopStred = false;
      end if;
      _kopCinnost = true;
      if(not compareId(_idCinnostDoklSklad, _polDokl.idCinnost)) then
        _kopCinnost = false;
      end if;
      _kopZakazku = true;
      if(not compareId(_idZakazkyDoklSklad, _polDokl.idZakazky)) then
        _kopZakazku = false;
      end if;

      select nextval('seq_dpolsklad') into _idPolSklad;
      insert into dpolsklad(idPolSklad, lastUpdate, idcenik, idBspSkl, idkarty, datVyst, datSklad, datUcto, kopDatUcto, modul, typPolozkyK, idDoklSklad,
          mnozMj, cenaMj, cenaMjPoriz, sumZkl, sumZklMen, idZklMdUcet, idZklDalUcet, kopZklMdUcet, kopZklDalUcet, baleniid, mnozBaleni, idmj,
          nazev, nazevA, nazevB, nazevC, sarze, expirace, datTrvan, datVyroby, cisRad, ucetni, eanKod, 
          idStred, kopStred, idZakazky, kopZakazku, idMeny, zdrojProFak, idCinnost, kopCinnost, idTypUcOp)
        values (_idPolSklad, LOCALTIMESTAMP, _polozkaSady.idcenik, _idSkladu, _skladKarta.idKarty, _polDokl.datVyst, clock_timestamp(), _datUcto, _polDokl.kopDatUcto, 'SKL', 'typPolozky.katalog', _idDoklSklad,    
          _mnozMj, _cenaMj, _cenaMj, _sumZkl, _sumZklMen, _idZklMdUcet, _idZklDalUcet, _kopZklMdUcet, _kopZklDalUcet, _polDokl.baleniid, 0.0, _cenik.idmj1,
          _cenik.nazev, _cenik.nazevA, _cenik.nazevB, _cenik.nazevC, null, null, _polDokl.datTrvan, _polDokl.datVyroby, _cisRad, true, _cenik.eanKod,
          _polDokl.idStred, _kopStred, _polDokl.idZakazky, _kopZakazku, _idMeny, false, _polDokl.idCinnost, _kopCinnost, _idTypUcOp);

      insert into dvazebtab (castka, idtabA, idtabB, modulA, modulB, storno, typVazbyK)  
        values (0.0, _idPolDokl, _idPolSklad, _modul, 'SKL', false, 'typVazbyDokl.polozkaSklSada');  

      _datPosl = greatest(_datPosl, (select datposl from skarty where idkarty = _skladKarta.idkarty));
    end if; 
  END LOOP; 
  
  _datPosl = coalesce(_datPosl, _polDokl.datVyst);
  update skarty set datposl = _datPosl where idSkladu = _polDokl.idBspSkl and idCenik = _polDokl.idCenik and idUcetObdobi = _idUcetObdobi;

  execute sprepoctiSumyDokladu(_idDoklSklad); 
end; 
$BODY$
  LANGUAGE 'plpgsql' VOLATILE
  COST 100;
ALTER FUNCTION createPolSkladSada(integer, character varying) OWNER TO %firmaRole%;




--drop function createPolSklad(integer, character varying);

CREATE OR REPLACE FUNCTION createPolSklad(_idPolDokl integer, _modul character varying) RETURNS integer AS
$BODY$
declare
  _idDoklSklad integer;
  _idDoklad integer;
  _polDokl record;
  _iducetobdobi integer;
  _idkarty integer;
  _idPolSklad integer;
  _cenaMj numeric(19,6);
  _cenaMjTuz numeric(19,6);
  _typPohybuK character varying;
  _prumCenaTuz numeric(19,6);
  _sumZkl numeric(19,6);
  _sumZklMen numeric(19,6);
  _kurz numeric(19,6);
  _idZklMdUcet integer;
  _idZklDalUcet integer;
  _kopZklMdUcet boolean;
  _kopZklDalUcet boolean;
  _sklad record;
  _typDoklK character varying;
  _mnozMj numeric(19,6);
  _isCiziMena boolean;
  _idMeny integer;  
  _skupZboz record;
  _idTypUcOp integer;
  _kopStred boolean;
  _kopCinnost boolean;
  _kopZakazku boolean;
  _idStredDoklSklad integer;
  _idCinnostDoklSklad integer;
  _idZakazkyDoklSklad integer;
  _typPohybuSkladKVydej character varying;
  _datUcto date; 
begin
  _kopZklMdUcet =false;
  _kopZklDalUcet = false;
  if(_modul like 'FAV' or _modul like 'FAP') then
    select * into _polDokl from dPolFak where idPolFak = _idPolDokl;
    _idDoklad = _polDokl.idDoklFak;
    select typDoklK into _typDoklK from dTypDokl where idTypDokl = (select idTypDokl from dDoklFak where idDoklFak = _polDokl.idDoklFak);
  else
    select * into _polDokl from dPolInt where idPolInt = _idPolDokl;
    _idDoklad = _polDokl.idDoklInt;
    _typDoklK = null; -- pro pokladnu dobropisy nejsou
  end if;

  if(_polDokl.datVyst >= '2014-01-01') then
    _datUcto = _polDokl.datUcto;
    if(_datUcto is null) then
      _datUcto = _polDokl.datVyst;
    end if;
  else
    _datUcto = _polDokl.datVyst;
  end if;
 
  select findparovanydoklsklad(_iddoklad, _modul, 'typVazbyDokl.hlavaSkl', true, _polDokl.idbspskl) into _idDoklSklad; 
  
  if(_idDoklSklad is null) then
    select createDoklSklad(_idPolDokl, _idDoklad, _modul) into _idDoklSklad;
    if(_idDoklSklad is null) then
      return null;
    end if;
  end if;

  if(_polDokl.idPolDobropis is not null and _polDokl.idPolDobropis > 0) then
    return null;
  end if;

  select findIdUcetniObdobiKDatu(_datUcto) into _iducetobdobi;
  select idkarty into _idkarty from skarty where idcenik = _polDokl.idcenik and idskladu = _polDokl.idbspskl and iducetobdobi = _iducetobdobi;
  select * into _sklad from dBsp where idBsp = _polDokl.idBspSkl;
  
  select typPohybuK, idstred, idcinnost, idZakazky
    into _typPohybuK, _idStredDoklSklad, _idCinnostDoklSklad, _idZakazkyDoklSklad
    from dDoklSklad where idDoklSklad = _idDoklSklad; 
  select getkurzfromdoklad(_idDoklSklad, _polDokl.datVyst) into _kurz;
  select prumCenaTuz into _prumCenaTuz from sKarty where idKarty = _idKarty;
  select isDoklSkladMenaCizi(_idDoklSklad, _polDokl.datVyst) into _isCiziMena;
  select idMeny, idTypUcOp into _idMeny, _idTypUcOp from dDoklSklad where idDoklSklad = _idDoklSklad;
      
  if(_typPohybuK like 'typPohybu.vydej') then
    -- vsechny dobropisy generuji prijemky
    --if(_typDoklK like 'typDokladu.dobropis') then
    --  _mnozMj = -_polDokl.mnozMj;
    --else
      _mnozMj = _polDokl.mnozMj;
    --end if;
    -- pokud vydavam ze skladu tak ma byt pouzita prumerna cena
    _cenaMj = _prumCenaTuz / _kurz;
    _sumZkl = _prumCenaTuz * _mnozMj;
    if(_isCiziMena) then
      _sumZklMen = _mnozMj * _cenaMj;
    else
      _sumZklMen = 0.0;
    end if;
    -- nastaveni uctu pro polSklad vydej
    select typPohybuSkladK into _typPohybuSkladKVydej from ddoklsklad where iddoklsklad = _idDoklSklad;
    select * into _skupZboz from cskupzboz where idskupzboz = (select idskupzboz from ccenik where idcenik = _polDokl.idCenik);
    if(_skupzboz.idskupzboz is not null) then
      -- Úpravy dle Bug 32848 – [WinStrom] - nerespektování upřesnění typu pohybu při tvorbě výdejky
      if(_typPohybuSkladKVydej is null) then
        _idZklMdUcet = _skupZboz.idprotiucetsklv;
      -- typPohybuSklad.vydejHoly -  Holý" výdej  - idprotiucetsklholyv
      elsif(_typPohybuSkladKVydej like 'typPohybuSklad.vydejHoly') then
        _idZklMdUcet = _skupZboz.idprotiucetsklholyv;
      -- typPohybuSklad.vydejPoFak - Výdej po faktuře  - idprotiucetsklv
      elsif(_typPohybuSkladKVydej like 'typPohybuSklad.vydejPoFak') then
        _idZklMdUcet = _skupZboz.idprotiucetsklv;
      -- typPohybuSklad.vydejPolot - Výdej polotovaru - idprotiucetsklholyv
      elsif(_typPohybuSkladKVydej like 'typPohybuSklad.vydejPolot') then
        _idZklMdUcet = _skupZboz.idprotiucetsklholyv;
      -- typPohybuSklad.vydejPoObch - Výdej po objednávce  -  idprotiucetsklv
      elsif(_typPohybuSkladKVydej like 'typPohybuSklad.vydejPoObch') then
        _idZklMdUcet = _skupZboz.idprotiucetsklv;
      -- typPohybuSklad.vydejPrevod -  Výdej pro převodku  - idprotiucetsklprevv
      elsif(_typPohybuSkladKVydej like 'typPohybuSklad.vydejPrevod') then
        _idZklMdUcet = _skupZboz.idprotiucetsklprevv;
      -- typPohybuSklad.vydejProFakCenik - Výdej pro fakturaci (ceny z ceníku) - idprotiucetsklv
      elsif(_typPohybuSkladKVydej like 'typPohybuSklad.vydejProFakCenik') then
        _idZklMdUcet = _skupZboz.idprotiucetsklv;
      -- typPohybuSklad.vydejProFakRucne - Výdej pro fakturaci (ceny ručně) - idprotiucetsklv
      elsif(_typPohybuSkladKVydej like 'typPohybuSklad.vydejProFakRucne') then
        _idZklMdUcet = _skupZboz.idprotiucetsklv; 
      --typPohybuSklad.vydejVyrob - Výdej výrobku  - idprotiucetsklholyv
      elsif(_typPohybuSkladKVydej like 'typPohybuSklad.vydejVyrob') then
        _idZklMdUcet = _skupZboz.idprotiucetsklholyv; 
      end if;
    end if; 
    
    if(_idZklMdUcet is null) then 
      -- pokud neni ve skupine zbozi nastaven protiucet tak vemu protiucet z hlavicky a dam kopirovat na true
      select idProtiUcet into _idZklMdUcet from dDoklSklad where idDoklSklad = _idDoklSklad;
      _kopZklMdUcet = true;
    end if;
    _idZklDalUcet = _sklad.idPrimUcet;
  else
    if(_typDoklK like 'typDokladu.dobropis' and _modul like 'FAV') then
      -- pokud se jedna o dobropis tak doplnuji nakupni ceny;
      _mnozMj = -_polDokl.mnozMj;
      --_cenaMjTuz = _prumCenaTuz;
      _cenaMjTuz = _polDokl.cenaMjNakup;
      if(_cenaMjTuz = 0.0) then
        _cenaMjTuz = (select nakupCena from ccenik where idCenik = _polDokl.idCenik);
        if(_cenaMj = 0.0) then
          _cenaMj = _polDokl.cenaMj * _kurz;
        end if;
      end if;
      _cenaMj = _cenaMjTuz / _kurz;
      _sumZkl = _mnozMj * _cenaMjTuz;
      if(_isCiziMena) then
        _sumZklMen = _mnozMj * _cenaMj;
      else
        _sumZklMen = 0.0;
      end if;
      select * into _skupZboz from cskupzboz where idskupzboz = (select idskupzboz from ccenik where idcenik = _polDokl.idCenik);
      if(_skupZboz.idProtiUcetSklV is not null) then
        -- pokud je ve skupine zbozi nastaven protiucet pro vydej ze skladu pouzije se ten a kopirovani se necha na false;
        _idZklDalUcet = _skupZboz.idProtiUcetSklV;
      else
        -- pokud neni ve skupine zbozi nastaven protiucet tak vemu protiucet z hlavicky a dam kopirovat na true
        select idProtiUcet into _idZklDalUcet from dDoklSklad where idDoklSklad = _idDoklSklad;
        _kopZklDalUcet = true;
      end if;
    else
      -- pokud na sklad prijmam zkopiruji se ceny z faktury
      _mnozMj = _polDokl.mnozMj;
      _sumZkl = _polDokl.sumZkl;
      if(_mnozMj <> 0.0) then
        _cenaMj = _sumZkl / _mnozMj;
      else
        _cenaMj = _sumZkl;
      end if;
      _cenaMj = _cenaMj / _kurz;
      if(_isCiziMena) then
        if(_mnozMj <> 0.0) then
          _sumZklMen = _cenaMj * _mnozMj;  
        else
          _sumZklMen = _cenaMj;
        end if;
      else
        _sumZklMen = 0.0;  
      end if;
      _idZklDalUcet = _polDokl.idZklMdUcet;
    end if;
    -- nastaveni uctu pro polSklad prijem
    _idZklMdUcet = _sklad.idPrimUcet;
  end if; 

  _kopStred = true;
  if(not compareId(_idStredDoklSklad, _polDokl.idStred)) then
    _kopStred = false;
  end if;
  _kopCinnost = true;
  if(not compareId(_idCinnostDoklSklad, _polDokl.idCinnost)) then
    _kopCinnost = false;
  end if;
  _kopZakazku = true;
  if(not compareId(_idZakazkyDoklSklad, _polDokl.idZakazky)) then
    _kopZakazku = false;
  end if;
  
  select nextval('seq_dpolsklad') into _idPolSklad;  
  insert into dpolsklad(idPolSklad, lastUpdate, idcenik, idbspskl, idkarty, datVyst, datSklad, datUcto, kopDatUcto, modul, typPolozkyK, idDoklSklad,
        mnozMj, cenaMj, cenaMjPoriz, sumZkl, sumZklMen, idZklMdUcet, idZklDalUcet, kopZklMdUcet, kopZklDalUcet, baleniid, mnozBaleni, idmj,
        nazev, nazevA, nazevB, nazevC, sarze, expirace, datTrvan, datVyroby, cisRad, ucetni, eanKod, 
        idStred, kopStred, idZakazky, kopZakazku, idMeny, zdrojProFak, idCinnost, kopCinnost, idTypUcOp)
      values (_idPolSklad, LOCALTIMESTAMP, _polDokl.idcenik, _polDokl.idbspskl, _idkarty, _polDokl.datVyst, clock_timestamp(), _datUcto, _polDokl.kopDatUcto,'SKL', 'typPolozky.katalog', _idDoklSklad,
        _mnozMj, _cenaMj, _cenaMj, _sumZkl, _sumZklMen, _idZklMdUcet, _idZklDalUcet, _kopZklMdUcet, _kopZklDalUcet, _polDokl.baleniid, _polDokl.mnozBaleni, _polDokl.idmj,
        _polDokl.nazev, _polDokl.nazevA, _polDokl.nazevB, _polDokl.nazevC, _polDokl.sarze, _polDokl.expirace, _polDokl.datTrvan, _polDokl.datVyroby, _polDokl.cisRad, true, _polDokl.eanKod,
        _polDokl.idStred, _kopStred, _polDokl.idZakazky, _kopZakazku, _idMeny, false, _polDokl.idCinnost, _kopCinnost, _idTypUcOp);
 
  insert into dvazebtab (castka, idtabA, idtabB, modulA, modulB, storno, typVazbyK)
       values (0.0, _idPolDokl, _idPolSklad, _modul, 'SKL', false, 'typVazbyDokl.polozkaSkl');

  -- prepocteni sum na dokladu
  execute sprepoctiSumyDokladu(_idDoklSklad);

  return _idPolSklad;
end;
$BODY$
  LANGUAGE 'plpgsql' VOLATILE
  COST 100;
ALTER FUNCTION createPolSklad(integer, character varying) OWNER TO %firmaRole%;

CREATE OR REPLACE FUNCTION skladrealizujpozadavky(i_idpolsklad integer, i_limit integer)
  RETURNS integer AS
$BODY$ 
declare
  _polSklad record;
  _polSkladDoklSklad record;
  _stavSkladu numeric(19,6);
  _rezervovano numeric(19,6);
  _firmaId integer; 
  _cenaMj numeric(19,6);
  _pocetPozadavku integer;
  _pocet integer;
  _resultId integer;
  _pocetPolSklad integer;
begin
  _pocet = 0;
  _resultId = null;
  for _polSklad in (
    select p.*, d.typPohybuK, d.stavSkladK from dpolsklad p join ddoklsklad d on p.idDoklSklad = d.idDoklSklad
    where typPohybuK = 'typPohybu.vydej' and mnozMj = 0 and mnozMjPlan != 0 and p.storno = false and idPolSklad > i_idpolsklad
    order by idPolSklad) loop
      /*behem aktualizace pozadavku na vydej muze dojit ke smazani nektere z polozek
      a diky tomu se pak muze rozhodit suma pozadavku na skladove karte*/
      select 1 into _pocetPolSklad from dPolSklad ps where ps.idPolSklad = _polSklad.idPolSklad limit 1;
      continue when _pocetPolSklad is null;
      
      _pocet = _pocet + 1;
      exit when _pocet > i_limit;
      _resultId = _polSklad.idPolSklad;
      select idFirmy into _firmaId from ddoklsklad where iddoklsklad = _polSklad.iddoklsklad;
      select dGetRezervovano(_polSklad.idcenik, _polSklad.idbspskl, _firmaId) into _rezervovano;
      --select dgetstavnasklade(_polSklad.idkarty, _polSklad.datSklad, /*_polSklad.idpolsklad*/ -1, _polSklad.sarze, _polSklad.expirace, _polSklad.typPohybuK) into _stavSkladu;
      select stavMj, prumCenaTuz into _stavSkladu, _cenaMj from skarty where idkarty = _polSklad.idKarty;
      
      if((_stavSkladu - _rezervovano) >= _polSklad.mnozMjPlan) then
        update skarty set
          pozadavkyMj = pozadavkyMj - _polSklad.mnozMjPlan,
          stavMjSPozadavky = stavMjSPozadavky + _polSklad.mnozMjPlan
        where idkarty = _polSklad.idKarty;
        
        update dpolsklad set mnozMj = mnozMjPlan, cenaMj = _cenaMj, sumZkl = mnozMjPlan * _cenaMj, datSklad = clock_timestamp(), lastupdate = LOCALTIMESTAMP
          where idpolsklad = _polSklad.idPolSklad;
        perform sprepoctisumydokladu(_polSklad.iddoklsklad);
        
        select count(*) into _pocetPozadavku from dpolSklad where idDoklSklad = _polSklad.idDoklSklad and mnozMj = 0.0 and mnozMjPlan != 0.0;
        if(_pocetPozadavku = 0) then
          if(compareVarchar(_polSklad.stavSkladK, 'stavSklad.vygenFaktNeuplny')) then
            update ddoklsklad set stavSkladK = 'stavSklad.vygenFakt' where idDoklSklad = _polSklad.idDoklSklad;
          end if;
          if(compareVarchar(_polSklad.stavSkladK, 'stavSklad.vygenPoklNeuplny')) then
            update ddoklsklad set stavSkladK = 'stavSklad.vygenPokl' where idDoklSklad = _polSklad.idDoklSklad;
          end if;
        end if;
      end if;
  end loop;
  return _resultId;
end;    
$BODY$
  LANGUAGE 'plpgsql' VOLATILE
  COST 100;
ALTER FUNCTION skladrealizujpozadavky(integer, integer) OWNER TO %firmaRole%;

CREATE OR REPLACE FUNCTION createpolskladdobropis(_iddobropisovanepolsklad integer, _iddobropisujicipoldokl integer, _modul character varying, _typVazbyK character varying)
  RETURNS integer AS
$BODY$ 
declare
  _idPolSklad integer;
  _idDoklad integer;
  _idDoklSklad integer; 
  _dobropisovanaPolSklad record;
  _kopZakazku boolean;
  _kopStred boolean;
  _kopCinnost boolean;
  _polDokl record;
  _idStredDoklSklad integer;
  _idCinnostDoklSklad integer;
  _idZakazkyDoklSklad integer;
  _iducetobdobi integer;
  _idkarty integer;
  _mnozMj numeric(19,6); 
  _pomer numeric(19,12);
  _cenaMj numeric(19,6); 
  _cenaMjTuz numeric(19,6); 
  _sumZkl numeric(19,6); 
  _sumZklMen numeric(19,6); 
  _mnozMjDobropisovane numeric(19,6);
  _idZklMdUcet integer; 
  _idZklDalUcet integer; 
  _kopZklMdUcet boolean; 
  _kopZklDalUcet boolean; 
  _idMeny integer;
  _idTypUcOp integer; 
  _kurz numeric(19,6); 
  _ciziMena boolean;
  _typPohybuK character varying;
  _nasobic numeric(19,6);
  _skupZboz record;
  _datUcto date;
  _idBspSkl integer; 
begin
  if(_modul like 'FAV' or _modul like 'FAP') then 
    select * into _polDokl from dPolFak where idPolFak = _iddobropisujicipoldokl;
    _idDoklad = _polDokl.idDoklFak;
    select mnozMj into _mnozMjDobropisovane from dPolFak where idPolFak = _polDokl.idPolDobropis;
    select typPohybuK into _typPohybuK from ddoklfak where iddoklfak = _idDoklad;
  else 
    select * into _polDokl from dPolInt where idPolInt = _iddobropisujicipoldokl;
    _idDoklad = _polDokl.idDoklInt;
    select mnozMj into _mnozMjDobropisovane from dPolInt where idPolInt = _polDokl.idPolDobropis;
    select typPohybuK into _typPohybuK from ddoklfak where iddoklfak = _idDoklad;
  end if; 

  if(_polDokl.datVyst >= '2014-01-01') then
    _datUcto = _polDokl.datUcto;
    if(_datUcto is null) then
      _datUcto = _polDokl.datVyst;
    end if;
  else
    _datUcto = _polDokl.datVyst;
  end if;

  select findparovanydoklsklad(_iddoklad, _modul, 'typVazbyDokl.hlavaSkl', true, _polDokl.idBspSkl) into _idDoklSklad;  
   
  if(_idDoklSklad is null) then 
    select createDoklSklad(_iddobropisujicipoldokl, _idDoklad, _modul) into _idDoklSklad; 
    if(_idDoklSklad is null) then 
      return null; 
    end if; 
  end if; 

  select * into _dobropisovanaPolSklad from dpolsklad where idpolsklad = _iddobropisovanepolsklad;
  select idstred, idcinnost, idZakazky, idMeny
      into _idStredDoklSklad, _idCinnostDoklSklad, _idZakazkyDoklSklad, _idMeny 
      from dDoklSklad where idDoklSklad = _idDoklSklad;  

  select findIdUcetniObdobiKDatu(_datUcto) into _iducetobdobi; 
  _idBspSkl = _dobropisovanaPolSklad.idbspskl; -- puvodni stav
  _idBspSkl = _polDokl.idBspSkl; -- dle Bug 39343 – [Bug] Dobropis + jiný sklad, než ze kterého bylo vyskladněno
  select idkarty into _idkarty from skarty where idcenik = _dobropisovanaPolSklad.idcenik and idskladu = _idBspSkl and iducetobdobi = _iducetobdobi;
  if(_idkarty is null) then
    raise exception 'idkarty je null protoze neexistuje skladova karta pro ucetni obdobi';
  end if;
  
  _nasobic = 1;
  if(_typPohybuK is not null and _typPohybuK = 'typPohybu.vydej')then
    _nasobic = -1;
  end if;
  if(_dobropisovanaPolSklad.mnozMj = 0) then
    _mnozMj = _nasobic * _dobropisovanaPolSklad.mnozMjPlan; 
  else
    _mnozMj = _nasobic * _dobropisovanaPolSklad.mnozMj; 
  end if;
  _sumZkl = _nasobic * _dobropisovanaPolSklad.sumZkl;
  
  if((-1 * _polDokl.mnozMj) <> _mnozMjDobropisovane) then
    _pomer = -1 * _polDokl.mnozMj / _mnozMjDobropisovane;
    _mnozMj = _mnozMj * _pomer;
    _sumZkl = _sumZkl * _pomer;
  end if;
  --raise exception '% % %', _polDokl.mnozMj, _mnozMjDobropisovane, _mnozMj;
  -- zohledneni meny
  select ismenacizi(_idMeny, _polDokl.datvyst) into _ciziMena;
  if(_ciziMena) then
    select getkurzfromdoklad(_idDoklSklad, _polDokl.datVyst) into _kurz; 
    _sumZklMen = _sumZkl / _kurz;
    _cenaMj = _sumZklMen / _mnozMj;
  else
    _sumZklMen = 0.0;
    _cenaMj = _sumZkl / _mnozMj;
  end if;
    
  _kopStred = true; 
  if(not compareId(_idStredDoklSklad, _polDokl.idStred)) then 
    _kopStred = false; 
  end if; 
  _kopCinnost = true; 
  if(not compareId(_idCinnostDoklSklad, _polDokl.idCinnost)) then 
    _kopCinnost = false; 
  end if; 
  _kopZakazku = true; 
  if(not compareId(_idZakazkyDoklSklad, _polDokl.idZakazky)) then 
    _kopZakazku = false; 
  end if;  

  select idprimucet into _idZklMdUcet from dbsp where idbsp = _idBspSkl;
  _kopZklMdUcet = false;

  _kopZklDalUcet = false;
  if(_modul like 'FAV') then
    select * into _skupZboz from cskupzboz where idskupzboz = (select idskupzboz from ccenik where idcenik = _dobropisovanaPolSklad.idCenik); 
    if(_skupZboz.idProtiUcetSklV is not null) then 
      -- pokud je ve skupine zbozi nastaven protiucet pro vydej ze skladu pouzije se ten a kopirovani se necha na false; 
      _idZklDalUcet = _skupZboz.idProtiUcetSklV; 
    else 
      -- pokud neni ve skupine zbozi nastaven protiucet tak vemu protiucet z hlavicky a dam kopirovat na true 
      select idProtiUcet into _idZklDalUcet from dDoklSklad where idDoklSklad = _idDoklSklad; 
      _kopZklDalUcet = true;
    end if; 
  else
    _idZklDalUcet = _polDokl.idZklMdUcet; 
  end if;

  select nextval('seq_dpolsklad') into _idPolSklad;   
  insert into dpolsklad(idPolSklad, lastUpdate, idcenik, idbspskl, idkarty, datVyst, datSklad, datUcto, kopDatUcto, modul, typPolozkyK, idDoklSklad, 
      mnozMj, cenaMj, cenaMjPoriz, sumZkl, sumZklMen, idZklMdUcet, idZklDalUcet, kopZklMdUcet, kopZklDalUcet, baleniid, mnozBaleni, idmj, 
      nazev, nazevA, nazevB, nazevC, sarze, expirace, datTrvan, datVyroby, cisRad, ucetni, eanKod,  
      idStred, kopStred, idZakazky, kopZakazku, idMeny, zdrojProFak, idCinnost, kopCinnost, idTypUcOp,
      idPolDobropis) 
    values (_idPolSklad, LOCALTIMESTAMP, _dobropisovanaPolSklad.idcenik, _idBspSkl, _idkarty, _polDokl.datVyst, /*_dobropisovanaPolSklad.datSklad + interval '0.000001'*/ clock_timestamp(), _datUcto, _polDokl.kopDatUcto, 'SKL', 'typPolozky.katalog', _idDoklSklad, 
      _mnozMj, _cenaMj, _cenaMj, _sumZkl, _sumZklMen, _idZklMdUcet, _idZklDalUcet, _kopZklMdUcet, _kopZklDalUcet, _polDokl.baleniid, _polDokl.mnozBaleni, _dobropisovanaPolSklad.idmj, 
      _dobropisovanaPolSklad.nazev, _dobropisovanaPolSklad.nazevA, _dobropisovanaPolSklad.nazevB, _dobropisovanaPolSklad.nazevC, _dobropisovanaPolSklad.sarze, _dobropisovanaPolSklad.expirace, _dobropisovanaPolSklad.datTrvan, _dobropisovanaPolSklad.datVyroby, _dobropisovanaPolSklad.cisRad, true, _dobropisovanaPolSklad.eanKod, 
      _polDokl.idStred, _kopStred, _polDokl.idZakazky, _kopZakazku, _idMeny, false, _polDokl.idCinnost, _kopCinnost, _idTypUcOp,
      _iddobropisovanepolsklad); 

  insert into dvazebtab (castka, idtabA, idtabB, modulA, modulB, storno, typVazbyK) 
       values (0.0, _iddobropisujicipoldokl, _idPolSklad, _modul, 'SKL', false, _typVazbyK); 

  -- prepocteni sum na dokladu 
  execute sprepoctiSumyDokladu(_idDoklSklad); 

  return _idPolSklad; 
end; 
$BODY$
  LANGUAGE plpgsql VOLATILE
  COST 100;
ALTER FUNCTION createpolskladdobropis(integer, integer, character varying, character varying) OWNER TO %firmaRole%;



-- zvyseni verze databaze
insert into dbversion(iddbversion, dbversion, needreimport, appversion, upgradetime)
  values (nextval('seq_dbversion'), 1117, 't', %appVersion%, clock_timestamp());
