--------------------------------------------------------------------------------
-- Bug 17058 – [WinStrom] Kusovník - vyskladňování polotovarů
--------------------------------------------------------------------------------

-- Function: dpolskladinsertkusovnik()

-- DROP FUNCTION dpolskladinsertkusovnik();

CREATE OR REPLACE FUNCTION dpolskladinsertkusovnik()
  RETURNS trigger AS
$BODY$
declare
    _typPohybuK varchar;
    _typZasobyK varchar;
    _idKusovnik integer;
    _iducetobdobi integer;
    _stavNaSklade numeric;
    _sklKarta record;
    _cenik record;
    _sklzapor boolean;
    _idVydejPolSklad integer;
    _idParovanyDoklad integer;
    potKus record;
    _i integer;
    _kurz numeric;
    _isCiziMena boolean;
    _idMeny integer;
    _mnozMj numeric;
    _cenaMj numeric;
    _sumZkl numeric;
    _newSumZkl numeric;
    _sumZklMen numeric;
begin
  -- napriklad pokud se jedna o prevodku mezi sklady nechceme z prijemky vytvaret prevodku
  if (NEW.prevodka = true) then
    return NEW;
  end if;

  -- typem pohybu musi byt prijem
  select typpohybuk into _typPohybuK from ddoklsklad d where d.iddoklsklad = NEW.iddoklsklad;
  if (_typPohybuK <> 'typPohybu.prijem') then
    return NEW;
  end if;

  -- idcenik odkazuje na neco co je soucasti kusovniku.
  select ckus.idkusovnik into _idKusovnik from ckusovnik ckus where ckus.idcenik = NEW.idcenik
    and exists (select 1 from ckusovnik cPot where cPot.idotec = ckus.idkusovnik);
  if (_idKusovnik is null) then
    return NEW;
  end if;

  -- vytvorime nebo nalezneme zparovany doklad skl vydej
  select findparovanydoklsklad(NEW.iddoklsklad, NEW.modul, 'typVazbyDokl.hlavaSklKus') into _idParovanyDoklad;
  if (_idParovanyDoklad is null) then
    select createdoklsklad(NEW.idpolsklad, NEW.iddoklsklad, NEW.modul) into _idParovanyDoklad;
  end if;

  select isDoklSkladMenaCizi(_idParovanyDoklad, NEW.datVyst) into _isCiziMena;
  select getkurzfromdoklad(_idParovanyDoklad, NEW.datVyst) into _kurz;
  select idMeny into _idMeny from dDoklSklad where idDoklSklad = _idParovanyDoklad;

  _i = 0;
  _newSumZkl = 0.0;

  -- pro vsechny kusovniky, ktere jsou pod nasim kusovnikem a jsou skladove nebo vyroba
  for potKus in (select * from ckusovnik cPot where cPot.idotec = _idKusovnik) loop
    select * into _cenik from cCenik where idCenik = potKus.idCenik;

    select typZasobyK into _typZasobyK from cCenik cen where cen.idCenik = potKus.idCenik;
    if (_typZasobyK <> 'typZasoby.skladem' and _typZasobyK <> 'typZasoby.vyrobek') then
        _newSumZkl = _newSumZkl + _cenik.nakupcena * potKus.mnoz;
        continue;
    end if;
    _i = _i + 1;

    select findIdUcetniObdobiKDatu(NEW.datVyst) into _iducetobdobi;
    select * into _sklKarta from skarty where idcenik = potKus.idcenik and idtypdokl = NEW.idskladu and iducetobdobi = _iducetobdobi;

   _mnozMj = potKus.mnoz * NEW.mnozMj;
   _cenaMj = _sklKarta.prumCenaTuz / _kurz;
   _sumZkl = _mnozMj * _sklKarta.prumCenaTuz;
   _newSumZkl = _newSumZkl + _sumZkl;

   if(_isCiziMena) then
     _sumZklMen = _mnozMj * _cenaMj;
   else
     _sumZklMen = 0.0;
   end if;

    -- TODO doplnit sarze a expirace
    select dgetstavnasklade(_sklKarta.idkarty, NEW.datVyst, -1, NEW.idfirmy, NULL, NULL) into _stavNaSklade;

    -- osetreni zda je povoleny zaporny stav na sklade
    if (_stavNaSklade - _mnozMj < 0) then
        select sklzapor into _sklzapor from nNastav where NEW.datVyst >= PlatiOdData order by PlatiOdData desc limit 1;
        if (_sklzapor is null) then
            select sklzapor into _sklzapor from nNastav where PlatiOdData is null limit 1;
        end if;
        if _sklzapor is null or _sklzapor = false then
            raise exception '0002:sPreporctiSkladKartu: neni povolen zaporny stav skladu';
        end if;
    end if;

    select nextval('seq_dpolsklad'::regclass) into _idVydejPolSklad;

    insert into dpolsklad(idPolSklad, idcenik, idskladu, idkarty, datVyst, datUcto, modul, typPolozkyK, idDoklSklad,
        mnozMj, cenaMj, sumZkl, sumZklMen, idZklMdUcet, idZklDalUcet, kopZklMdUcet, kopZklDalUcet, baleniid, mnozBaleni,
        nazev, nazevA, nazevB, nazevC, sarze, expirace, cisRad, ucetni, idStred, idZakazky, idMeny)
    values (_idVydejPolSklad, potKus.idcenik, NEW.idskladu, _sklKarta.idkarty, NEW.datVyst, NEW.datUcto, NEW.modul, NEW.typpolozkyk, _idParovanyDoklad,
        _mnozMj, _cenaMj, _sumZkl, _sumZklMen, NEW.idzkldalucet, NEW.idzklmducet, false, false, NEW.baleniid, NEW.mnozbaleni,
        _cenik.nazev, _cenik.nazevA, _cenik.nazevB, _cenik.nazevC, NULL, NULL, NEW.cisRad * 100 + _i, true,
        NEW.idStred, NEW.idZakazky, _idMeny);

    insert into dvazebtab (castka, idtabA, idtabB, modulA, modulB, storno, typVazbyK)
      values (0.0, NEW.idpolsklad, _idVydejPolSklad, 'SKL', 'SKL', false, 'typVazbyDokl.polozkaSklKus');

  end loop;

  select getkurzfromdoklad(NEW.iddoklsklad, NEW.datvyst) into _kurz;
  _cenaMj = _newSumZkl / NEW.mnozMj / _kurz;

  select isDoklSkladMenaCizi(NEW.iddoklsklad, NEW.datVyst) into _isCiziMena;
  if (_isCiziMena) then
    _sumZklMen = NEW.cenaMj * NEW.mnozMj;
  else
    _sumZklMen = 0.0;
  end if;

  update dpolsklad set sumzkl = _newSumZkl, cenaMj = _cenaMj, sumZklMen = _sumZklMen where idpolsklad = NEW.idpolsklad;

  return NEW;
end;
$BODY$
  LANGUAGE 'plpgsql' VOLATILE
  COST 100;
ALTER FUNCTION dpolskladinsertkusovnik() OWNER TO %firmaRole%;

CREATE TRIGGER dpolskladinsertkusovnik AFTER INSERT ON dpolsklad
  FOR EACH ROW EXECUTE PROCEDURE dpolskladinsertkusovnik();

-- Function: createdoklsklad(integer, integer, character varying)

-- DROP FUNCTION createdoklsklad(integer, integer, character varying);

CREATE OR REPLACE FUNCTION createdoklsklad(_idpoldokl integer, _iddoklad integer, _modul character varying)
  RETURNS integer AS
$BODY$
declare
  _doklad record;
  _polDokl record;
  _doklSklad record;
  _idDoklSklad integer;
  _cisDoklad character varying;
  _idrada integer;
  _rada record;
  _sklad record;
  _idUcetObd integer;
  _radaRok record;
  _typPohybuSklad character varying;
  _idTypUcOp integer;
  _cisloDokladuCursor refCursor;
  _typDoklFak record;
  _idUcetSklad integer;
  _idProtiUcet integer;
  _typUcOp record;
  _kurz numeric(19,6);
  _idMenyTuzemsko integer;
  _idMenySklad integer;
  _kurzMnozstvi integer;
  _typVazby character varying;
begin
  if(_modul like 'POK') then
    select * into _doklad from dDoklInt where idDoklInt = _idDoklad;
    select * into _polDokl from dPolInt where idPolInt = _idPolDokl;
  elsif (_modul = 'FAK' or _modul = 'FAP') then
    select * into _doklad from dDoklFak where idDoklFak = _idDoklad;
    select * into _polDokl from dPolFak where idPolFak = _idPolDokl;
  else
    select * into _doklad from dDoklSklad where idDoklSklad = _idDoklad;
    select * into _polDokl from dPolSklad where idPolSklad = _idPolDokl;
  end if;

  select * into _sklad from dTypDokl where idTypDokl = _polDokl.idskladu;
  select * into _typDoklFak from dTypDokl where idTypDokl = _doklad.idTypDokl;

  -- urceni spravneho typu pohybu
  if((_typDoklFak.modul like 'FAV' or _typDoklFak.modul like 'FAV') and (_typDoklFak.typDoklK like 'typDokladu.dobropis')) then
    _typPohybuSklad = _doklad.typPohybuK;
  else
    if(_doklad.typPohybuK like 'typPohybu.prijem') then
      _typPohybuSklad = 'typPohybu.vydej';
    else
      _typPohybuSklad = 'typPohybu.prijem';
    end if;
  end if;

  -- doplneni zbyvajicich veci zavislich na typu pohybu
  if(_typPohybuSklad like 'typPohybu.prijem') then
    _idrada = _sklad.idRadyPrijem;
    _idTypUcOp = _sklad.idTypUcOpP;
  else
    _idrada = _sklad.idRadyVydej;
    _idTypUcOp = _sklad.idTypUcOpV;
  end if;

  _idUcetSklad = _sklad.idPrimUcet;
  select * into _typUcOp from utypucop where idtypucop = _idTypUcOp;
  if(_typPohybuSklad like 'typPohybu.prijem') then
    _idProtiUcet = _typUcOp.idProtiucetP;
  else
    _idProtiUcet = _typUcOp.idProtiucetV;
  end if;

  select * into _rada from drady where idRady = _idrada;
  select findIdUcetniObdobiKDatu(_polDokl.datVyst) into _idUcetObd;
  select * into _radaRok from dRadyRok where iducetobdobi = _idUcetObd and idrady = _idrada;

  select nextval('seq_ddoklsklad') into _idDoklSklad;
  select dgetcisdokl(null, _idrada, _idUcetObd, 'SKL', _typPohybuSklad) into _cisloDokladuCursor;
  fetch _cisloDokladuCursor into _cisDoklad;

  -- ziskani kurzu pro menu skladu
  -- select getKurzFromSklad(_polDokl.idSkladu, _polDokl.datVyst) into _kurz;
  -- nejde pouzit protoze potrebuju jak kurz tak kurzove mnozstvi a ne prepocteny nna kurzove mnozstvi 1
  select idMeny into _idMenyTuzemsko from nNastav where _polDokl.datVyst >= PlatiOdData order by PlatiOdData desc limit 1;
  if (_idMenyTuzemsko is null) then
    select idMeny into _idMenyTuzemsko from nNastav where PlatiOdData is null limit 1;
  end if;
  --select idMeny into _idMenySklad from dTypDokl where idTypDokl = _polDokl.idSkladu;

  if(_sklad.idMeny is null or _sklad.idMeny = _idMenyTuzemsko) then
    _kurz = 0.0;
    _kurzMnozstvi = 1;
  else
    select nbStred, kurzMnozstvi into _kurz, _kurzMnozstvi from uKurzy
      where idMeny = _sklad.idMeny and _polDokl.datVyst >= PlatiOdData and cenotvorba = false
      order by PlatiOdData desc limit 1;
    if(_kurz is null) then
      select nbStred, kurzMnozstvi into _kurz, _kurzMnozstvi from uKurzy
        where idMeny = _sklad.idMeny and PlatiOdData is null and cenotvorba = false
        limit 1;
    end if;
    if(_kurz is null) then
      raise exception '0001:getKurzFromSklad : neexistuje kurz!';
    end if;
  end if;

  -- vytvoreni noveho skladoveho dokladu
  insert into dDoklSklad (idDoklSklad, datVyst, datUcto, kod, modul, typPohybuK, idTypDokl, ucetni,
         idFirmy, nazFirmy, ulice, mesto, psc, idStatu, ic, dic, vatId,
         idMur, idKrajUrc, idStatUrc, idTypUcOp, cisObj, cisDodak, doprava, idDruhDopr, popis, poznam,
         idStred, idZakazky, idMeny, kurz, kurzMnozstvi,
         eanKod, idstatpuvod, idstatodesl,
         idPrimUcet, idProtiUcet
         )
       values (_idDoklSklad, _doklad.datVyst, _doklad.datVyst, _cisDoklad, 'SKL', _typPohybuSklad, _sklad.idTypDokl, _sklad.ucetni,
         _doklad.idFirmy, _doklad.nazFirmy, _doklad.ulice, _doklad.mesto, _doklad.psc, _doklad.idStatu, _doklad.ic, _doklad.dic, _doklad.vatId,
         _doklad.idMur, _doklad.idKrajUrc, _doklad.idStatUrc, _idTypUcOp, _doklad.cisObj, null, _doklad.doprava, _doklad.idDruhDopr, _doklad.popis, _doklad.poznam,
         _doklad.idStred, _doklad.idZakazky, _sklad.idMeny, _kurz, _kurzMnozstvi,
         _doklad.eanKod, _doklad.idstatpuvod, _doklad.idstatodesl,
         _idUcetSklad, _idProtiUcet
         );


  if (_modul = 'SKL') then
    _typVazby = 'typVazbyDokl.hlavaSklKus';
  else
    _typVazby = 'typVazbyDokl.hlavaSkl';
  end if;

  -- TODO mozna doresit storno
  insert into dvazebtab (castka, idtabA, idtabB, modulA, modulB, storno, typVazbyK)
       values (0.0, _idDoklad, _idDoklSklad, _doklad.modul, 'SKL', false, _typVazby);

  return _idDoklSklad;
end;
$BODY$
  LANGUAGE 'plpgsql' VOLATILE
  COST 100;
ALTER FUNCTION createdoklsklad(integer, integer, character varying) OWNER TO %firmaRole%;


CREATE OR REPLACE FUNCTION findparovanydoklsklad(_iddoklad integer, _modul character varying)
  RETURNS integer AS
$BODY$
declare
  _idDoklSklad integer;
begin
  return findparovanydoklsklad(_iddoklad, _modul, 'typVazbyDokl.hlavaSkl');
end;
$BODY$
  LANGUAGE 'plpgsql' VOLATILE
  COST 100;
ALTER FUNCTION findparovanydoklsklad(integer, character varying) OWNER TO %firmaRole%;

CREATE OR REPLACE FUNCTION findparovanydoklsklad(_iddoklad integer, _modul character varying, _typvazby character varying)
  RETURNS integer AS
$BODY$
declare
  _idDoklSklad integer;
begin
  select idTabB into _idDoklSklad from dvazebtab where
    typVazbyK = _typVazby
    and idTabA = _idDoklad and modulA = _modul;
  if(_idDoklSklad is null) then
    select idTabA into _idDoklSklad from dvazebtab where
      typVazbyK = _typVazby
      and idTabB = _idDoklad and modulB = _modul;
  end if;
  return _idDoklSklad;
end; 
$BODY$
  LANGUAGE 'plpgsql' VOLATILE
  COST 100;
ALTER FUNCTION findparovanydoklsklad(integer, character varying, character varying) OWNER TO %firmaRole%;


-- Function: findvicevazeb(character varying, integer, character varying, integer, boolean)

-- DROP FUNCTION findvicevazeb(character varying, integer, character varying, integer, boolean);

CREATE OR REPLACE FUNCTION findvicevazeb(_inserttab character varying, _iddokl integer, _modul character varying, _uroven integer, _prechodvp boolean)
  RETURNS void AS
$BODY$
declare
  tabulka varchar(20);
  poleStav varchar(20);
  dotaz varchar;
  prechodPom varchar;
begin
  tabulka = getdoklnamepostfixbymodul(_modul);
  prechodPom = '';

  if (_uroven = 0) then
    if (tabulka = 'Fak') then
      poleStav = 'stavUhrK';
    else
      poleStav = 'stavUzivK';
    end if;

    dotaz =
      'select 0 as idVazby, null as typVazbyK, D.idDokl' || tabulka || ' as idDokl, D.modul, ''modulUcetni.''||D.Modul as ModulK, D.kod, D.varSym,
        D.datVyst, D.datUcto, D.sumCelkem, D.sumCelkemMen, D.popis, D.poznam, D.idTypDokl, D.idMeny, D.idUziv, D.' || poleStav || ' as stavK
        from dDokl' || tabulka || ' as D where D.idDokl' || tabulka || ' = ' || _idDokl || ' and D.modul = ''' || _modul || '''';
  else

  if (tabulka = 'Fak') then
    dotaz =
      'select V.idVazby, V.typVazbyK, D.idDoklInt as idDokl, D.modul, ''modulUcetni.''||D.Modul as ModulK, D.kod, D.varSym,
        D.datVyst, D.datUcto, D.sumCelkem, D.sumCelkemMen, D.popis, D.poznam, D.idTypDokl, D.idMeny, D.idUziv, D.stavUzivK as stavK
        from dVazebTab as V left outer join dDoklInt as D on V.idTabB=D.idDoklInt and V.modulB=D.modul
        where V.idTabA=' || _idDokl || ' and V.modulA=''' || _modul || ''' and V.typVazbyK in (''typVazbyDokl.uhrada'', ''typVazbyDokl.uhradaKR'',
            ''typVazbyDokl.uhradaZB'', ''typVazbyDokl.preceneni'', ''typVazbyDokl.vazbaZDD'', ''typVazbyDokl.uhradaZALOHY'')
      union
      select V.idVazby, V.typVazbyK, D.idDoklSklad as idDokl, D.modul, ''modulUcetni.''||D.Modul as ModulK, D.kod, D.varSym,
        D.datVyst, D.datUcto, D.sumCelkem, D.sumCelkemMen, D.popis, D.poznam, D.idTypDokl, D.idMeny, D.idUziv, D.stavUzivK as stavK
        from dVazebTab as V left outer join dDoklSklad as D on V.idTabB=D.idDoklSklad and V.modulB=D.modul
        where V.idTabA=' || _idDokl || ' and V.modulA=''' || _modul || ''' and V.typVazbyK = ''typVazbyDokl.hlavaSkl''
      union
      select min(V.idVazby) as idVazby, V.typVazbyK, D.idDoklFak as idDokl, D.modul, ''modulUcetni.''||D.Modul as ModulK, D.kod, D.varSym,
        D.datVyst, D.datUcto, D.sumCelkem, D.sumCelkemMen, D.popis, D.poznam, D.idTypDokl, D.idMeny, D.idUziv, D.stavUhrK as stavK
        from dVazebTab as V left outer join dDoklFak as D on V.idTabB=D.idDoklFak and V.modulB=D.modul
        where V.idTabA in (select idPolFak from dPolFak where idDoklFak=' || _idDokl || ') and V.modulA=''' || _modul || ''' and
          V.typVazbyK in (''typVazbyDokl.odpocetZALOHY'', ''typVazbyDokl.odpocetZDD'')
        group by V.typVazbyK, D.idDoklFak, D.modul, D.kod, D.varSym, D.datVyst, D.datUcto, D.sumCelkem, D.sumCelkemMen, D.popis, D.poznam, D.idTypDokl, D.idMeny, D.idUziv, D.stavuhrK
      union
      select min(V.idVazby) as idVazby, V.typVazbyK, D.idDoklFak as idDokl, D.modul, ''modulUcetni.''||D.Modul as ModulK, D.kod, D.varSym,
        D.datVyst, D.datUcto, D.sumCelkem, D.sumCelkemMen, D.popis, D.poznam, D.idTypDokl, D.idMeny, D.idUziv, D.stavUhrK as stavK
        from dVazebTab as V left outer join dDoklFak as D on D.idDoklFak=
                (select idDoklFak from dPolFak where idPolFak = V.idTabA) and V.modulA=D.modul
        where V.idTabB=' || _idDokl || ' and V.modulB=''' || _modul || ''' and V.typVazbyK in (''typVazbyDokl.odpocetZALOHY'', ''typVazbyDokl.odpocetZDD'')
        group by V.typVazbyK, D.idDoklFak, D.modul, D.kod, D.varSym, D.datVyst, D.datUcto, D.sumCelkem, D.sumCelkemMen, D.popis, D.poznam, D.idTypDokl, D.idMeny, D.idUziv, D.stavUhrK
      union
      select V.idVazby, V.typVazbyK, D.idDoklObch as idDokl, D.modul, ''modulUcetni.''||D.Modul as ModulK, D.kod, D.varSym,
        D.datVyst, D.datUcto, D.sumCelkem, D.sumCelkemMen, D.popis, D.poznam, D.idTypDokl, D.idMeny, D.idUziv, D.stavUzivK as stavK
        from dVazebTab as V left outer join dDoklObch as D on V.idTabA=D.idDoklObch and V.modulA=D.modul
        where V.idTabB=' || _idDokl || ' and V.modulB=''' || _modul || ''' and V.typVazbyK in (''typVazbyDokl.obchod_faktura_hla'', ''typVazbyDokl.obchod_zaloha_hla'')';
  else if (tabulka = 'Int') then
    dotaz =
      'select V.idVazby, V.typVazbyK, D.idDoklFak as idDokl, D.modul, ''modulUcetni.''||D.Modul as ModulK, D.kod, D.varSym,
        D.datVyst, D.datUcto, D.sumCelkem, D.sumCelkemMen, D.popis, D.poznam, D.idTypDokl, D.idMeny, D.idUziv, D.stavUhrK as stavK
        from dVazebTab as V left outer join dDoklFak as D on V.idTabA=D.idDoklFak and V.modulA=D.modul
        where V.idTabB=' || _idDokl || ' and V.modulB=''' || _modul || ''' and V.typVazbyK in (''typVazbyDokl.uhrada'', ''typVazbyDokl.uhradaKR'',
            ''typVazbyDokl.uhradaZB'', ''typVazbyDokl.preceneni'', ''typVazbyDokl.vazbaZDD'', ''typVazbyDokl.uhradaZALOHY'')
      union
      select V.idVazby, V.typVazbyK, D.idDoklSklad as idDokl, D.modul, ''modulUcetni.''||D.Modul as ModulK, D.kod, D.varSym,
        D.datVyst, D.datUcto, D.sumCelkem, D.sumCelkemMen, D.popis, D.poznam, D.idTypDokl, D.idMeny, D.idUziv, D.stavUzivK as stavK
        from dVazebTab as V left outer join dDoklSklad as D on V.idTabB=D.idDoklSklad and V.modulB=D.modul
        where V.idTabA=' || _idDokl || ' and V.modulA=''' || _modul || ''' and V.typVazbyK in (''typVazbyDokl.hlavaSkl'')
      union
      select V.idVazby, V.typVazbyK, D.idDoklInt as idDokl, D.modul, ''modulUcetni.''||D.Modul as ModulK, D.kod, D.varSym,
        D.datVyst, D.datUcto, D.sumCelkem, D.sumCelkemMen, D.popis, D.poznam, D.idTypDokl, D.idMeny, D.idUziv, D.stavUzivK as stavK
        from dVazebTab as V left outer join dDoklInt as D on V.idTabA=D.idDoklInt and V.modulA=D.modul
        where V.idTabB=' || _idDokl || ' and V.modulB=''' || _modul || ''' and V.typVazbyK in (''typVazbyDokl.KR'', ''typVazbyDokl.ZB'')
      union
      select V.idVazby, V.typVazbyK, D.idDoklInt as idDokl, D.modul, ''modulUcetni.''||D.Modul as ModulK, D.kod, D.varSym,
        D.datVyst, D.datUcto, D.sumCelkem, D.sumCelkemMen, D.popis, D.poznam, D.idTypDokl, D.idMeny, D.idUziv, D.stavUzivK as stavK
        from dVazebTab as V left outer join dDoklInt as D on V.idTabB=D.idDoklInt and V.modulB=D.modul
        where V.idTabA=' || _idDokl || ' and V.modulA=''' || _modul || ''' and V.typVazbyK in (''typVazbyDokl.KR'', ''typVazbyDokl.ZB'')';
  else if (tabulka = 'Sklad') then
    dotaz =
      'select V.idVazby, V.typVazbyK, D.idDoklFak as idDokl, D.modul, ''modulUcetni.''||D.Modul as ModulK, D.kod, D.varSym,
        D.datVyst, D.datUcto, D.sumCelkem, D.sumCelkemMen, D.popis, D.poznam, D.idTypDokl, D.idMeny, D.idUziv, D.stavUhrK as stavK
        from dVazebTab as V left outer join dDoklFak as D on V.idTabA=D.idDoklFak and V.modulA=D.modul
        where V.idTabB=' || _idDokl || ' and V.modulB=''' || _modul || ''' and
          V.modulA in (''FAV'',''FAP'',''PHL'',''ZAV'') and V.typVazbyK = ''typVazbyDokl.hlavaSkl''
      union
      select V.idVazby, V.typVazbyK, D.idDoklInt as idDokl, D.modul, ''modulUcetni.''||D.Modul as ModulK, D.kod, D.varSym,
        D.datVyst, D.datUcto, D.sumCelkem, D.sumCelkemMen, D.popis, D.poznam, D.idTypDokl, D.idMeny, D.idUziv, D.stavUzivK as stavK
        from dVazebTab as V left outer join dDoklInt as D on V.idTabA=D.idDoklInt and V.modulA=D.modul
        where V.idTabB=' || _idDokl || ' and V.modulB=''' || _modul || ''' and
          V.modulA in (''BAN'',''POK'',''INT'') and V.typVazbyK = ''typVazbyDokl.hlavaSkl''
      union
      select V.idVazby, V.typVazbyK, D.idDoklSklad as idDokl, D.modul, ''modulUcetni.''||D.Modul as ModulK, D.kod, D.varSym,
        D.datVyst, D.datUcto, D.sumCelkem, D.sumCelkemMen, D.popis, D.poznam, D.idTypDokl, D.idMeny, D.idUziv, D.stavUzivK as stavK
        from dVazebTab as V left outer join dDoklSklad as D on V.idTabA=D.idDoklSklad and V.modulA=D.modul
        where V.idTabB=' || _idDokl || ' and V.modulB=''' || _modul || ''' and
          V.modulA in (''SKL'') and V.typVazbyK = ''typVazbyDokl.hlavaSklKus''
      union
      select V.idVazby, V.typVazbyK, D.idDoklSklad as idDokl, D.modul, ''modulUcetni.''||D.Modul as ModulK, D.kod, D.varSym,
        D.datVyst, D.datUcto, D.sumCelkem, D.sumCelkemMen, D.popis, D.poznam, D.idTypDokl, D.idMeny, D.idUziv, D.stavUzivK as stavK
        from dVazebTab as V left outer join dDoklSklad as D on V.idTabB=D.idDoklSklad and V.modulB=D.modul
        where V.idTabA=' || _idDokl || ' and V.modulA=''' || _modul || ''' and
          V.modulB in (''SKL'') and V.typVazbyK = ''typVazbyDokl.hlavaSklKus''
      union
      select V.idVazby, V.typVazbyK, D.idDoklObch as idDokl, D.modul, ''modulUcetni.''||D.Modul as ModulK, D.kod, D.varSym,
        D.datVyst, D.datUcto, D.sumCelkem, D.sumCelkemMen, D.popis, D.poznam, D.idTypDokl, D.idMeny, D.idUziv, D.stavUzivK as stavK
        from dVazebTab as V left outer join dDoklObch as D on V.idTabA=D.idDoklObch and V.modulA=D.modul
        where V.idTabB=' || _idDokl || ' and V.modulB=''' || _modul || ''' and V.typVazbyK = ''typVazbyDokl.obchod_sklad_hla''';
  else if (tabulka = 'Obch') then
    if (_prechodVP = false and _modul in ('PPV', 'OBV')) then
      prechodPom = ' and D.modul in (''PPV'', ''NAP'', ''OBV'')';
    end if;

    dotaz =
      'select V.idVazby, V.typVazbyK, D.idDoklObch as idDokl, D.modul, ''modulUcetni.''||D.Modul as ModulK, D.kod, D.varSym,
        D.datVyst, D.datUcto, D.sumCelkem, D.sumCelkemMen, D.popis, D.poznam, D.idTypDokl, D.idMeny, D.idUziv, D.stavUzivK as stavK
        from dVazebTab as V left outer join dDoklObch as D on V.idTabA=D.idDoklObch and V.modulA=D.modul
        where V.idTabB=' || _idDokl || ' and V.modulB=''' || _modul || ''' and V.typVazbyK = ''typVazbyDokl.obchod_obchod_hla''' || prechodPom || '
      union
      select V.idVazby, V.typVazbyK, D.idDoklObch as idDokl, D.modul, ''modulUcetni.''||D.Modul as ModulK, D.kod, D.varSym,
        D.datVyst, D.datUcto, D.sumCelkem, D.sumCelkemMen, D.popis, D.poznam, D.idTypDokl, D.idMeny, D.idUziv, D.stavUzivK as stavK
        from dVazebTab as V left outer join dDoklObch as D on V.idTabB=D.idDoklObch and V.modulB=D.modul
        where V.idTabA=' || _idDokl || ' and V.modulA=''' || _modul || ''' and V.typVazbyK = ''typVazbyDokl.obchod_obchod_hla''' || prechodPom || '
      union
      select V.idVazby, V.typVazbyK, D.idDoklFak as idDokl, D.modul, ''modulUcetni.''||D.Modul as ModulK, D.kod, D.varSym,
        D.datVyst, D.datUcto, D.sumCelkem, D.sumCelkemMen, D.popis, D.poznam, D.idTypDokl, D.idMeny, D.idUziv, D.stavUhrK as stavK
        from dVazebTab as V left outer join dDoklFak as D on V.idTabB=D.idDoklFak and V.modulB=D.modul
        where V.idTabA=' || _idDokl || ' and V.modulA=''' || _modul || ''' and V.typVazbyK in (''typVazbyDokl.obchod_faktura_hla'', ''typVazbyDokl.obchod_zaloha_hla'')
      union
      select V.idVazby, V.typVazbyK, D.idDoklSklad as idDokl, D.modul, ''modulUcetni.''||D.Modul as ModulK, D.kod, D.varSym,
        D.datVyst, D.datUcto, D.sumCelkem, D.sumCelkemMen, D.popis, D.poznam, D.idTypDokl, D.idMeny, D.idUziv, D.stavUzivK as stavK
        from dVazebTab as V left outer join dDoklSklad as D on V.idTabB=D.idDoklSklad and V.modulB=D.modul
        where V.idTabA=' || _idDokl || ' and V.modulA=''' || _modul || ''' and V.typVazbyK = ''typVazbyDokl.obchod_sklad_hla''';
  else
    raise exception 'findVazbyDokladu(): Neznámá tabulka % pro modul %', tabulka, _modul;
  end if;
  end if;
  end if;
  end if;

  end if;

  execute 'insert into ' || _insertTab || ' (idvazebnidoklad, typVazbyK, idDokl, modul, modulk, kod, varsym, datvyst, datucto, sumcelkem,
  sumcelkemmen, popis, poznam, idtypdokl,idmeny, iduziv, stavk, uroven, novy) select *, ' || _uroven || ', true from ('
     || dotaz || ') as UN where (select count(*) from ' || _insertTab || ' as TV where
     (TV.idDokl = UN.idDokl and TV.modul = UN.modul) or UN.idVazby = TV.idvazebnidoklad) = 0';
end;
$BODY$
  LANGUAGE 'plpgsql' VOLATILE
  COST 100;
ALTER FUNCTION findvicevazeb(character varying, integer, character varying, integer, boolean) OWNER TO %firmaRole%;

CREATE OR REPLACE FUNCTION dpolskladdeletesklad()
  RETURNS trigger AS
$BODY$
declare
  _idPolDokl integer;
  _idDoklad integer;
  _idDoklSklad integer;
begin
  if(OLD.idPolSklad is not null) then
    execute deleteNavazanePolSklad(OLD.idPolSklad, OLD.modul);
  end if;

  select findParovanyDoklSklad(OLD.idDoklSklad, OLD.modul, 'typVazbyDokl.hlavaSklKus') into _idDoklSklad;
  if(_idDoklSklad is not null) then
    execute sprepoctiSumyDokladu(_idDoklSklad);
  end if;

  return OLD;
end;
$BODY$
  LANGUAGE 'plpgsql' VOLATILE
  COST 100;
ALTER FUNCTION dpolskladdeletesklad() OWNER TO %firmaRole%;

CREATE TRIGGER dpolskladdeletesklad BEFORE DELETE
  ON dpolsklad FOR EACH ROW EXECUTE PROCEDURE dpolskladdeletesklad();

-- zvyseni verze databaze
insert into dbversion(iddbversion, dbversion, needreimport)
  values (nextval('seq_dbversion'), 193, 't');