--------------------------------------------------------------------------------
-- Bug 17871 – [WinStrom] 26. Proplacení pracovní neschopnosti až podle nastavení
--------------------------------------------------------------------------------

-- rozsireni tabulky slozek
ALTER TABLE pmzdsloz DISABLE TRIGGER pmzdslozupdatedatumvypoctu;

ALTER TABLE pmzdsloz ADD COLUMN datumdoklad date;
ALTER TABLE pmzdsloz ADD COLUMN mesicplatba integer;
ALTER TABLE pmzdsloz ADD COLUMN rokplatba integer;

UPDATE pmzdsloz SET mesicplatba = mesic, rokplatba = rok WHERE idnepritomnost is not null;

ALTER TABLE pmzdsloz ENABLE TRIGGER pmzdslozupdatedatumvypoctu;


ALTER TABLE nnastav ADD COLUMN mzdnemocproplatauto boolean DEFAULT true;
ALTER TABLE nnastav ADD COLUMN mzdnemocproplatvarov boolean DEFAULT false;
ALTER TABLE nnastav ADD COLUMN mzdpromilepovpojisteni numeric(6,2) DEFAULT 0.0;

update nnastav set mzdnemocproplatauto = true, mzdnemocproplatvarov = false, mzdpromilepovpojisteni = 0.0;


-- pridani funkce na porovnani datumu pracujici spravne s null hodnotama
CREATE OR REPLACE FUNCTION comparedate(datea date, dateb date)
  RETURNS boolean AS $$   
begin   
  if (datea is null and dateb is null) or (datea is not null and dateb is not null and datea = dateb) then   
    return true;   
  end if;   
   
  return false;   
end;   
$$ LANGUAGE plpgsql;
ALTER FUNCTION comparedate(date, date) OWNER TO %firmaRole%;

-- uprava triggeru hlidajiciho zamykani a datum vypoctu
CREATE OR REPLACE FUNCTION pmzdslozupdatedatumvypoctu()
  RETURNS trigger AS $$
declare
  ZAZNAM record;
  _datumOd date;
  _datumDo date;
  _zamku integer;
  vypocet boolean;
  datDokl boolean;
begin
  if (tg_op = 'UPDATE') then
    if (OLD.idstalamzdsloz is not null and NEW.idstalamzdsloz is null) then     -- mazame stalou mzdovou slozku - to je ok
      return NEW;
    end if;
    if (OLD.idsrazka is not null and NEW.idsrazka is null) then     -- mazame srazku - to je ok
      return NEW;
    end if;
  end if;

  if (tg_op = 'DELETE') then
    ZAZNAM := OLD;  
  else  
    ZAZNAM := NEW;  
  end if;
  
  _datumOd = date (ZAZNAM.rok || '-' || ZAZNAM.mesic || '-01');
  _datumDo = _datumOd + interval '1 month';
  
  select count(*) into _zamku from wzamky where modulmzd = true and zamekk in ('zamek.polozamceno', 'zamek.zamceno') and
      platioddata < _datumDo and platidodata >= _datumOd;
  if (_zamku > 0) then
    raise exception 'Mzdy v období %/% jsou zamčené a není možno je měnit.', ZAZNAM.rok, ZAZNAM.mesic;
  end if;
  
  -- pokud se jedna o update resime cachry s datumava (vypocet, nastaveni datumu dokladu a jeho proplaceni)
  if (tg_op = 'UPDATE') then
    -- zjistime zda nastavujeme datum vypoctu
    vypocet = (NEW.typslozkyk = 'typSlozky.kopie' and NEW.skupslozkyk = 'skupinaSlozky.doba' and not comparedate(NEW.datumod, OLD.datumod));
    -- zjisitme zda se meni datum dokladu nebo obdobi platby
    datDokl = (NEW.idnepritomnost is not null and (not comparedate(NEW.datumdoklad, OLD.datumdoklad) or 
                  not compareid(NEW.rokplatba, OLD.rokplatba) or not compareid(NEW.mesicplatba, OLD.mesicplatba)));
  
    if (not vypocet and not datDokl) then
      update pmzdsloz set datumod = null where rok = ZAZNAM.rok and mesic = ZAZNAM.mesic and idosoba = ZAZNAM.idosoba and 
          typslozkyk = 'typSlozky.kopie' and skupslozkyk = 'skupinaSlozky.doba' and datumod is not null;
    end if;
    
    if (datDokl) then
      update pmzdsloz set datumdoklad = NEW.datumdoklad, rokplatba = NEW.rokplatba, mesicplatba = NEW.mesicplatba 
        where idnepritomnost = NEW.idnepritomnost and rok = NEW.rok and mesic = NEW.mesic and idmzdsloz <> NEW.idmzdsloz;
    end if;
  end if;
  
  return ZAZNAM;
end;
$$ LANGUAGE plpgsql;
ALTER FUNCTION public.pmzdslozupdatedatumvypoctu() OWNER TO %firmaRole%;



--------------------------------------------------------------------------------
-- Bug 18697 – [WinStrom] Jelínek - DE - Střediska se z dokladu přenášejí z hlavičky - ne z položek
--------------------------------------------------------------------------------

ALTER TABLE udanevid ADD COLUMN idzakazky integer;
ALTER TABLE udanevid ADD COLUMN idstred integer;

ALTER TABLE udanevid ADD CONSTRAINT fk32c69288f0c53e9f FOREIGN KEY (idstred)
  REFERENCES ustrediska (idstred) MATCH SIMPLE ON UPDATE NO ACTION ON DELETE NO ACTION;
ALTER TABLE udanevid ADD CONSTRAINT fk32c69288fa994e2f FOREIGN KEY (idzakazky)
  REFERENCES uzakazky (idzakazky) MATCH SIMPLE ON UPDATE NO ACTION ON DELETE NO ACTION;

CREATE INDEX uDanEvid_IdStred_index ON uDanEvid USING btree (IdStred);
CREATE INDEX uDanEvid_IdZakazky_index ON uDanEvid USING btree (IdZakazky);

create or replace function uUpdateDanEvid (in _idDoklInt integer) returns void as $$
declare
  doklInt record;
  tempRec record;
  podvoj boolean;
  navazaneDotaz varchar;
  vazeb integer;
  _danEvidK character varying(50);
  _idTypUcOp integer;
  _idStred integer;
  _idZakazky integer;
  nespadaDph character varying(50);
  _idDanEvid integer;
  _idDoklFak integer;
  castka numeric(15,2);
  castkaMen numeric(15,2);
  castkaZbyva numeric(15,2);
  castkaZbyvaMen numeric(15,2);
  castkaOdpocet numeric(15,2);
  castkaOdpocetMen numeric(15,2);
  zapocteno numeric(15,2);
  zapoctenoMen numeric(15,2);
  domaciMena integer;
  ciziMena boolean;
begin
  nespadaDph := 'danEvid.NeDanEvid';

  select * into doklInt from ddoklint where iddoklint = _idDoklInt;
  select podvojucto, idmeny into podvoj, domaciMena from nnastav where (platioddata is null or platioddata <= doklInt.datUcto) 
    order by coalesce(platioddata, '0001-01-01') desc limit 1;
  
  ciziMena := (doklInt.idMeny is not null and domaciMena <> doklInt.idMeny);
  
  if (doklInt.modul not in ('BAN', 'POK')) then
    return;
  end if;

  if (podvoj = false and doklInt.ucetni = true) then
    -- dotaz ktery nalezne veschny navazane faktury a navazane castky
    navazaneDotaz := '(select V.idDoklFak, sum(V.castka) as castka from
         (select idTabA as idDoklFak, castka, 1 as unionId from dvazebtab where typvazbyk = ''typVazbyDokl.uhrada'' and idtabb = ' || doklInt.idDoklInt || '
          union
          select P.idDoklFak, V.castka, 2 as unionId from dpolfak as P join dVazebTab as V on 
            (V.typvazbyk = ''typVazbyDokl.uhradaZALOHY'' and V.idTabB = ' || doklInt.idDoklInt || ' and V.idTabA in 
              (select idTabB from dVazebTab where typVazbyK = ''typVazbyDokl.odpocetZALOHY'' and idTabA = P.idPolFak))
          union
          select P.idDoklFak, V.castka, 3 as unionId from dpolfak as P join dVazebTab as V on 
            (V.typvazbyk = ''typVazbyDokl.vazbaZDD'' and V.idTabB = ' || doklInt.idDoklInt || ' and V.idTabA in 
              (select idTabB from dVazebTab where typVazbyK = ''typVazbyDokl.odpocetZDD'' and idTabA = P.idPolFak))
          ) as V group by V.idDoklFak)';
    
    execute 'select count(*) from ' || navazaneDotaz || ' as D' into vazeb;
  
    if (vazeb = 0) then			-- pro interni doklad neni vazba
      DELETE from uDanEvid where idDoklInt = doklInt.idDoklInt and idDoklFak is not null;           -- vymazeme ty co byli vytvorene pres vazbu
      UPDATE uDanEvid set sumZkl = 0, sumDph = 0, sumZklMen = 0, sumDphMen = 0 where idDoklInt = doklInt.idDoklInt;     -- vynulujeme ostatni
             
      -- DPH
      select idDanEvid into _idDanEvid from uDanEvid where danEvidK = nespadaDph and idDoklInt = doklInt.idDoklInt;  -- zjistime zda uz zaznam existuje
      select coalesce(sum(sumDph),0),coalesce(sum(sumDphMen),0) into castka,castkaMen from dPolInt 
        where idDoklInt = doklInt.idDoklInt and storno = false;      -- secteme dane
      
      if (_idDanEvid is null) then              -- zaznam jeste nebyl vytvoren
        INSERT into uDanEvid (danEvidK, sumDph, sumDphMen, kurz, kurzMnozstvi, idDoklInt, idMeny) 
          VALUES (nespadaDph, castka, castkaMen, doklInt.kurz, doklInt.kurzmnozstvi, doklInt.idDoklInt, doklInt.idMeny);
      else                                     -- zaznam jiz byl vytvoren, vynulujeme i zaklad pokud by jiz nebyl pouzit
        UPDATE uDanEvid SET sumDph = castka, sumDphMen = castkaMen, sumZkl = 0, sumZklMen = 0, kurz = doklInt.kurz, 
          kurzMnozstvi = doklInt.kurzmnozstvi, idMeny = doklInt.idMeny, idDoklFak = null WHERE idDanEvid = _idDanEvid;
      end if;
    
      -- Zaklad
      for _danEvidK, _idTypUcOp, _idStred, _idZakazky, castka, castkaMen in select danEvidK, idTypUcOp, idStred, idZakazky, sum(sumZkl), sum(sumZklMen) 
                from dPolInt where idDoklInt = doklInt.idDoklInt and storno = false group by danEvidK, idTypUcOp, idStred, idZakazky loop 
        select idDanEvid into _idDanEvid from uDanEvid where danEvidK = _danEvidK and idDoklInt = doklInt.idDoklInt
          and compareid(_idTypUcOp, idTypUcOp) and compareid(_idStred, idStred) and compareid(_idZakazky, idZakazky);  -- zjistime zda uz zaznam existuje
        
        if (_idDanEvid is null) then              -- zaznam jeste nebyl vytvoren
          INSERT into uDanEvid (danEvidK, idTypUcOp, idStred, idZakazky, sumZkl, sumZklMen, kurz, kurzMnozstvi, idDoklInt, idMeny) 
            VALUES (_danEvidK, _idTypUcOp, _idStred, _idZakazky, castka, castkaMen, doklInt.kurz, doklInt.kurzmnozstvi, doklInt.idDoklInt, doklInt.idMeny);
        else                                     -- zaznam jiz byl vytvoren
          UPDATE uDanEvid SET sumZkl = castka, sumZklMen = castkaMen, kurz = doklInt.kurz, kurzMnozstvi = doklInt.kurzmnozstvi,
            idMeny = doklInt.idMeny, idDoklFak = null WHERE idDanEvid = _idDanEvid;
        end if;
      end loop; 
    else				            -- pro interni doklad je vazba
      DELETE from uDanEvid where idDoklInt = doklInt.idDoklInt and idDoklFak is null;           -- smazeme polozky co nemaji vazbu na fakturu
      UPDATE uDanEvid set sumZkl = 0, sumDph = 0, sumZklMen = 0, sumDphMen = 0 where idDoklInt = doklInt.idDoklInt;     -- vynulujeme ostatni
      
      drop table if exists tempDanEvid;
      
      CREATE TEMPORARY TABLE tempDanEvid
        (idDoklFak integer NOT NULL,
         danEvidK character varying(50),
         idTypUcOp integer,
         idStred integer,
         idZakazky integer,
         dan boolean DEFAULT false,
         kUhrade numeric(15,2) DEFAULT 0.0 NOT NULL,
         kUhradeMen numeric(15,2) DEFAULT 0.0 NOT NULL,
         uhrazeno numeric(15,2) DEFAULT 0.0 NOT NULL,
         uhrazenoMen numeric(15,2) DEFAULT 0.0 NOT NULL,
         vazbaFak numeric(15,2) DEFAULT 0.0 NOT NULL
        );

      for tempRec in execute 'select * from ' || navazaneDotaz || ' as D' loop
        _idDoklFak = tempRec.idDoklFak;
        castka = tempRec.castka;
      
        select * into tempRec from dDoklFak where idDoklFak = _idDoklFak;
        if ((ciziMena = true and (tempRec.idMeny is null or tempRec.idMeny <> doklInt.idMeny)) 
         or (ciziMena = false and tempRec.idMeny is not null and tempRec.idMeny <> domaciMena)) then
          raise exception 'uUpdateDanEvid: Nesouhlasi meny dokladu';
        end if;
      
        -- zjistime kolik je treba na fakture uhradit a kolik je uhrazeno
        insert into tempDanEvid (idDoklFak, danEvidK, idTypUcOp, idStred, idZakazky, vazbaFak, kUhrade, kUhradeMen, uhrazeno, uhrazenoMen) 
          select _idDoklFak, danEvidK, idTypUcOp, idStred, idZakazky, castka, sum(sumZkl), sum(sumZklMen), 
               (select coalesce(sum(sumZkl),0) from uDanEvid where idDoklFak = _idDoklFak and danEvidK = P.danEvidK and 
                 compareid(P.idTypUcOp, idTypUcOp) and compareid(P.idStred, idStred) and compareid(P.idZakazky, idZakazky)),
               (select coalesce(sum(sumZklMen),0) from uDanEvid where idDoklFak = _idDoklFak and danEvidK = P.danEvidK and
                 compareid(P.idTypUcOp, idTypUcOp) and compareid(P.idStred, idStred) and compareid(P.idZakazky, idZakazky))
           from dPolFak as P where idDoklFak = _idDoklFak and storno = false
             and typPolozkyK not in ('typPolozky.odpocetZdd', 'typPolozky.odpocetZalohy') group by danEvidK, idTypUcOp, idStred, idZakazky;
           
        -- dan
        insert into tempDanEvid (idDoklFak, danEvidK, dan, vazbaFak, kUhrade, kUhradeMen, uhrazeno, uhrazenoMen) values (_idDoklFak, null, true, castka,   -- dan si oznacime jako null
           (select coalesce(sum(sumDph),0) from dPolFak where idDoklFak = _idDoklFak and storno = false 
              and typPolozkyK not in ('typPolozky.odpocetZdd', 'typPolozky.odpocetZalohy')),     -- castka dane dokladu
           (select coalesce(sum(sumDphMen),0) from dPolFak where idDoklFak = _idDoklFak and storno = false
              and typPolozkyK not in ('typPolozky.odpocetZdd', 'typPolozky.odpocetZalohy')),  -- castka dane dokladu v mene
           (select coalesce(sum(sumDph),0) from uDanEvid where idDoklFak = _idDoklFak),                             -- uhrazena dan dokladu
           (select coalesce(sum(sumDphMen),0) from uDanEvid where idDoklFak = _idDoklFak)                           -- uhrazena dan dokladu v mene
          );
      end loop;
      
      zapocteno := 0;
      zapoctenoMen := 0;
      
      for _idDoklFak,castka in select idDoklFak,vazbaFak from tempDanEvid group by idDoklFak,vazbaFak loop           -- prochazime navazene faktury
        castkaMen := castka;

        for tempRec in select * from tempDanEvid where idDoklFak = _idDoklFak order by (case when (kUhrade-uhrazeno) < 0 then -1 else 1 end), -- napred zaporny 
                                                                                              dan desc, danEvidK desc loop    -- sestupne aby null (dan) bylo prvni
        
          if ((ciziMena = false and castka <> 0) or (ciziMena = true and castkaMen <> 0)) then                -- pokud jeste mame nejakou castku
            castkaZbyva := tempRec.kUhrade - tempRec.uhrazeno;
            castkaZbyvaMen := tempRec.kUhradeMen - tempRec.uhrazenoMen;
            
            if ((ciziMena = false and castkaZbyva <> 0) or (ciziMena = true and castkaZbyvaMen <> 0)) then    -- pokud na aktualnim zaznamu zbyva neco uhradit
              if ((ciziMena = false and castka > 0 and castkaZbyva > 0) or (ciziMena = true and castkaMen > 0 and castkaZbyvaMen > 0)) then
                castkaOdpocet := minValue(castka, castkaZbyva);      -- spocteme kolik budeme "odpocitavat"
                castkaOdpocetMen := minValue(castkaMen, castkaZbyvaMen);
              else
                if ((ciziMena = false and castka < 0 and castkaZbyva < 0) or (ciziMena = true and castkaMen < 0 and castkaZbyvaMen < 0)) then
                  castkaOdpocet := maxValue(castka, castkaZbyva);
                  castkaOdpocetMen := maxValue(castkaMen, castkaZbyvaMen);
                else
                  castkaOdpocet := castkaZbyva;
                  castkaOdpocetMen := castkaZbyvaMen;
                end if;
              end if;
              
              castka := castka - castkaOdpocet;                    -- odebereme to z odpocitatelne castky
              castkaMen := castkaMen - castkaOdpocetMen;
              zapocteno := zapocteno + castkaOdpocet;
              zapoctenoMen := zapoctenoMen + castkaOdpocetMen;
              
              if (ciziMena = true) then
                castkaOdpocet := castkaOdpocetMen * doklInt.kurz / doklInt.kurzmnozstvi;
              else
                castkaOdpocetMen := 0;
              end if;
              
              if (tempRec.dan) then        -- pokud se jedna o dan
                select idDanEvid into _idDanEvid from uDanEvid where idDoklInt = doklInt.idDoklInt and idDoklFak = tempRec.idDoklFak and danEvidK = nespadaDph;  -- zjistime zda uz zaznam existuje
                
                if (_idDanEvid is null) then            -- zaznam jeste nebyl vytvoren
                  INSERT into uDanEvid (danEvidK, sumDph, sumDphMen, kurz, kurzMnozstvi, idDoklInt, idDoklFak, idMeny) 
                    VALUES (nespadaDph, castkaOdpocet, castkaOdpocetMen, doklInt.kurz, doklInt.kurzmnozstvi, doklInt.idDoklInt, tempRec.idDoklFak, doklInt.idMeny);
                else                                    -- zaznam jiz byl vytvoren
                  UPDATE uDanEvid SET sumDph = castkaOdpocet, sumDphMen = castkaOdpocetMen, 
                    kurz = doklInt.kurz, kurzMnozstvi = doklInt.kurzmnozstvi, idMeny = doklInt.idMeny WHERE idDanEvid = _idDanEvid;
                end if;
              else                                      -- pokud se nejedna o dan
                select idDanEvid into _idDanEvid from uDanEvid where idDoklInt = doklInt.idDoklInt and idDoklFak = tempRec.idDoklFak 
                  and ((tempRec.danEvidK is null and danEvidK is null) or (danEvidK = tempRec.danEvidK))
                  and compareid(tempRec.idTypUcOp, idTypUcOp) and compareid(tempRec.idStred, idStred) and compareid(tempRec.idZakazky, idZakazky);  -- zjistime zda uz zaznam existuje
                
                if (_idDanEvid is null) then            -- zaznam jeste nebyl vytvoren
                  INSERT into uDanEvid (danEvidK, idTypUcOp, idStred, idZakazky, sumZkl, sumZklMen, kurz, kurzMnozstvi, idDoklInt, idDoklFak, idMeny) 
                    VALUES (tempRec.danEvidK, tempRec.idTypUcOp, tempRec.idStred, tempRec.idZakazky, castkaOdpocet, castkaOdpocetMen, doklInt.kurz, doklInt.kurzmnozstvi, doklInt.idDoklInt, tempRec.idDoklFak, doklInt.idMeny);
                else                                    -- zaznam jiz byl vytvoren
                  UPDATE uDanEvid SET sumZkl = castkaOdpocet, sumZklMen = castkaOdpocetMen, 
                    kurz = doklInt.kurz, kurzMnozstvi = doklInt.kurzmnozstvi, idMeny = doklInt.idMeny WHERE idDanEvid = _idDanEvid;
                end if;
              end if;
            
              --select idDanEvid into _idDanEvid from uDanEvid where idDoklInt = doklInt.idDoklInt and idDoklFak = tempRec.idDoklFak and
              --   ((tempRec.danEvidK is not null and danEvidK = tempRec.danEvidK) or (tempRec.danEvidK is null and danEvidK = nespadaDph));  -- zjistime zda uz zaznam existuje
            end if;
          end if;
        
        end loop;
      end loop;
      
      if ((ciziMena = false and zapocteno < doklInt.sumCelkem) or (ciziMena = true and zapoctenoMen < doklInt.sumCelkemMen)) then     -- pokud neni komplet sparovan
        castka = doklInt.sumCelkem - zapocteno;
        castkaMen = doklInt.sumCelkemMen - zapoctenoMen;
        if (ciziMena = true) then
          castka := castkaMen * doklInt.kurz / doklInt.kurzmnozstvi;
        else
          castkaMen := 0;
        end if;
        
        -- vybereme typ pro ktery je nejvetsi castka, pokud takovy neni nechame castku viset, protoze se jedna o celou castku
        select danEvidK, idTypUcOp, idStred, idZakazky into _danEvidK, _idTypUcOp, _idStred, _idZakazky from uDanEvid 
          where idDoklInt = doklInt.idDoklInt order by sumZkl desc limit 1;
        if _danEvidK is not null then
          INSERT into uDanEvid (danEvidK, idTypUcOp, idStred, idZakazky, sumZkl, sumZklMen, kurz, kurzMnozstvi, idDoklInt, idMeny) 
            VALUES (_danEvidK, _idTypUcOp, _idStred, _idZakazky, castka, castkaMen, doklInt.kurz, doklInt.kurzmnozstvi, doklInt.idDoklInt, doklInt.idMeny);
        end if;
        
      end if;
      
    end if;
  end if;
  
  DELETE from uDanEvid WHERE idDoklInt = doklInt.idDoklInt and sumZkl = 0 and sumZklMen = 0 and sumDph = 0 and sumDphMen = 0;
end;
$$ LANGUAGE plpgsql;
ALTER FUNCTION public.uUpdateDanEvid(integer) OWNER TO %firmaRole%;


create or replace function runupgrade()
    returns void as $$
declare
    _podvojUcto boolean;
begin
  select podvojucto into _podvojUcto from nnastav
    where ((select max(platioddata) from nnastav) is null or (select max(platioddata) from nnastav) = platioddata);
  
  -- prepocet danove evidence provedeme jen pro DE
  if (_podvojUcto = false) then
    perform uUpdateDanEvid(idDoklInt) from dDoklInt;
  end if; 
end;
$$ LANGUAGE plpgsql;
ALTER FUNCTION runupgrade() OWNER TO %firmaRole%;

select runupgrade();

drop function runupgrade();

------------------------------------------------------------------------------------------------
-- Bug 18727 – [WinStrom] Vystaveni faktury z prijemky s vyrobnimi cisly zduplikuje onu prijemku
------------------------------------------------------------------------------------------------

-- Function: dpoldoklupdatesklad()
-- DROP FUNCTION dpoldoklupdatesklad();
CREATE OR REPLACE FUNCTION dpoldoklupdatesklad()
  RETURNS trigger AS
$BODY$ 
declare 
  _idPolDokl integer; 
  _idDoklad integer; 
  _idPolSklad integer; 
  _idDoklSklad integer; 
  _dokladGenerovatSkl boolean; 
  _typDoklGenerovatSkl boolean; 
  _typZasobyK character varying; 
  _vyrobniCislo record; 
  _typPohybuK character varying; 
  _sadaItemCount integer; 
begin 
  if(NEW.storno <> OLD.storno or NEW.stornoPol <> OLD.stornoPol) then 
    return NEW; 
  end if; 
 
  if(NEW.modul like 'POK') then 
    _idPolDokl = NEW.idPolInt; 
    _idDoklad = NEW.idDoklInt; 
    select generovatSkl into _dokladGenerovatSkl from dDoklInt where idDoklInt = NEW.idDoklInt; 
    select generovatSkl into _typDoklGenerovatSkl from dTypDokl where idTypDokl = (select idTypDokl from dDoklInt where idDoklInt = NEW.idDoklInt); 
 
    select count(*) into _sadaItemCount from csady where idceniksada = (select idcenik from dpolInt where idpolint = NEW.idPolInt); 
  elsif (NEW.modul like 'FAV' or NEW.modul like 'FAP') then 
    _idPolDokl = NEW.idPolFak; 
    _idDoklad = NEW.idDoklFak; 
    select generovatSkl into _dokladGenerovatSkl from dDoklFak where idDoklFak = NEW.idDoklFak; 
    select generovatSkl into _typDoklGenerovatSkl from dTypDokl where idTypDokl = (select idTypDokl from dDoklFak where idDoklFak = NEW.idDoklFak); 
 
    select count(*) into _sadaItemCount from csady where idceniksada = (select idcenik from dpolfak where idpolfak = NEW.idPolFak); 
  else  
    return NEW;   
  end if; 
 
  -- pokud se lisi sklad, cenik, typPolozky, mnozMj, sarze, expirace 
  -- tak skladovou polozku musim smazat a vytvorit znovu kvuli fifu 
  if((NEW.idSkladu <> OLD.idSkladu or NEW.idCenik <> OLD.idCenik  
     or NEW.typPolozkyK <> OLD.typPolozkyK or NEW.mnozMj <> OLD.mnozMj 
     or NEW.sarze <> OLD.sarze or NEW.expirace <> OLD.expirace )
     and _dokladGenerovatSkl) then 
    --raise exception 'mazu polSklad'; 
    execute deleteNavazanePolSklad(_idPolDokl, NEW.modul); 
  end if; 
 
  if(NEW.typPolozkyK like 'typPolozky.katalog' and _dokladGenerovatSkl and _typDoklGenerovatSkl) then 
    select typZasobyK into _typZasobyK from cCenik where idCenik = NEW.idCenik; 
     
    if (_typZasobyK like 'typZasoby.skladem' or _typZasobyK like 'typZasoby.vyrobek') then 
      select findParovanyPolSklad(_idPolDokl, NEW.modul) into _idPolSklad; 
      if(_idPolSklad is null) then 
        --select createPolSklad(_idPolDokl, NEW.modul) into _idPolSklad;  
        if(_sadaItemCount > 0) then 
          execute createPolSkladSada(_idPolDokl, NEW.modul); 
        else 
          select createPolSklad(_idPolDokl, NEW.modul) into _idPolSklad;  
        end if; 
      else 
        if(_sadaItemCount > 0) then 
          execute fillPolSkladSada(_idPolDokl, NEW.modul); 
        else 
          select fillPolSklad(_idPolSklad, _idPolDokl, NEW.modul) into _idPolSklad;  
        end if; 
      end if; 
      -- provedeni upgradu vyrobnich cisel 
      if(NEW.modul like 'FAV') then 
        for _vyrobniCislo in select * from sVyrobniCislo where idPolFakVydej = NEW.idPolFak loop 
          update sVyrobniCislo set idPolSkladVydej = _idPolSklad where idVyrobniCislo = _vyrobniCislo.idVyrobniCislo; 
        end loop; 
      end if; 
      if(NEW.modul like 'FAP') then 
        for _vyrobniCislo in select * from sVyrobniCislo where idPolFakPrijem = NEW.idPolFak loop 
          update sVyrobniCislo set idPolSkladPrijem = _idPolSklad where idVyrobniCislo = _vyrobniCislo.idVyrobniCislo; 
        end loop; 
      end if; 
      if(NEW.modul like 'POK') then 
        select typPohybuK into _typPohybuK from dDoklInt where idDoklInt = _idDoklad; 
        if(_typPohybuK like 'typPohybu.prijem') then 
          for _vyrobniCislo in select * from sVyrobniCislo where idPolIntVydej = NEW.idPolInt loop 
            update sVyrobniCislo set idPolSkladVydej = _idPolSklad where idVyrobniCislo = _vyrobniCislo.idVyrobniCislo; 
          end loop; 
        else 
          for _vyrobniCislo in select * from sVyrobniCislo where idPolIntPrijem = NEW.idPolInt loop 
            update sVyrobniCislo set idPolSkladPrijem = _idPolSklad where idVyrobniCislo = _vyrobniCislo.idVyrobniCislo; 
          end loop; 
        end if; 
      end if; 
 
    end if; 
    -- prepocteni sum na dokladu 
    select findParovanyDoklSklad(_idDoklad, NEW.modul) into _idDoklSklad; 
    execute sprepoctiSumyDokladu(_idDoklSklad, NEW.datVyst); 
  end if; 
 
  return NEW; 
end 
$BODY$
  LANGUAGE 'plpgsql' VOLATILE
  COST 100;
ALTER FUNCTION dpoldoklupdatesklad() OWNER TO %firmaRole%;


--------------------------------------------------------------------------------
-- Bug 18728 – [WinStrom] Permission denied při rozpadu kusovníku
--------------------------------------------------------------------------------


CREATE OR REPLACE FUNCTION putglotxvariable(i_kod character varying, i_hodnota character varying)
  RETURNS void AS $$
DECLARE
  tmpGloTxVarTableExists boolean;
  pom integer;
BEGIN
  tmpGloTxVarTableExists := (SELECT istableexist('tmp_gloTxVar'));

  IF (tmpGloTxVarTableExists <> true) THEN
    CREATE TEMP TABLE tmp_gloTxVar (
      kod character varying(50),
      hodnota character varying(50)
    ) WITH (OIDS=FALSE) ON COMMIT DROP;
  END IF;

  SELECT count(1) INTO pom FROM tmp_gloTxVar WHERE kod=i_kod;
  if (pom > 0) then
    UPDATE tmp_gloTxVar SET hodnota=i_hodnota WHERE kod=i_kod;
  ELSE
    INSERT INTO tmp_gloTxVar VALUES(i_kod, i_hodnota);
  END IF;

END;
$$ LANGUAGE plpgsql SECURITY DEFINER;

ALTER FUNCTION putglotxvariable(i_kod character varying, i_hodnota character varying) OWNER TO %firmaRole%;

--------------------------------------------------------------------------------

CREATE OR REPLACE FUNCTION removeglotxvariable(i_kod character varying)
  RETURNS void AS $$
DECLARE
  tmpGloTxVarTableExists boolean;
BEGIN
  tmpGloTxVarTableExists := (SELECT istableexist('tmp_gloTxVar'));

  IF (tmpGloTxVarTableExists = true) THEN
    DELETE FROM tmp_gloTxVar WHERE kod=i_kod;
  END IF;

END;
$$ LANGUAGE plpgsql SECURITY DEFINER;

ALTER FUNCTION removeglotxvariable(i_kod character varying) OWNER TO %firmaRole%;

--------------------------------------------------------------------------------

CREATE OR REPLACE FUNCTION getglotxvariable(i_kod character varying)
  RETURNS character varying AS $$
DECLARE
  result varchar(50);
  tmpGloTxVarTableExists boolean;
BEGIN
  tmpGloTxVarTableExists := (SELECT istableexist('tmp_gloTxVar'));

  IF (tmpGloTxVarTableExists = true) THEN
    SELECT hodnota INTO result FROM tmp_gloTxVar WHERE kod=i_kod LIMIT 1;
  END IF;

  RETURN result;
END;
$$ LANGUAGE plpgsql SECURITY DEFINER;

ALTER FUNCTION getglotxvariable(i_kod character varying) OWNER TO %firmaRole%;



--------------------------------------------------------------------------------
-- Bug 18676 – [WinStrom] Při tvorbě rozvahy se přiřazuje ke spoustě účtů řádek, chybí účet 314
--------------------------------------------------------------------------------

create or replace function runupgrade()
    returns void as $$
declare
    _idtyporgread integer;
begin
  select idtyporg into _idtyporgread from nnastav limit 1;
  
  -- sestavy aktualizujeme jen pro cechy
  if (_idtyporgread = 1) then
-- zacatek bloku ceskych podnikatelu

UPDATE usesradky SET nazev = 'Dospělá zvířata a jejich skupiny' WHERE IdSesRadky = 15;
UPDATE usesradky SET nazev = 'Půjčky a úvěry-ovládající a řídící osoba, podstatný vliv' WHERE IdSesRadky = 23;
UPDATE usesradky SET oznaceni = 'A  B  III   7' WHERE IdSesRadky = 26;
UPDATE usesradky SET nazev = 'Mladá a ostatní zvířata a jejich skupiny' WHERE IdSesRadky = 30;
UPDATE usesradky SET nazev = 'Pohledávky-podstatný vliv' WHERE IdSesRadky = 33;
UPDATE usesradky SET nazev = 'Pohledávky ovládajícíi a řídícíi osoba' WHERE IdSesRadky = 36;
UPDATE usesradky SET nazev = 'Výsledek hospodaření běžného účet.období' WHERE IdSesRadky = 60;
UPDATE usesradky SET nazev = 'Rezervy podle zvláštních pravních předpisů' WHERE IdSesRadky = 61;
UPDATE usesradky SET nazev = 'Závazky-ovládající a řídící osoba' WHERE IdSesRadky = 64;
UPDATE usesradky SET nazev = 'Závazky-podstatný vliv' WHERE IdSesRadky = 65;
UPDATE usesradky SET nazev = 'Závazky-ovládající a řídící osoba' WHERE IdSesRadky = 69;
UPDATE usesradky SET nazev = 'Závazky-podstatný vliv' WHERE IdSesRadky = 70;
UPDATE usesradky SET nazev = 'Závazky ke společníkům, členům družstva a k účastníkům sdružení' WHERE IdSesRadky = 71;
UPDATE usesradky SET nazev = 'Stát-daňové závazky a dotace' WHERE IdSesRadky = 74;
UPDATE usesradky SET nazev = 'Pohledávky-ovládající a řídící osoba' WHERE IdSesRadky = 95;
UPDATE usesradky SET nazev = 'Krátkodobé poskytnuté zálohy' WHERE IdSesRadky = 100;
UPDATE usesradky SET nazev = 'Pohledávky podstatný vliv' WHERE IdSesRadky = 101;
UPDATE usesradky SET nazev = 'Závazky ke společníkům, členům družstva a k účastníkům sdružení' WHERE IdSesRadky = 120;
UPDATE usesradky SET nazev = 'Tržby z prodeje materiálu' WHERE IdSesRadky = 137;
UPDATE usesradky SET nazev = 'Prodaný materiál' WHERE IdSesRadky = 139;

UPDATE usesstdpred SET jenkladne = true WHERE IdSesStdPred = 123;
UPDATE usesstdpred SET jenkladne = true WHERE IdSesStdPred = 124;
UPDATE usesstdpred SET jenkladne = true WHERE IdSesStdPred = 165;
UPDATE usesstdpred SET jenkladne = true WHERE IdSesStdPred = 166;
UPDATE usesstdpred SET jenkladne = true WHERE IdSesStdPred = 375;
UPDATE usesstdpred SET jenkladne = true WHERE IdSesStdPred = 383;
INSERT INTO usesstdpred (idsesstdpred, lastUpdate, druhuctuk, cissloup, zpusobvypk, prevratznam, jenkladne, idsesradky, idstducet)
 VALUES (386, '2010-03-15 00:00:00.000', 'druhUctu.aktivni', 1, 'zpusobVypo.stavKon', false, false, 94, (select IdStdUcet from uStdUcty where uuid = 'CZ_POD_390'));
INSERT INTO usesstdpred (idsesstdpred, lastUpdate, druhuctuk, cissloup, zpusobvypk, prevratznam, jenkladne, idsesradky, idstducet)
 VALUES (387, '2010-03-15 00:00:00.000', 'druhUctu.aktivni', 1, 'zpusobVypo.stavKon', false, false, 94, (select IdStdUcet from uStdUcty where uuid = 'CZ_POD_391'));
INSERT INTO usesstdpred (idsesstdpred, lastUpdate, druhuctuk, cissloup, zpusobvypk, prevratznam, jenkladne, idsesradky, idstducet)
 VALUES (388, '2010-03-15 00:00:00.000', 'druhUctu.aktivni', 1, 'zpusobVypo.stavKon', false, false, 95, (select IdStdUcet from uStdUcty where uuid = 'CZ_POD_350'));
INSERT INTO usesstdpred (idsesstdpred, lastUpdate, druhuctuk, cissloup, zpusobvypk, prevratznam, jenkladne, idsesradky, idstducet)
 VALUES (389, '2010-03-15 00:00:00.000', 'druhUctu.aktivni', 1, 'zpusobVypo.stavKon', false, false, 95, (select IdStdUcet from uStdUcty where uuid = 'CZ_POD_351'));
INSERT INTO usesstdpred (idsesstdpred, lastUpdate, druhuctuk, cissloup, zpusobvypk, prevratznam, jenkladne, idsesradky, idstducet)
 VALUES (390, '2010-03-15 00:00:00.000', 'druhUctu.aktivni', 1, 'zpusobVypo.stavKon', false, false, 95, (select IdStdUcet from uStdUcty where uuid = 'CZ_POD_391'));
INSERT INTO usesstdpred (idsesstdpred, lastUpdate, druhuctuk, cissloup, zpusobvypk, prevratznam, jenkladne, idsesradky, idstducet)
 VALUES (391, '2010-03-15 00:00:00.000', 'druhUctu.aktivni', 1, 'zpusobVypo.stavKon', false, false, 94, (select IdStdUcet from uStdUcty where uuid = 'CZ_POD_310'));
INSERT INTO usesstdpred (idsesstdpred, lastUpdate, druhuctuk, cissloup, zpusobvypk, prevratznam, jenkladne, idsesradky, idstducet)
 VALUES (392, '2010-03-15 00:00:00.000', 'druhUctu.aktivni', 1, 'zpusobVypo.stavKon', false, false, 33, (select IdStdUcet from uStdUcty where uuid = 'CZ_POD_391'));
INSERT INTO usesstdpred (idsesstdpred, lastUpdate, druhuctuk, cissloup, zpusobvypk, prevratznam, jenkladne, idsesradky, idstducet)
 VALUES (393, '2010-03-15 00:00:00.000', 'druhUctu.aktivni', 1, 'zpusobVypo.stavKon', false, false, 96, (select IdStdUcet from uStdUcty where uuid = 'CZ_POD_354'));
INSERT INTO usesstdpred (idsesstdpred, lastUpdate, druhuctuk, cissloup, zpusobvypk, prevratznam, jenkladne, idsesradky, idstducet)
 VALUES (394, '2010-03-15 00:00:00.000', 'druhUctu.aktivni', 1, 'zpusobVypo.stavKon', false, false, 96, (select IdStdUcet from uStdUcty where uuid = 'CZ_POD_355'));
INSERT INTO usesstdpred (idsesstdpred, lastUpdate, druhuctuk, cissloup, zpusobvypk, prevratznam, jenkladne, idsesradky, idstducet)
 VALUES (395, '2010-03-15 00:00:00.000', 'druhUctu.aktivni', 1, 'zpusobVypo.stavKon', false, false, 96, (select IdStdUcet from uStdUcty where uuid = 'CZ_POD_358'));
INSERT INTO usesstdpred (idsesstdpred, lastUpdate, druhuctuk, cissloup, zpusobvypk, prevratznam, jenkladne, idsesradky, idstducet)
 VALUES (396, '2010-03-15 00:00:00.000', 'druhUctu.aktivni', 1, 'zpusobVypo.stavKon', false, false, 96, (select IdStdUcet from uStdUcty where uuid = 'CZ_POD_391'));
INSERT INTO usesstdpred (idsesstdpred, lastUpdate, druhuctuk, cissloup, zpusobvypk, prevratznam, jenkladne, idsesradky, idstducet)
 VALUES (397, '2010-03-15 00:00:00.000', 'druhUctu.aktivni', 1, 'zpusobVypo.stavKon', false, false, 97, (select IdStdUcet from uStdUcty where uuid = 'CZ_POD_314'));
INSERT INTO usesstdpred (idsesstdpred, lastUpdate, druhuctuk, cissloup, zpusobvypk, prevratznam, jenkladne, idsesradky, idstducet)
 VALUES (398, '2010-03-15 00:00:00.000', 'druhUctu.aktivni', 1, 'zpusobVypo.stavKon', false, false, 97, (select IdStdUcet from uStdUcty where uuid = 'CZ_POD_391'));
INSERT INTO usesstdpred (idsesstdpred, lastUpdate, druhuctuk, cissloup, zpusobvypk, prevratznam, jenkladne, idsesradky, idstducet)
 VALUES (399, '2010-03-15 00:00:00.000', 'druhUctu.aktivni', 1, 'zpusobVypo.stavKon', false, false, 98, (select IdStdUcet from uStdUcty where uuid = 'CZ_POD_388'));
INSERT INTO usesstdpred (idsesstdpred, lastUpdate, druhuctuk, cissloup, zpusobvypk, prevratznam, jenkladne, idsesradky, idstducet)
 VALUES (400, '2010-03-15 00:00:00.000', 'druhUctu.aktivni', 1, 'zpusobVypo.stavKon', false, false, 99, (select IdStdUcet from uStdUcty where uuid = 'CZ_POD_335'));
INSERT INTO usesstdpred (idsesstdpred, lastUpdate, druhuctuk, cissloup, zpusobvypk, prevratznam, jenkladne, idsesradky, idstducet)
 VALUES (401, '2010-03-15 00:00:00.000', 'druhUctu.aktivni', 1, 'zpusobVypo.stavKon', false, false, 99, (select IdStdUcet from uStdUcty where uuid = 'CZ_POD_374'));
INSERT INTO usesstdpred (idsesstdpred, lastUpdate, druhuctuk, cissloup, zpusobvypk, prevratznam, jenkladne, idsesradky, idstducet)
 VALUES (402, '2010-03-15 00:00:00.000', 'druhUctu.aktivni', 1, 'zpusobVypo.stavKon', false, false, 99, (select IdStdUcet from uStdUcty where uuid = 'CZ_POD_375'));
INSERT INTO usesstdpred (idsesstdpred, lastUpdate, druhuctuk, cissloup, zpusobvypk, prevratznam, jenkladne, idsesradky, idstducet)
 VALUES (403, '2010-03-15 00:00:00.000', 'druhUctu.aktivni', 1, 'zpusobVypo.stavKon', false, false, 99, (select IdStdUcet from uStdUcty where uuid = 'CZ_POD_376'));
INSERT INTO usesstdpred (idsesstdpred, lastUpdate, druhuctuk, cissloup, zpusobvypk, prevratznam, jenkladne, idsesradky, idstducet)
 VALUES (404, '2010-03-15 00:00:00.000', 'druhUctu.aktivni', 1, 'zpusobVypo.stavKon', false, false, 99, (select IdStdUcet from uStdUcty where uuid = 'CZ_POD_378'));
INSERT INTO usesstdpred (idsesstdpred, lastUpdate, druhuctuk, cissloup, zpusobvypk, prevratznam, jenkladne, idsesradky, idstducet)
 VALUES (405, '2010-03-15 00:00:00.000', 'druhUctu.aktivni', 1, 'zpusobVypo.stavKon', false, false, 99, (select IdStdUcet from uStdUcty where uuid = 'CZ_POD_391'));
INSERT INTO usesstdpred (idsesstdpred, lastUpdate, druhuctuk, cissloup, zpusobvypk, prevratznam, jenkladne, idsesradky, idstducet)
 VALUES (406, '2010-03-15 00:00:00.000', 'druhUctu.aktivni', 1, 'zpusobVypo.stavKon', false, true, 99, (select IdStdUcet from uStdUcty where uuid = 'CZ_POD_370'));
INSERT INTO usesstdpred (idsesstdpred, lastUpdate, druhuctuk, cissloup, zpusobvypk, prevratznam, jenkladne, idsesradky, idstducet)
 VALUES (407, '2010-03-15 00:00:00.000', 'druhUctu.aktivni', 1, 'zpusobVypo.stavKon', false, true, 98, (select IdStdUcet from uStdUcty where uuid = 'CZ_POD_380'));
INSERT INTO usesstdpred (idsesstdpred, lastUpdate, druhuctuk, cissloup, zpusobvypk, prevratznam, jenkladne, idsesradky, idstducet)
 VALUES (408, '2010-03-16 00:00:00.000', 'druhUctu.aktivni', 1, 'zpusobVypo.stavKon', false, false, 94, (select IdStdUcet from uStdUcty where uuid = 'CZ_POD_395'));
INSERT INTO usesstdpred (idsesstdpred, lastUpdate, druhuctuk, cissloup, zpusobvypk, prevratznam, jenkladne, idsesradky, idstducet)
 VALUES (409, '2010-03-16 00:00:00.000', 'druhUctu.aktivni', 1, 'zpusobVypo.stavKon', false, true, 96, (select IdStdUcet from uStdUcty where uuid = 'CZ_POD_398'));
INSERT INTO usesstdpred (idsesstdpred, lastUpdate, druhuctuk, cissloup, zpusobvypk, prevratznam, jenkladne, idsesradky, idstducet)
 VALUES (410, '2010-03-16 00:00:00.000', 'druhUctu.aktivni', 1, 'zpusobVypo.stavKon', false, false, 100, (select IdStdUcet from uStdUcty where uuid = 'CZ_POD_314'));
INSERT INTO usesstdpred (idsesstdpred, lastUpdate, druhuctuk, cissloup, zpusobvypk, prevratznam, jenkladne, idsesradky, idstducet)
 VALUES (411, '2010-03-16 00:00:00.000', 'druhUctu.aktivni', 1, 'zpusobVypo.stavKon', false, true, 40, (select IdStdUcet from uStdUcty where uuid = 'CZ_POD_380'));
INSERT INTO usesstdpred (idsesstdpred, lastUpdate, druhuctuk, cissloup, zpusobvypk, prevratznam, jenkladne, idsesradky, idstducet)
 VALUES (412, '2010-03-16 00:00:00.000', 'druhUctu.pasivni', 1, 'zpusobVypo.stavKon', true, false, 53, (select IdStdUcet from uStdUcty where uuid = 'CZ_POD_410'));
INSERT INTO usesstdpred (idsesstdpred, lastUpdate, druhuctuk, cissloup, zpusobvypk, prevratznam, jenkladne, idsesradky, idstducet)
 VALUES (413, '2010-03-16 00:00:00.000', 'druhUctu.pasivni', 1, 'zpusobVypo.stavKon', true, false, 121, (select IdStdUcet from uStdUcty where uuid = 'CZ_POD_470'));
INSERT INTO usesstdpred (idsesstdpred, lastUpdate, druhuctuk, cissloup, zpusobvypk, prevratznam, jenkladne, idsesradky, idstducet)
 VALUES (414, '2010-03-16 00:00:00.000', 'druhUctu.pasivni', 1, 'zpusobVypo.stavKon', true, false, 120, (select IdStdUcet from uStdUcty where uuid = 'CZ_POD_360'));
INSERT INTO usesstdpred (idsesstdpred, lastUpdate, druhuctuk, cissloup, zpusobvypk, prevratznam, jenkladne, idsesradky, idstducet)
 VALUES (415, '2010-03-16 00:00:00.000', 'druhUctu.pasivni', 1, 'zpusobVypo.stavKon', true, true, 118, (select IdStdUcet from uStdUcty where uuid = 'CZ_POD_380'));
INSERT INTO usesstdpred (idsesstdpred, lastUpdate, druhuctuk, cissloup, zpusobvypk, prevratznam, jenkladne, idsesradky, idstducet)
 VALUES (417, '2010-03-16 00:00:00.000', 'druhUctu.pasivni', 1, 'zpusobVypo.stavKon', true, true, 117, (select IdStdUcet from uStdUcty where uuid = 'CZ_POD_370'));
INSERT INTO usesstdpred (idsesstdpred, lastUpdate, druhuctuk, cissloup, zpusobvypk, prevratznam, jenkladne, idsesradky, idstducet)
 VALUES (418, '2010-03-16 00:00:00.000', 'druhUctu.aktivni', 1, 'zpusobVypo.stavKon', false, false, 98, (select IdStdUcet from uStdUcty where uuid = 'CZ_POD_386'));
INSERT INTO usesstdpred (idsesstdpred, lastUpdate, druhuctuk, cissloup, zpusobvypk, prevratznam, jenkladne, idsesradky, idstducet)
 VALUES (419, '2010-03-16 00:00:00.000', 'druhUctu.pasivni', 1, 'zpusobVypo.stavKon', true, false, 78, (select IdStdUcet from uStdUcty where uuid = 'CZ_POD_479'));
INSERT INTO usesstdpred (idsesstdpred, lastUpdate, druhuctuk, cissloup, zpusobvypk, prevratznam, jenkladne, idsesradky, idstducet)
 VALUES (420, '2010-03-16 00:00:00.000', 'druhUctu.pasivni', 1, 'zpusobVypo.stavKon', true, true, 78, (select IdStdUcet from uStdUcty where uuid = 'CZ_POD_370'));
INSERT INTO usesstdpred (idsesstdpred, lastUpdate, druhuctuk, cissloup, zpusobvypk, prevratznam, jenkladne, idsesradky, idstducet)
 VALUES (421, '2010-03-16 00:00:00.000', 'druhUctu.pasivni', 1, 'zpusobVypo.stavKon', true, false, 80, (select IdStdUcet from uStdUcty where uuid = 'CZ_POD_461'));
INSERT INTO usesstdpred (idsesstdpred, lastUpdate, druhuctuk, cissloup, zpusobvypk, prevratznam, jenkladne, idsesradky, idstducet)
 VALUES (422, '2010-03-16 00:00:00.000', 'druhUctu.pasivni', 1, 'zpusobVypo.stavKon', true, true, 82, (select IdStdUcet from uStdUcty where uuid = 'CZ_POD_380'));
INSERT INTO usesstdpred (idsesstdpred, lastUpdate, druhuctuk, cissloup, zpusobvypk, prevratznam, jenkladne, idsesradky, idstducet)
 VALUES (423, '2010-03-16 00:00:00.000', 'druhUctu.aktivni', 1, 'zpusobVypo.stavKon', false, false, 17, (select IdStdUcet from uStdUcty where uuid = 'CZ_POD_040'));
INSERT INTO usesstdpred (idsesstdpred, lastUpdate, druhuctuk, cissloup, zpusobvypk, prevratznam, jenkladne, idsesradky, idstducet)
 VALUES (424, '2010-03-16 00:00:00.000', 'druhUctu.aktivni', 1, 'zpusobVypo.stavKon', false, false, 25, (select IdStdUcet from uStdUcty where uuid = 'CZ_POD_040'));
INSERT INTO usesstdpred (idsesstdpred, lastUpdate, druhuctuk, cissloup, zpusobvypk, prevratznam, jenkladne, idsesradky, idstducet)
 VALUES (425, '2010-03-16 00:00:00.000', 'druhUctu.aktivni', 1, 'zpusobVypo.stavKon', false, false, 18, (select IdStdUcet from uStdUcty where uuid = 'CZ_POD_050'));
INSERT INTO usesstdpred (idsesstdpred, lastUpdate, druhuctuk, cissloup, zpusobvypk, prevratznam, jenkladne, idsesradky, idstducet)
 VALUES (426, '2010-03-16 00:00:00.000', 'druhUctu.aktivni', 1, 'zpusobVypo.stavKon', false, false, 26, (select IdStdUcet from uStdUcty where uuid = 'CZ_POD_050'));
INSERT INTO usesstdpred (idsesstdpred, lastUpdate, druhuctuk, cissloup, zpusobvypk, prevratznam, jenkladne, idsesradky, idstducet)
 VALUES (427, '2010-03-16 00:00:00.000', 'druhUctu.aktivni', 1, 'zpusobVypo.stavKon', false, false, 11, (select IdStdUcet from uStdUcty where uuid = 'CZ_POD_090'));
INSERT INTO usesstdpred (idsesstdpred, lastUpdate, druhuctuk, cissloup, zpusobvypk, prevratznam, jenkladne, idsesradky, idstducet)
 VALUES (428, '2010-03-16 00:00:00.000', 'druhUctu.aktivni', 1, 'zpusobVypo.stavKon', false, false, 20, (select IdStdUcet from uStdUcty where uuid = 'CZ_POD_090'));
INSERT INTO usesstdpred (idsesstdpred, lastUpdate, druhuctuk, cissloup, zpusobvypk, prevratznam, jenkladne, idsesradky, idstducet)
 VALUES (434, '2010-03-16 00:00:00.000', 'druhUctu.pasivni', 1, 'zpusobVypo.stavKon', true, false, 58, (select IdStdUcet from uStdUcty where uuid = 'CZ_POD_420'));
INSERT INTO usesstdpred (idsesstdpred, lastUpdate, druhuctuk, cissloup, zpusobvypk, prevratznam, jenkladne, idsesradky, idstducet)
 VALUES (435, '2010-03-16 00:00:00.000', 'druhUctu.pasivni', 1, 'zpusobVypo.stavKon', true, false, 81, (select IdStdUcet from uStdUcty where uuid = 'CZ_POD_240'));
INSERT INTO usesstdpred (idsesstdpred, lastUpdate, druhuctuk, cissloup, zpusobvypk, prevratznam, jenkladne, idsesradky, idstducet)
 VALUES (436, '2010-03-16 00:00:00.000', 'druhUctu.pasivni', 1, 'zpusobVypo.stavKon', true, true, 50, (select IdStdUcet from uStdUcty where uuid = 'CZ_POD_250'));
INSERT INTO usesstdpred (idsesstdpred, lastUpdate, druhuctuk, cissloup, zpusobvypk, prevratznam, jenkladne, idsesradky, idstducet)
 VALUES (437, '2010-03-16 00:00:00.000', 'druhUctu.pasivni', 1, 'zpusobVypo.stavKon', true, true, 119, (select IdStdUcet from uStdUcty where uuid = 'CZ_POD_250'));
INSERT INTO usesstdpred (idsesstdpred, lastUpdate, druhuctuk, cissloup, zpusobvypk, prevratznam, jenkladne, idsesradky, idstducet)
 VALUES (438, '2010-03-16 00:00:00.000', 'druhUctu.pasivni', 1, 'zpusobVypo.stavKon', true, true, 76, (select IdStdUcet from uStdUcty where uuid = 'CZ_POD_250'));
INSERT INTO usesstdpred (idsesstdpred, lastUpdate, druhuctuk, cissloup, zpusobvypk, prevratznam, jenkladne, idsesradky, idstducet)
 VALUES (439, '2010-03-16 00:00:00.000', 'druhUctu.pasivni', 1, 'zpusobVypo.stavKon', true, false, 71, (select IdStdUcet from uStdUcty where uuid = 'CZ_POD_360'));
INSERT INTO usesstdpred (idsesstdpred, lastUpdate, druhuctuk, cissloup, zpusobvypk, prevratznam, jenkladne, idsesradky, idstducet)
 VALUES (440, '2010-03-16 00:00:00.000', 'druhUctu.pasivni', 1, 'zpusobVypo.stavKon', true, true, 77, (select IdStdUcet from uStdUcty where uuid = 'CZ_POD_380'));
INSERT INTO usesstdpred (idsesstdpred, lastUpdate, druhuctuk, cissloup, zpusobvypk, prevratznam, jenkladne, idsesradky, idstducet)
 VALUES (442, '2010-03-16 00:00:00.000', 'druhUctu.pasivni', 1, 'zpusobVypo.stavKon', true, false, 68, (select IdStdUcet from uStdUcty where uuid = 'CZ_POD_470'));
INSERT INTO usesstdpred (idsesstdpred, lastUpdate, druhuctuk, cissloup, zpusobvypk, prevratznam, jenkladne, idsesradky, idstducet)
 VALUES (443, '2010-03-16 00:00:00.000', 'druhUctu.naklady', 1, 'zpusobVypo.obrat', false, false, 135, (select IdStdUcet from uStdUcty where uuid = 'CZ_POD_557'));
INSERT INTO usesstdpred (idsesstdpred, lastUpdate, druhuctuk, cissloup, zpusobvypk, prevratznam, jenkladne, idsesradky, idstducet)
 VALUES (445, '2010-03-16 00:00:00.000', 'druhUctu.naklady', 1, 'zpusobVypo.obrat', false, false, 144, (select IdStdUcet from uStdUcty where uuid = 'CZ_POD_590'));
INSERT INTO usesstdpred (idsesstdpred, lastUpdate, druhuctuk, cissloup, zpusobvypk, prevratznam, jenkladne, idsesradky, idstducet)
 VALUES (446, '2010-03-16 00:00:00.000', 'druhUctu.naklady', 1, 'zpusobVypo.obrat', false, false, 156, (select IdStdUcet from uStdUcty where uuid = 'CZ_POD_567'));
INSERT INTO usesstdpred (idsesstdpred, lastUpdate, druhuctuk, cissloup, zpusobvypk, prevratznam, jenkladne, idsesradky, idstducet)
 VALUES (447, '2010-03-16 00:00:00.000', 'druhUctu.vynosy', 1, 'zpusobVypo.obrat', true, false, 157, (select IdStdUcet from uStdUcty where uuid = 'CZ_POD_690'));
INSERT INTO usesstdpred (idsesstdpred, lastUpdate, druhuctuk, cissloup, zpusobvypk, prevratznam, jenkladne, idsesradky, idstducet)
 VALUES (448, '2010-03-16 00:00:00.000', 'druhUctu.naklady', 1, 'zpusobVypo.obrat', false, false, 158, (select IdStdUcet from uStdUcty where uuid = 'CZ_POD_590'));
DELETE FROM usesstdpred WHERE IdSesStdPred = 92;
DELETE FROM usesstdpred WHERE IdSesStdPred = 161;
DELETE FROM usesstdpred WHERE IdSesStdPred = 182;
DELETE FROM usesstdpred WHERE IdSesStdPred = 272;
DELETE FROM usesstdpred WHERE IdSesStdPred = 342;

-- konec ceskeho bloku
  end if; 
end;
$$ LANGUAGE plpgsql;
ALTER FUNCTION runupgrade() OWNER TO %firmaRole%;

select runupgrade();

drop function runupgrade();


-- zvyseni verze databaze
insert into dbversion(iddbversion, dbversion, needreimport)
  values (nextval('seq_dbversion'), 275, 't');
