--------------------------------------------------------------------------------
-- Bug 19061 – [WinStrom] Zbytečné záznamy do žurnálu při výpočtu mezd
--------------------------------------------------------------------------------

-- trigger odnastavujici datum vypoctu mzdy pri zmene a hlidajici ze se nezmeni slozka v zamcenem obdobi
CREATE OR REPLACE FUNCTION pmzdslozupdatedatumvypoctu()
  RETURNS trigger AS $$
declare
  ZAZNAM record;
  _datumOd date;
  _datumDo date;
  _zamku integer;
  vypocet boolean;
  datDokl boolean;
  txName varchar;
begin
  if (tg_op = 'UPDATE') then
    if (OLD.idstalamzdsloz is not null and NEW.idstalamzdsloz is null) then     -- mazame stalou mzdovou slozku - to je ok
      return NEW;
    end if;
    if (OLD.idsrazka is not null and NEW.idsrazka is null) then     -- mazame srazku - to je ok
      return NEW;
    end if;
  end if;

  if (tg_op = 'DELETE') then
    ZAZNAM := OLD;  
  else  
    ZAZNAM := NEW;  
  end if;
  
  _datumOd = date (ZAZNAM.rok || '-' || ZAZNAM.mesic || '-01');
  _datumDo = _datumOd + interval '1 month';
  
  select count(*) into _zamku from wzamky where modulmzd = true and zamekk in ('zamek.polozamceno', 'zamek.zamceno') and
      platioddata < _datumDo and platidodata >= _datumOd;
  if (_zamku > 0) then
    raise exception 'Mzdy v období %/% jsou zamčené a není možno je měnit.', ZAZNAM.rok, ZAZNAM.mesic;
  end if;
  
  -- pokud se jedna o update resime cachry s datumava (vypocet, nastaveni datumu dokladu a jeho proplaceni)
  if (tg_op = 'UPDATE') then
    -- zjistime zda nastavujeme datum vypoctu
    vypocet = (NEW.typslozkyk = 'typSlozky.kopie' and NEW.skupslozkyk = 'skupinaSlozky.doba' and not comparedate(NEW.datumod, OLD.datumod));
    -- zjisitme zda se meni datum dokladu nebo obdobi platby
    datDokl = (NEW.idnepritomnost is not null and (not comparedate(NEW.datumdoklad, OLD.datumdoklad) or 
                  not compareid(NEW.rokplatba, OLD.rokplatba) or not compareid(NEW.mesicplatba, OLD.mesicplatba)));
  
    if (not vypocet and not datDokl) then
      SELECT getglotxvariable('zurnalTxName') INTO txName;
      if (txName is null or txName <> 'vypocetMezd') then       -- pokud pocitame mzdy tak neschazujeme - zbytecny zaznam v zurnalu
        update pmzdsloz set datumod = null where rok = ZAZNAM.rok and mesic = ZAZNAM.mesic and idosoba = ZAZNAM.idosoba and 
          typslozkyk = 'typSlozky.kopie' and skupslozkyk = 'skupinaSlozky.doba' and datumod is not null;
      end if;
    end if;
    
    if (datDokl) then
      update pmzdsloz set datumdoklad = NEW.datumdoklad, rokplatba = NEW.rokplatba, mesicplatba = NEW.mesicplatba 
        where idnepritomnost = NEW.idnepritomnost and rok = NEW.rok and mesic = NEW.mesic and idmzdsloz <> NEW.idmzdsloz;
    end if;
  end if;
  
  return ZAZNAM;
end;
$$ LANGUAGE plpgsql;
ALTER FUNCTION public.pmzdslozupdatedatumvypoctu() OWNER TO %firmaRole%;

-- zvyseni verze databaze
insert into dbversion(iddbversion, dbversion, needreimport)
  values (nextval('seq_dbversion'), 296, 't');
