--------------------------------------------------------------------------------
-- Bug 19293 – [WinStrom] Pri pouzivani FIFO skladu se neprepocitavaji skladove karty
--------------------------------------------------------------------------------
-- Function: dpolskladinsert()
-- DROP FUNCTION dpolskladinsert();
CREATE OR REPLACE FUNCTION dpolskladinsert()
  RETURNS trigger AS
$BODY$ 
declare 
  _rozsirenaPolozka record; 
  _typPohybuk character varying; 
  _mazatTmpTable boolean; 
begin 
  -- generuji rozsirene polozky 
  select skladGenerujRozsirenePolozky(NEW.datVyst, NEW.idPolSklad, NEW.idKarty) into _mazatTmpTable; 
 
  execute putglotxvariable('zurnalTxName', 'prepocetSkladu');  
 
  for _rozsirenaPolozka in (select * from tmp_RozsirenePolozky order by datum, skupina, iddoklskladMaster, typpohybuk desc, idpolsklad) loop 
    if(_rozsirenaPolozka.idPolSklad <> NEW.idPolSklad) then 
      execute skladOdpoctiRozsirenouPolozku(_rozsirenaPolozka.idPolSklad); 
    end if; 
  end loop; 
   
  for _rozsirenaPolozka in (select * from tmp_RozsirenePolozky order by datum, skupina, iddoklskladMaster, typpohybuk desc, idpolsklad) loop 
    execute skladZapoctiRozsirenouPolozku(_rozsirenaPolozka.idPolSklad); 
  end loop; 
 
  if(_mazatTmpTable) then 
    drop table tmp_RozsirenePolozky; 
    drop table tmp_SkladoveKarty; 
    execute removeglotxvariable('zurnalTxName'); 
  end if; 
  
  select typpohybuk into _typPohybuk from ddoklsklad where iddoklsklad = NEW.iddoklsklad;   
  --pokud výdej, tak fifuju vazby   
  if (_typPohybuk = 'typPohybu.vydej') then   
    execute sFifoVydej(NEW.idPolSklad, NEW.idCenik, NEW.idSkladu, NEW.idKarty, NEW.mnozmj);   
    execute sPrepoctiSumyDokladu(NEW.idDoklSklad);   
  end if;
  
  return NEW;
end;
$BODY$
  LANGUAGE 'plpgsql' VOLATILE
  COST 100;
ALTER FUNCTION dpolskladinsert() OWNER TO %firmaRole%;

-- Function: dpolskladupdate()
-- DROP FUNCTION dpolskladupdate();
CREATE OR REPLACE FUNCTION dpolskladupdate()
  RETURNS trigger AS
$BODY$ 
declare 
  tmp_rozsirenepolozky_exists boolean; 
  _updatuj boolean; 
  _pocetPolozek integer; 
  _rozsirenaPolozka record; 
  _sklFifo boolean; 
  _mazatTmpTable boolean; 
begin 
  /* 
    prepocet skladu budeme provadet jen v okamziku kdy neexistuje tabulka tmp_rozsirenePolozky 
    nebo kdyz tato tabulka existuje ale neni v ni obsazeno polozka kteoru chceme updatovat 
  */ 
  _updatuj = true; 
  tmp_rozsirenepolozky_exists := (select istableexist('tmp_RozsirenePolozky')); 
  if(tmp_rozsirenepolozky_exists = true) then 
    select count(*) into _pocetPolozek from tmp_rozsirenePolozky where idPolSklad = NEW.idPolSklad; 
    if(_pocetPolozek > 0) then 
      _updatuj = false; 
    end if; 
  end if; 

  if(OLD.idPolSklad < 0 and NEW.idPolSklad > 0) then
    _updatuj = false;
  end if;
  
  _sklfifo := (select isfifosklad(NEW.datvyst));   
  if (_sklfifo) then   
    --fifo   
    if (OLD.storno=false and NEW.storno=true) then   
      execute sfifoStorno(OLD.idpolsklad, OLD.iddoklsklad);   
    end if;     
    if (OLD.cenamj <> NEW.cenamj) then   
      execute sfifocenamj(OLD.idpolsklad, OLD.iddoklsklad, NEW.cenamj);   
    end if;     
    --fifo konec   
  end if;   
 
  if(_updatuj) then 
 
    -- generuji rozsirene polozky 
    select skladGenerujRozsirenePolozky(NEW.datVyst, NEW.idPolSklad, NEW.idKarty) into _mazatTmpTable; 
     
    execute putglotxvariable('zurnalTxName', 'prepocetSkladu'); 
  
    for _rozsirenaPolozka in (select * from tmp_RozsirenePolozky order by datum, skupina, iddoklskladMaster, typpohybuk desc, idpolsklad) loop 
      if(_rozsirenaPolozka.idPolSklad <> OLD.idPolSklad) then 
        execute skladOdpoctiRozsirenouPolozku(_rozsirenaPolozka.idPolSklad); 
      end if; 
    end loop; 
 
    execute sprepoctiskladkartu(OLD.mnozMj, OLD.cenaMj, OLD.datVyst, OLD.idKarty, OLD.idDoklSklad, OLD.idSkladu, -1); 
   
    for _rozsirenaPolozka in (select * from tmp_RozsirenePolozky order by datum, skupina, iddoklskladMaster, typpohybuk desc, idpolsklad) loop 
      execute skladZapoctiRozsirenouPolozku(_rozsirenaPolozka.idPolSklad); 
    end loop; 
 
    if(_mazatTmpTable) then 
      drop table tmp_RozsirenePolozky; 
      drop table tmp_SkladoveKarty; 
      execute removeglotxvariable('zurnalTxName'); 
    end if; 
      
  end if; 
   
  return NEW; 
end; 
$BODY$
  LANGUAGE 'plpgsql' VOLATILE
  COST 100;
ALTER FUNCTION dpolskladupdate() OWNER TO %firmaRole%;

-- zvyseni verze databaze
insert into dbversion(iddbversion, dbversion, needreimport)
  values (nextval('seq_dbversion'), 303, 't');
