-- Bug 12197 – [WinStrom] Rekurzivní zobrazení vazeb
-- realizace uložených procedur vyhledávajících vazby

-- návratový typ pro funkci hledající vazby
CREATE TYPE vazbadokladu AS
  (idvazebnidoklad integer,
  datucto timestamp without time zone,
  datvyst timestamp without time zone,
  iddokl integer,
  kod character varying(255),
  modul character varying(255),
  modulk character varying(255),
  popis character varying(255),
  poznam character varying(255),
  sumcelkem double precision,
  sumcelkemmen double precision,
  typvazbyk character varying(255),
  varsym character varying(255),
  idmeny integer,
  iduziv integer,
  idtypdokl integer,
  novy boolean,
  uroven integer);
ALTER TYPE vazbadokladu OWNER TO %firmaRole%;
  
-- funkce pro nalezení všech dokladů, které jsou nějak navázané na doklad na vstupu
CREATE OR REPLACE FUNCTION findVazbyDokladu(_idDokl integer, _modul varchar(20))
  RETURNS setof vazbadokladu AS $$
declare
  zaznam Record;
  _uroven integer;
  _prechodVP boolean;    -- zda povolen prechod z PPV/OBV na PPP/OBP
begin
  _uroven = 1;
  _prechodVP = true;
  
  drop table if exists vazbydokladu;
  
  CREATE TEMPORARY TABLE vazbydokladu
   (idvazebnidoklad integer NOT NULL,
    datucto timestamp without time zone,
    datvyst timestamp without time zone,
    iddokl integer,
    kod character varying(255),
    modul character varying(255),
    modulk character varying(255),
    popis character varying(255),
    poznam character varying(255),
    sumcelkem double precision,
    sumcelkemmen double precision,
    typvazbyk character varying(255),
    varsym character varying(255),
    idmeny integer,
    iduziv integer,
    idtypdokl integer,
    novy boolean,
    uroven integer,
    CONSTRAINT vazbydokladu_pkey PRIMARY KEY (idvazebnidoklad)
  ) WITH (OIDS=FALSE);
  
  perform findViceVazeb('vazbydokladu', _idDokl, _modul, 0, true);
  
  while (select count(*) from vazbydokladu where novy = true) > 0 LOOP
    if _prechodVP then
      select (count(*) = 0) into _prechodVP from vazbydokladu where modul in ('OBP', 'PPP') and novy = true;
    end if;
  
    for zaznam in (select * from vazbydokladu where novy = true)
    LOOP
      perform findViceVazeb('vazbydokladu', zaznam.idDokl, zaznam.modul, _uroven, _prechodVP);
  
      update vazbydokladu set novy = false where idvazebnidoklad = zaznam.idvazebnidoklad;
    end loop;

    _uroven = _uroven + 1;
  end loop;
  
  update vazbydokladu as V set modulK = modulK || 
    (case when (select typPohybuK from dDoklSklad where idDoklSklad = V.idDokl) = 'typPohybu.prijem' then 'PRI' else 'VYD' end) 
    where V.modul = 'SKL';

  return query SELECT * FROM vazbydokladu;
end; 
$$ LANGUAGE plpgsql;
ALTER FUNCTION public.findVazbyDokladu(integer, varchar) OWNER TO %firmaRole%;


-- funkce vkládající do zadané tabulky, vazby pro zadaný doklad pokud tam již nejsou
CREATE OR REPLACE FUNCTION findViceVazeb(_insertTab varchar(50), _idDokl integer, _modul varchar(20), _uroven integer, _prechodVP boolean)
  RETURNS void AS $$
declare
  tabulka varchar(20);
  dotaz varchar;
  prechodPom varchar;  
begin
  tabulka = getdoklnamepostfixbymodul(_modul);
  prechodPom = '';

  if (_uroven = 0) then
    dotaz = 
      'select 0 as idVazby, null as typVazbyK, D.idDokl' || tabulka || ' as idDokl, D.modul
        , ''modulUcetni.''||D.Modul as ModulK, D.kod, D.varSym, D.datVyst, D.datUcto, D.sumCelkem, D.sumCelkemMen, D.popis, D.poznam, D.idTypDokl, D.idMeny, D.idUziv
        from dDokl' || tabulka || ' as D where D.idDokl' || tabulka || ' = ' || _idDokl || ' and D.modul = ''' || _modul || '''';
  else

  if (tabulka = 'Fak') then
    dotaz = 
      'select V.idVazby, V.typVazbyK, D.idDoklInt as idDokl, D.modul
        , ''modulUcetni.''||D.Modul as ModulK, D.kod, D.varSym, D.datVyst, D.datUcto, D.sumCelkem, D.sumCelkemMen, D.popis, D.poznam, D.idTypDokl, D.idMeny, D.idUziv
        from dVazebTab as V left outer join dDoklInt as D on V.idTabB=D.idDoklInt and V.modulB=D.modul 
        where V.idTabA=' || _idDokl || ' and V.modulA=''' || _modul || ''' and V.typVazbyK in (''typVazbyDokl.uhrada'', ''typVazbyDokl.uhradaKR'',
            ''typVazbyDokl.uhradaZB'', ''typVazbyDokl.preceneni'', ''typVazbyDokl.vazbaZDD'', ''typVazbyDokl.uhradaZALOHY'') 
      union
      select V.idVazby, V.typVazbyK, D.idDoklSklad as idDokl, D.modul
        , ''modulUcetni.''||D.Modul as ModulK, D.kod, D.varSym, D.datVyst, D.datUcto, D.sumCelkem, D.sumCelkemMen, D.popis, D.poznam, D.idTypDokl, D.idMeny, D.idUziv
        from dVazebTab as V left outer join dDoklSklad as D on V.idTabB=D.idDoklSklad and V.modulB=D.modul
        where V.idTabA=' || _idDokl || ' and V.modulA=''' || _modul || ''' and V.typVazbyK = ''typVazbyDokl.hlavaSkl''
      union
      select min(V.idVazby) as idVazby, V.typVazbyK, D.idDoklFak as idDokl, D.modul
        , ''modulUcetni.''||D.Modul as ModulK, D.kod, D.varSym, D.datVyst, D.datUcto, D.sumCelkem, D.sumCelkemMen, D.popis, D.poznam, D.idTypDokl, D.idMeny, D.idUziv
        from dVazebTab as V left outer join dDoklFak as D on V.idTabB=D.idDoklFak and V.modulB=D.modul
        where V.idTabA in (select idPolFak from dPolFak where idDoklFak=' || _idDokl || ') and V.modulA=''' || _modul || ''' and
          V.typVazbyK in (''typVazbyDokl.odpocetZALOHY'', ''typVazbyDokl.odpocetZDD'') 
        group by V.typVazbyK, D.idDoklFak, D.modul, D.kod, D.varSym, D.datVyst, D.datUcto, D.sumCelkem, D.sumCelkemMen, D.popis, D.poznam, D.idTypDokl, D.idMeny, D.idUziv
      union 
      select min(V.idVazby) as idVazby, V.typVazbyK, D.idDoklFak as idDokl, D.modul
        , ''modulUcetni.''||D.Modul as ModulK, D.kod, D.varSym, D.datVyst, D.datUcto, D.sumCelkem, D.sumCelkemMen, D.popis, D.poznam, D.idTypDokl, D.idMeny, D.idUziv
        from dVazebTab as V left outer join dDoklFak as D on D.idDoklFak=
                (select idDoklFak from dPolFak where idPolFak = V.idTabA) and V.modulA=D.modul
        where V.idTabB=' || _idDokl || ' and V.modulB=''' || _modul || ''' and V.typVazbyK in (''typVazbyDokl.odpocetZALOHY'', ''typVazbyDokl.odpocetZDD'')
        group by V.typVazbyK, D.idDoklFak, D.modul, D.kod, D.varSym, D.datVyst, D.datUcto, D.sumCelkem, D.sumCelkemMen, D.popis, D.poznam, D.idTypDokl, D.idMeny, D.idUziv
      union 
      select V.idVazby, V.typVazbyK, D.idDoklObch as idDokl, D.modul
        , ''modulUcetni.''||D.Modul as ModulK, D.kod, D.varSym, D.datVyst, D.datUcto, D.sumCelkem, D.sumCelkemMen, D.popis, D.poznam, D.idTypDokl, D.idMeny, D.idUziv
        from dVazebTab as V left outer join dDoklObch as D on V.idTabA=D.idDoklObch and V.modulA=D.modul
        where V.idTabB=' || _idDokl || ' and V.modulB=''' || _modul || ''' and V.typVazbyK in (''typVazbyDokl.obchod_faktura_hla'', ''typVazbyDokl.obchod_zaloha_hla'')';
  else if (tabulka = 'Int') then
    dotaz = 
      'select V.idVazby, V.typVazbyK, D.idDoklFak as idDokl, D.modul
        , ''modulUcetni.''||D.Modul as ModulK, D.kod, D.varSym, D.datVyst, D.datUcto, D.sumCelkem, D.sumCelkemMen, D.popis, D.poznam, D.idTypDokl, D.idMeny, D.idUziv
        from dVazebTab as V left outer join dDoklFak as D on V.idTabA=D.idDoklFak and V.modulA=D.modul
        where V.idTabB=' || _idDokl || ' and V.modulB=''' || _modul || ''' and V.typVazbyK in (''typVazbyDokl.uhrada'', ''typVazbyDokl.uhradaKR'',
            ''typVazbyDokl.uhradaZB'', ''typVazbyDokl.preceneni'', ''typVazbyDokl.vazbaZDD'', ''typVazbyDokl.uhradaZALOHY'') 
      union
      select V.idVazby, V.typVazbyK, D.idDoklSklad as idDokl, D.modul
        , ''modulUcetni.''||D.Modul as ModulK, D.kod, D.varSym, D.datVyst, D.datUcto, D.sumCelkem, D.sumCelkemMen, D.popis, D.poznam, D.idTypDokl, D.idMeny, D.idUziv
        from dVazebTab as V left outer join dDoklSklad as D on V.idTabB=D.idDoklSklad and V.modulB=D.modul
        where V.idTabA=' || _idDokl || ' and V.modulA=''' || _modul || ''' and V.typVazbyK in (''typVazbyDokl.hlavaSkl'') 
      union
      select V.idVazby, V.typVazbyK, D.idDoklInt as idDokl, D.modul
        , ''modulUcetni.''||D.Modul as ModulK, D.kod, D.varSym, D.datVyst, D.datUcto, D.sumCelkem, D.sumCelkemMen, D.popis, D.poznam, D.idTypDokl, D.idMeny, D.idUziv
        from dVazebTab as V left outer join dDoklInt as D on V.idTabA=D.idDoklInt and V.modulA=D.modul
        where V.idTabB=' || _idDokl || ' and V.modulB=''' || _modul || ''' and V.typVazbyK in (''typVazbyDokl.KR'', ''typVazbyDokl.ZB'') 
      union
      select V.idVazby, V.typVazbyK, D.idDoklInt as idDokl, D.modul
        , ''modulUcetni.''||D.Modul as ModulK, D.kod, D.varSym, D.datVyst, D.datUcto, D.sumCelkem, D.sumCelkemMen, D.popis, D.poznam, D.idTypDokl, D.idMeny, D.idUziv
        from dVazebTab as V left outer join dDoklInt as D on V.idTabB=D.idDoklInt and V.modulB=D.modul
        where V.idTabA=' || _idDokl || ' and V.modulA=''' || _modul || ''' and V.typVazbyK in (''typVazbyDokl.KR'', ''typVazbyDokl.ZB'')';
  else if (tabulka = 'Sklad') then
    dotaz = 
      'select V.idVazby, V.typVazbyK, D.idDoklFak as idDokl, D.modul
        , ''modulUcetni.''||D.Modul as ModulK, D.kod, D.varSym, D.datVyst, D.datUcto, D.sumCelkem, D.sumCelkemMen, D.popis, D.poznam, D.idTypDokl, D.idMeny, D.idUziv
        from dVazebTab as V left outer join dDoklFak as D on V.idTabA=D.idDoklFak and V.modulA=D.modul
        where V.idTabB=' || _idDokl || ' and V.modulB=''' || _modul || ''' and 
          V.modulA in (''FAV'',''FAP'',''PHL'',''ZAV'') and V.typVazbyK = ''typVazbyDokl.hlavaSkl'' 
      union 
      select V.idVazby, V.typVazbyK, D.idDoklInt as idDokl, D.modul
        , ''modulUcetni.''||D.Modul as ModulK, D.kod, D.varSym, D.datVyst, D.datUcto, D.sumCelkem, D.sumCelkemMen, D.popis, D.poznam, D.idTypDokl, D.idMeny, D.idUziv
        from dVazebTab as V left outer join dDoklInt as D on V.idTabA=D.idDoklInt and V.modulA=D.modul
        where V.idTabB=' || _idDokl || ' and V.modulB=''' || _modul || ''' and 
          V.modulA in (''BAN'',''POK'',''INT'') and V.typVazbyK = ''typVazbyDokl.hlavaSkl''
      union 
      select V.idVazby, V.typVazbyK, D.idDoklObch as idDokl, D.modul
        , ''modulUcetni.''||D.Modul as ModulK, D.kod, D.varSym, D.datVyst, D.datUcto, D.sumCelkem, D.sumCelkemMen, D.popis, D.poznam, D.idTypDokl, D.idMeny, D.idUziv
        from dVazebTab as V left outer join dDoklObch as D on V.idTabA=D.idDoklObch and V.modulA=D.modul
        where V.idTabB=' || _idDokl || ' and V.modulB=''' || _modul || ''' and V.typVazbyK = ''typVazbyDokl.obchod_sklad_hla''';
  else if (tabulka = 'Obch') then
    if (_prechodVP = false and _modul in ('PPV', 'OBV')) then
      prechodPom = ' and D.modul in (''PPV'', ''NAP'', ''OBV'')';
    end if;
  
    dotaz = 
      'select V.idVazby, V.typVazbyK, D.idDoklObch as idDokl, D.modul
        , ''modulUcetni.''||D.Modul as ModulK, D.kod, D.varSym, D.datVyst, D.datUcto, D.sumCelkem, D.sumCelkemMen, D.popis, D.poznam, D.idTypDokl, D.idMeny, D.idUziv
        from dVazebTab as V left outer join dDoklObch as D on V.idTabA=D.idDoklObch and V.modulA=D.modul 
        where V.idTabB=' || _idDokl || ' and V.modulB=''' || _modul || ''' and V.typVazbyK = ''typVazbyDokl.obchod_obchod_hla''' || prechodPom || '
      union 
      select V.idVazby, V.typVazbyK, D.idDoklObch as idDokl, D.modul
        , ''modulUcetni.''||D.Modul as ModulK, D.kod, D.varSym, D.datVyst, D.datUcto, D.sumCelkem, D.sumCelkemMen, D.popis, D.poznam, D.idTypDokl, D.idMeny, D.idUziv
        from dVazebTab as V left outer join dDoklObch as D on V.idTabB=D.idDoklObch and V.modulB=D.modul 
        where V.idTabA=' || _idDokl || ' and V.modulA=''' || _modul || ''' and V.typVazbyK = ''typVazbyDokl.obchod_obchod_hla''' || prechodPom || '
      union
      select V.idVazby, V.typVazbyK, D.idDoklFak as idDokl, D.modul
        , ''modulUcetni.''||D.Modul as ModulK, D.kod, D.varSym, D.datVyst, D.datUcto, D.sumCelkem, D.sumCelkemMen, D.popis, D.poznam, D.idTypDokl, D.idMeny, D.idUziv
        from dVazebTab as V left outer join dDoklFak as D on V.idTabB=D.idDoklFak and V.modulB=D.modul 
        where V.idTabA=' || _idDokl || ' and V.modulA=''' || _modul || ''' and V.typVazbyK in (''typVazbyDokl.obchod_faktura_hla'', ''typVazbyDokl.obchod_zaloha_hla'')
      union
      select V.idVazby, V.typVazbyK, D.idDoklSklad as idDokl, D.modul
        , ''modulUcetni.''||D.Modul as ModulK, D.kod, D.varSym, D.datVyst, D.datUcto, D.sumCelkem, D.sumCelkemMen, D.popis, D.poznam, D.idTypDokl, D.idMeny, D.idUziv
        from dVazebTab as V left outer join dDoklSklad as D on V.idTabB=D.idDoklSklad and V.modulB=D.modul 
        where V.idTabA=' || _idDokl || ' and V.modulA=''' || _modul || ''' and V.typVazbyK = ''typVazbyDokl.obchod_sklad_hla''';
  else
    raise exception 'findVazbyDokladu(): Neznámá tabulka % pro modul %', tabulka, _modul; 
  end if;
  end if;
  end if;
  end if;
  
  end if;
  
  execute 'insert into ' || _insertTab || ' (idvazebnidoklad, typVazbyK, idDokl, modul, modulk, kod, varsym, datvyst, datucto, sumcelkem,
  sumcelkemmen, popis, poznam, idtypdokl,idmeny, iduziv,uroven, novy) select *, ' || _uroven || ', true from (' 
     || dotaz || ') as UN where (select count(*) from ' || _insertTab || ' as TV where 
     (TV.idDokl = UN.idDokl and TV.modul = UN.modul) or UN.idVazby = TV.idvazebnidoklad) = 0';
end;
$$ LANGUAGE plpgsql;
ALTER FUNCTION public.findViceVazeb(varchar, integer, varchar, integer, boolean) OWNER TO %firmaRole%;
  

-- zvýšení verze databáze
insert into dbversion(iddbversion, dbversion, needreimport) values (nextval('seq_dbversion'), 32, 't');
