--------------------------------------------------------------------------------
-- Bug 19561 – [WinStrom] Na doklad doplnit položku "Činnost"
--------------------------------------------------------------------------------

-- navazani na typ dokladu
ALTER TABLE dtypdokl ADD COLUMN idcinnost integer;
ALTER TABLE dtypdokl ADD CONSTRAINT fkc4463b5394e84085 FOREIGN KEY (idcinnost)
  REFERENCES ucinnost (idcinnost) MATCH SIMPLE ON UPDATE NO ACTION ON DELETE NO ACTION;

CREATE INDEX dTypDokl_IdCinnost_index ON dTypDokl USING btree (IdCinnost);


-- navazani na obraty
ALTER TABLE uobraty ADD COLUMN idcinnost integer;
ALTER TABLE uobraty ADD CONSTRAINT uobraty_idcinnost FOREIGN KEY (idcinnost)
  REFERENCES ucinnost (idcinnost) MATCH SIMPLE ON UPDATE NO ACTION ON DELETE NO ACTION;

CREATE INDEX uObraty_IdCinnost_index ON uObraty USING btree (IdCinnost);

-- constraint nemusi existovat na vsech firmach, byl pridan v upgrade 184
SELECT drop_constraint_if_exists('uobraty', 'uobraty_unique_key');

-- vytvorime ho opet jen tam kde je to mozne
create or replace function runupgrade() returns void as $$
declare
  zaznam record;
  maxpocet integer;
begin
  -- V obdobi kde existuji duplicity v obratech provedeme jejich přepočet.
  for zaznam in select * from nucetobdobi loop
    select coalesce(max(a.pocet),0) into maxpocet
    from (
      select count(*) as pocet
      from uobraty
      where iducetobdobi = zaznam.iducetobdobi and obdobi not in (0,99)   -- obdobi not in (0,99) kvůli duplicitám u Kartonie
      group by obdobi,rok,idmeny,iducet,idstred,iducetobdobi
    ) as a;

    if(maxpocet > 1) then
      --raise notice 'prepoctiobraty(id=%, kod=%)', zaznam.iducetobdobi, zaznam.kod;
      perform prepoctiobraty(zaznam.iducetobdobi);
    end if;
  end loop;

  -- Přidání chybějícího integritního omezení na unikátnost obratu
  select coalesce(max(a.pocet),0) into maxpocet
  from (
    select count(*) as pocet
    from uobraty
    group by obdobi,rok,idmeny,iducet,idstred,iducetobdobi
  ) as a;

  if(maxpocet < 2) then
    ALTER TABLE uobraty
      ADD CONSTRAINT uobraty_unique_key UNIQUE(obdobi, rok, idmeny, iducet, idstred, idcinnost, iducetobdobi);
  end if;
end;
$$ LANGUAGE plpgsql;
ALTER FUNCTION runupgrade() OWNER TO %firmaRole%;

select runupgrade();
drop function runupgrade();


-- uprava funkce uctujici obraty
CREATE OR REPLACE FUNCTION createorupdateobraty(ucetni boolean, storno boolean, datumucto date, castkamducet integer, castkadalucet integer, mena integer, stredisko integer, cinnost integer, castka numeric, castkamen numeric)
  RETURNS void AS $$
declare
  nRok integer;
  nObdobi integer;
  idUcetniObdobi integer;
  idMenyTuzemsko integer;
  id integer;
  puvodniMD numeric(15,2);
  puvodniDal numeric(15,2);
begin
  if ucetni = false or storno = true or datumucto is null or stredisko is null or castkamducet is null or castkadalucet is null then
    return;
  end if;

  select idMeny into idMenyTuzemsko from nNastav where (platiOdData is null or platiOdData <= datumucto)
    order by coalesce(platioddata, '0001-01-01') desc limit 1;

  nRok:= date_part('year', datumUcto);
  nObdobi:= date_part('month', datumUcto);

  select idUcetObdobi into idUcetniObdobi from nUcetObdobi where (datumUcto >= PlatiOdData and datumUcto <= PlatiDoData) limit 1;

  /* TODO ma byt podminka na ucetni obdobi"?" */
  if(nRok is not null) and (nRok <> 0) and (nObdobi is not null) and (nObdobi <> 0) and (idMenyTuzemsko is not null) and (idUcetniObdobi is not null) then

    /*dph tuzemsko*/
    if (castkaMdUcet is not null and stredisko is not null and castka <> 0) then
      select idObrat, obrMD, obrDal into id, puvodniMD, puvodniDal from uObraty where idUcet = castkaMdUcet and Rok = nRok and Obdobi = nObdobi and
          idStred = stredisko and idMeny = idMenyTuzemsko and compareid(cinnost, idCinnost) for update; -- zamek !!!

      if (id is null) then
        insert into uObraty(idUcet, Rok, Obdobi, idStred, idMeny, idCinnost, ObrDal, ObrMD, idUcetObdobi, LastUpdate)
	         values(castkaMdUcet, nRok, nObdobi, stredisko, idMenyTuzemsko, cinnost, 0.0, castka, idUcetniObdobi, now());
      else
        puvodniMD := puvodniMD + castka;
        if (puvodniMD = 0 and puvodniDal = 0) then
          delete from uObraty where idObrat = id;
        else
          update uObraty set obrMD = obrMD + castka, LastUpdate = now() where idObrat = id;
        end if;
      end if;
    end if;

    if (castkaDalUcet is not null and stredisko is not null and castka <> 0) then
      select idObrat, obrMD, obrDal into id, puvodniMD, puvodniDal from uObraty where idUcet = castkaDalUcet and Rok = nRok and Obdobi = nObdobi and
          idStred = stredisko and idMeny = idMenyTuzemsko and compareid(cinnost, idCinnost) for update;

      if (id is null) then
        insert into uObraty (idUcet, Rok, Obdobi, idStred, idMeny, idCinnost, ObrDal, ObrMD, idUcetObdobi, LastUpdate)
           values(castkaDalUcet, nRok, nObdobi, stredisko, idMenyTuzemsko, cinnost, castka, 0.0, idUcetniObdobi, now());
      else
        puvodniDal := puvodniDal + castka;
        if (puvodniMD = 0 and puvodniDal = 0) then
          delete from uObraty where idObrat = id;
        else
          update uObraty set obrDal = obrDal + castka, LastUpdate = now() where idObrat = id;
        end if;
      end if;
    end if;

    -- cizi mena, v pripade, ze se mena nema zapocitavat tak ma hodnotu -1
    if (mena <> -1) and mena <> idMenyTuzemsko then
          if(castkaMdUcet is not null and stredisko is not null and castkaMen <> 0) then
            select idObrat, obrMD, obrDal into id, puvodniMD, puvodniDal from uObraty
            where idUcet = castkaMdUcet and Rok = nRok and Obdobi = nObdobi and
              idStred = stredisko and idMeny = mena and compareid(cinnost, idCinnost) for update;

            if (id is null) then
              insert into uObraty(idUcet, Rok, Obdobi, idStred, idMeny, idCinnost, ObrDal, ObrMD, idUcetObdobi, LastUpdate)
                 values(castkaMdUcet, nRok, nObdobi, stredisko, mena, cinnost, 0.0, castkaMen, idUcetniObdobi, now());
            else
              puvodniMD := puvodniMD + castkaMen;
              if (puvodniMD = 0 and puvodniDal = 0) then
                delete from uObraty where idObrat = id;
              else
                update uObraty set obrMD = obrMD + castkaMen, LastUpdate = now() where idObrat = id;
              end if;
            end if;
          end if;

          if(castkaDalUcet is not null and stredisko is not null and castkaMen <> 0) then
            select idObrat, obrMD, obrDal into id, puvodniMD, puvodniDal from uObraty
            where idUcet = castkaDalUcet and Rok = nRok and Obdobi = nObdobi and
              idStred = stredisko and idMeny = mena and compareid(cinnost, idCinnost) for update;

            if (id is null) then
              insert into uObraty(idUcet, Rok, Obdobi, idStred, idMeny, idCinnost, ObrDal, ObrMD, idUcetObdobi, LastUpdate)
                 values(castkaDalUcet, nRok, nObdobi, stredisko, mena, cinnost, castkaMen, 0.0, idUcetniObdobi, now());
            else
              puvodniDal := puvodniDal + castkaMen;
              if (puvodniMD = 0 and puvodniDal = 0) then
                delete from uObraty where idObrat = id;
              else
                update uObraty set obrDal = obrDal + castkaMen, LastUpdate = now() where idObrat = id;
              end if;
            end if;
          end if;
    end if;

  else
    -- v pripade, ze dany objekt nema vyplneno datumUcto, nejsou hodnoty nastaveny a nezapocitavaji se do obratu
    if (datumUcto is not null) then
        raise exception 'aktualizace zaznamu tabulky uObraty: Neplatné vstupní hodnoty. % % % %',idMenyTuzemsko,idUcetniObdobi,nRok,nObdobi;
    end if;
  end if;
end;
$$ LANGUAGE plpgsql;
ALTER FUNCTION public.createorupdateobraty(boolean, boolean, date, integer, integer, integer, integer, integer, numeric, numeric) OWNER TO %firmaRole%;


-- funkce se stejnyma parametrama jako driv pro zpetnou kompatibilitu na jinych tabulkach nez polozky dokladu
CREATE OR REPLACE FUNCTION createorupdateobraty(ucetni boolean, storno boolean, datumucto date, castkamducet integer, castkadalucet integer, mena integer, stredisko integer, castka numeric, castkamen numeric)
  RETURNS void AS $$
begin
  PERFORM createorupdateobraty(ucetni, storno, datumucto, castkamducet, castkadalucet, mena, stredisko, null, castka, castkamen);
end;
$$ LANGUAGE plpgsql;
ALTER FUNCTION public.createorupdateobraty(boolean, boolean, date, integer, integer, integer, integer, numeric, numeric) OWNER TO %firmaRole%;


-- aktualizace triggeru uctujicich polozky dokladu
CREATE OR REPLACE FUNCTION dpoldoklinsertucto() RETURNS trigger AS $$ 
begin 
  execute createOrUpdateObraty(NEW.Ucetni, NEW.Storno, NEW.DatUcto, NEW.idZklMdUcet, NEW.idZklDalUcet, 
        NEW.idMeny, NEW.idStred, NEW.idCinnost, NEW.sumZkl, NEW.sumZklMen); 
   
  execute createOrUpdateObraty(NEW.Ucetni, NEW.Storno, NEW.DatUcto, NEW.idDphMdUcet, NEW.idDphDalUcet, 
        NEW.idMeny, NEW.idStred, NEW.idCinnost, NEW.sumDph, NEW.sumDphMen); 
   
  return NEW; 
end; 
$$ LANGUAGE plpgsql;
ALTER FUNCTION public.dpoldoklinsertucto() OWNER TO %firmaRole%;

CREATE OR REPLACE FUNCTION dpoldoklupdateucto() RETURNS trigger AS $$ 
begin 
  execute createOrUpdateObraty(OLD.Ucetni, OLD.Storno, OLD.DatUcto, OLD.idZklMdUcet, OLD.idZklDalUcet, 
      OLD.idMeny, OLD.idStred, OLD.idCinnost, -OLD.sumZkl, -OLD.sumZklMen); 
   
  execute createOrUpdateObraty(OLD.Ucetni, OLD.Storno, OLD.DatUcto, OLD.idDphMdUcet, OLD.idDphDalUcet, 
      OLD.idMeny, OLD.idStred, OLD.idCinnost, -OLD.sumDph, -OLD.sumDphMen); 
   
  execute createOrUpdateObraty(NEW.Ucetni, NEW.Storno, NEW.DatUcto, NEW.idZklMdUcet, NEW.idZklDalUcet, 
      NEW.idMeny, NEW.idStred, NEW.idCinnost, NEW.sumZkl, NEW.sumZklMen); 
   
  execute createOrUpdateObraty(NEW.Ucetni, NEW.Storno, NEW.DatUcto, NEW.idDphMdUcet, NEW.idDphDalUcet, 
      NEW.idMeny, NEW.idStred, NEW.idCinnost, NEW.sumDph, NEW.sumDphMen); 
   
  return NEW; 
end; 
$$ LANGUAGE plpgsql;
ALTER FUNCTION public.dpoldoklupdateucto() OWNER TO %firmaRole%;

CREATE OR REPLACE FUNCTION dpoldokldeleteucto() RETURNS trigger AS $$ 
begin 
  execute createOrUpdateObraty(OLD.Ucetni, OLD.Storno, OLD.DatUcto, OLD.idZklMdUcet, OLD.idZklDalUcet, 
        OLD.idMeny, OLD.idStred, OLD.idCinnost, -OLD.sumZkl, -OLD.sumZklMen); 
   
  execute createOrUpdateObraty(OLD.Ucetni, OLD.Storno, OLD.DatUcto, OLD.idDphMdUcet, OLD.idDphDalUcet, 
        OLD.idMeny, OLD.idStred, OLD.idCinnost, -OLD.sumDph, -OLD.sumDphMen); 
   
  return OLD; 
end; 
$$ LANGUAGE plpgsql;
ALTER FUNCTION public.dpoldokldeleteucto() OWNER TO %firmaRole%;


-- uprava funkce pro prepocet obratu
CREATE OR REPLACE FUNCTION prepoctiobraty(idUcetObd integer)
  RETURNS void AS $$
declare
  platiOd date;
  platiDo date;
begin
  select platioddata, platidodata into platiOd, platiDo from nucetobdobi where iducetobdobi = idUcetObd;

  if (platiOd is not null and platiDo is not null) then
    delete from uobraty where iducetobdobi = idUcetObd and obdobi > 0;
    
    -- zauctovani polozek faktur
     perform createOrUpdateObraty(P.ucetni, P.storno, P.datucto, P.idZklMdUcet, P.idZklDalUcet, P.idMeny, P.idStred,
         P.idCinnost, P.sumZkl, P.sumZklMen) from dpolfak as P 
         where P.ucetni = true and P.storno = false and P.datUcto between platiOd and platiDo;
     
     perform createOrUpdateObraty(P.ucetni, P.storno, P.datucto, P.idDphMdUcet, P.idDphDalUcet, P.idMeny, P.idStred,
         P.idCinnost, P.sumDph, P.sumDphMen) from dpolfak as P 
         where P.ucetni = true and P.storno = false and P.datUcto between platiOd and platiDo;
       
     -- zauctovani polozek internich dokladu
     perform createOrUpdateObraty(P.ucetni, P.storno, P.datucto, P.idZklMdUcet, P.idZklDalUcet, P.idMeny, P.idStred,
         P.idCinnost, P.sumZkl, P.sumZklMen) from dpolint as P 
         where P.ucetni = true and P.storno = false and P.datUcto between platiOd and platiDo;
       
     perform createOrUpdateObraty(P.ucetni, P.storno, P.datucto, P.idDphMdUcet, P.idDphDalUcet, P.idMeny, P.idStred,
         P.idCinnost, P.sumDph, P.sumDphMen) from dpolint as P 
         where P.ucetni = true and P.storno = false and P.datUcto between platiOd and platiDo;
                                
     -- zauctovani polozek skladovych dokladu                     
     perform createOrUpdateObraty(P.ucetni, P.storno, P.datucto, P.idZklMdUcet, P.idZklDalUcet, P.idMeny, P.idStred,
         P.idCinnost, P.sumZkl, P.sumZklMen) from dpolsklad as P 
         where P.ucetni = true and P.storno = false and P.datUcto between platiOd and platiDo;
       
     perform createOrUpdateObraty(P.ucetni, P.storno, P.datucto, P.idDphMdUcet, P.idDphDalUcet, P.idMeny, P.idStred,
         P.idCinnost, P.sumDph, P.sumDphMen) from dpolsklad as P 
         where P.ucetni = true and P.storno = false and P.datUcto between platiOd and platiDo;
    
     -- danove naklady
     perform createOrUpdateObraty(P.ucetni, false, P.datUcto, P.idZklMdUcet, P.idZklDalUcet, L.idMeny, P.idStred, 
          P.sumZkl, P.sumZklMen) from mdannakl as P left outer join mleasing as L on L.idleasing = P.idleasing 
          where P.ucetni = true and P.datUcto between platiOd and platiDo;
          
     -- danove odpisy
     -- odpisy majetku jsou vedeny vzdy v mistni mene, proto za menu dosazujeme -1 a za sumZklMen 0.0
     perform createOrUpdateObraty(P.ucetni, false, P.datUcto, P.idZklMdUcet, P.idZklDalUcet, -1, P.idStred, 
          P.sumZkl, 0.0) from mdanodpisy as P
          where P.ucetni = true and P.datUcto between platiOd and platiDo;
     
     -- splatkovy kalendar
     perform createOrUpdateObraty(P.ucetni, false, P.datUcto, P.idZklMdUcet, P.idZklDalUcet, L.idMeny, P.idStred, 
          P.sumZkl, P.sumZklMen) from msplatkal as P left outer join mleasing as L on L.idleasing = P.idleasing 
          where P.ucetni = true and P.datUcto between platiOd and platiDo;
          
     perform createOrUpdateObraty(P.ucetni, false, P.datUcto, P.idDphMdUcet, P.idDphDalUcet, L.idMeny, P.idStred, 
          P.sumDph, P.sumDphMen) from msplatkal as P left outer join mleasing as L on L.idleasing = P.idleasing 
          where P.ucetni = true and P.datUcto between platiOd and platiDo;
     
     -- ucetni odpisy
     -- odpisy majetku jsou vedeny vzdy v mistni mene, proto za menu dosazujeme -1 a za sumZklMen 0.0
     perform createOrUpdateObraty(P.ucetni, false, P.datUcto, P.idZklMdUcet, P.idZklDalUcet, -1, P.idStred, 
          P.sumZkl, 0.0) from mucetodpisy as P
          where P.ucetni = true and P.datUcto between platiOd and platiDo;
     
     -- udalosti
     -- odpisy majetku jsou vedeny vzdy v mistni mene, proto za menu dosazujeme -1 a za sumZklMen 0.0
     perform createOrUpdateObraty(P.ucetni, false, P.datUcto, P.idZklMdUcet, P.idZklDalUcet, -1, P.idStred, 
          P.sumZkl, 0.0) from mudalosti as P
          where P.ucetni = true and P.datUcto between platiOd and platiDo;
       
  end if;
end; 
$$ LANGUAGE plpgsql;
ALTER FUNCTION public.prepoctiobraty(iducetobd integer) OWNER TO %firmaRole%;


-- zvyseni verze databaze
insert into dbversion(iddbversion, dbversion, needreimport)
  values (nextval('seq_dbversion'), 326, 't');