--------------------------------------------------------------------------------
-- Bug 20331 – [WinStrom] Rozsireni obchodnich partneru
--------------------------------------------------------------------------------

---- Pridani novych tabulek ----

-- aktivity
-- pridani tabulky s typy aktivity
CREATE SEQUENCE seq_atypakt
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;
ALTER TABLE public.seq_atypakt OWNER TO %firmaRole%;

CREATE TABLE atypakt (
    idtypakt integer DEFAULT nextval('seq_atypakt'::regclass) NOT NULL,
    lastupdate timestamp without time zone NOT NULL DEFAULT '2008-01-01 00:00:00'::timestamp without time zone,
    kod character varying(20) NOT NULL,
    nazev character varying(255) NOT NULL,
    nazeva character varying(255),
    nazevb character varying(255),
    nazevc character varying(255),
    popis text,
    poznam text,
    "show" boolean NOT NULL DEFAULT true,
    platido integer NOT NULL DEFAULT 9999,
    platiod integer NOT NULL DEFAULT 0,
    druhudalk character varying(50),
    iduzivatel integer,
    CONSTRAINT atypakt_pkey PRIMARY KEY (idtypakt),
    CONSTRAINT atypakt_idzodppracovnik FOREIGN KEY (iduzivatel)
        REFERENCES wuzivatel(iduzivatel) MATCH SIMPLE
        ON UPDATE NO ACTION ON DELETE NO ACTION,
    CONSTRAINT atypakt_unique_kod UNIQUE (kod)
);
ALTER TABLE atypakt OWNER TO %firmaRole%;

CREATE INDEX atypakt_iduzivatel_index ON atypakt USING btree (iduzivatel);

-- pridani tabulky s typem nakladu aktivity
CREATE SEQUENCE seq_atypnaklakt
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;
ALTER TABLE public.seq_atypnaklakt OWNER TO %firmaRole%;

CREATE TABLE atypnaklakt (
    idtypnaklakt integer DEFAULT nextval('seq_atypnaklakt'::regclass) NOT NULL,
    lastupdate timestamp without time zone NOT NULL DEFAULT '2008-01-01 00:00:00'::timestamp without time zone,
    kod character varying(20) NOT NULL,
    nazev character varying(255) NOT NULL,
    nazeva character varying(255),
    nazevb character varying(255),
    nazevc character varying(255),
    popis text,
    poznam text,
    "show" boolean NOT NULL DEFAULT true,
    fakturovat boolean NOT NULL DEFAULT TRUE,
    idcenik integer NOT NULL,
    CONSTRAINT atypnaklakt_pkey PRIMARY KEY (idtypnaklakt),
    CONSTRAINT atypnaklakt_idcenik FOREIGN KEY (idcenik)
        REFERENCES ccenik(idcenik) MATCH SIMPLE
        ON UPDATE NO ACTION ON DELETE NO ACTION,
    CONSTRAINT atypnaklakt_unique_kod UNIQUE (kod)
);
ALTER TABLE atypnaklakt OWNER TO %firmaRole%;

CREATE INDEX atypnaklakt_idcenik_index ON atypnaklakt USING btree (idcenik);

-- pridani tabulky s nakladem aktivity
CREATE SEQUENCE seq_anaklakt
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;
ALTER TABLE public.seq_anaklakt OWNER TO %firmaRole%;

CREATE TABLE anaklakt (
    idnaklakt integer DEFAULT nextval('seq_anaklakt'::regclass) NOT NULL,
    lastupdate timestamp without time zone NOT NULL DEFAULT '2008-01-01 00:00:00'::timestamp without time zone NOT NULL,
    faktmnoz numeric(19,6) NOT NULL DEFAULT 0.0,
    fakturovano boolean NOT NULL DEFAULT FALSE,
    fakturovat boolean NOT NULL DEFAULT TRUE,
    poznam text,
    realmnoz numeric(19,6) NOT NULL DEFAULT 0.0,
    termin timestamp without time zone,
    idtypnaklakt integer NOT NULL,
    idcenik integer NOT NULL,
    iduzivatel integer NOT NULL,
    idadrudalost integer NOT NULL,
    CONSTRAINT anaklakt_pkey PRIMARY KEY (idnaklakt),
    CONSTRAINT anaklakt_idtypnaklakt FOREIGN KEY (idtypnaklakt)
        REFERENCES atypnaklakt(idtypnaklakt) MATCH SIMPLE
        ON UPDATE NO ACTION ON DELETE NO ACTION,
    CONSTRAINT anaklakt_idcenik FOREIGN KEY (idcenik)
        REFERENCES ccenik(idcenik) MATCH SIMPLE
        ON UPDATE NO ACTION ON DELETE NO ACTION,
    CONSTRAINT anaklakt_iduzivatel FOREIGN KEY (iduzivatel)
        REFERENCES wuzivatel(iduzivatel) MATCH SIMPLE
        ON UPDATE NO ACTION ON DELETE NO ACTION,
    CONSTRAINT anaklakt_idadrudalost FOREIGN KEY (idadrudalost)
        REFERENCES audalosti(idudalost) MATCH SIMPLE
        ON UPDATE NO ACTION ON DELETE NO ACTION
);
ALTER TABLE anaklakt OWNER TO %firmaRole%;

CREATE INDEX anaklakt_idcenik_index ON anaklakt USING btree (idcenik);
CREATE INDEX anaklakt_iduzivatel_index ON anaklakt USING btree (iduzivatel);
CREATE INDEX anaklakt_idadrudalost_index ON anaklakt USING btree (idadrudalost);

-- pridani tabulky typ zakazky
CREATE SEQUENCE seq_utypzakazky
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;
ALTER TABLE public.seq_utypzakazky OWNER TO %firmaRole%;

CREATE TABLE utypzakazky (
    idtypzakazky integer DEFAULT nextval('seq_utypzakazky'::regclass) NOT NULL,
    lastupdate timestamp without time zone NOT NULL DEFAULT '2008-01-01 00:00:00'::timestamp without time zone,
    kod character varying(20) NOT NULL,
    nazev character varying(255) NOT NULL,
    nazeva character varying(255),
    nazevb character varying(255),
    nazevc character varying(255),
    popis text,
    poznam text,
    "show" boolean NOT NULL DEFAULT true,
    platido integer NOT NULL DEFAULT 9999,
    platiod integer NOT NULL DEFAULT 0,
    druhzakk character varying(50),
    idstred integer,
    iduzivatel integer,
    CONSTRAINT utypzakazky_pkey PRIMARY KEY (idtypzakazky),
    CONSTRAINT utypzakazky_idstred FOREIGN KEY (idstred)
        REFERENCES ustrediska(idstred) MATCH SIMPLE
        ON UPDATE NO ACTION ON DELETE NO ACTION,
    CONSTRAINT utypzakazky_iduzivatel FOREIGN KEY (iduzivatel)
        REFERENCES wuzivatel(iduzivatel) MATCH SIMPLE
        ON UPDATE NO ACTION ON DELETE NO ACTION,
    CONSTRAINT utypzakazky_unique_kod UNIQUE (kod)
);
ALTER TABLE utypzakazky OWNER TO %firmaRole%;

CREATE INDEX utypzakazky_idstred_index ON utypzakazky USING btree (idstred);
CREATE INDEX utypzakazky_iduzivatel_index ON utypzakazky USING btree (iduzivatel);

-- pridani tabulky stav zakazky

CREATE SEQUENCE seq_ustavzakazky
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;
ALTER TABLE public.seq_ustavzakazky OWNER TO %firmaRole%;

CREATE TABLE ustavzakazky (
    idstavzakazky integer DEFAULT nextval('seq_ustavzakazky'::regclass) NOT NULL,
    lastupdate timestamp without time zone NOT NULL DEFAULT '2008-01-01 00:00:00'::timestamp without time zone,
    kod character varying(20) NOT NULL,
    nazev character varying(255) NOT NULL,
    nazeva character varying(255),
    nazevb character varying(255),
    nazevc character varying(255),
    popis text,
    poznam text,
    "show" boolean NOT NULL DEFAULT true,
    druhstavuzakk character varying(50),
    CONSTRAINT ustavzakazky_pkey PRIMARY KEY (idstavzakazky),
    CONSTRAINT ustavzakazky_unique_kod UNIQUE (kod)
);
ALTER TABLE ustavzakazky OWNER TO %firmaRole%;

-- pridani tabulky pro vyhodnoceni zakazky

CREATE SEQUENCE seq_uvyhzakazky
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;
ALTER TABLE public.seq_uvyhzakazky OWNER TO %firmaRole%;

CREATE TABLE uvyhzakazky (
    idvyhzakazky integer DEFAULT nextval('seq_uvyhzakazky'::regclass) NOT NULL,
    lastupdate timestamp without time zone NOT NULL DEFAULT '2008-01-01 00:00:00'::timestamp without time zone,
    kod character varying(20) NOT NULL,
    nazev character varying(255) NOT NULL,
    nazeva character varying(255),
    nazevb character varying(255),
    nazevc character varying(255),
    popis text,
    poznam text,
    "show" boolean NOT NULL DEFAULT true,
    platido integer NOT NULL DEFAULT 9999,
    platiod integer NOT NULL DEFAULT 0,
    CONSTRAINT uvyhzakazky_pkey PRIMARY KEY (idvyhzakazky),
    CONSTRAINT uvyhzakazky_unique_kod UNIQUE (kod)
);
ALTER TABLE uvyhzakazky OWNER TO %firmaRole%;


-- vytvoreni vazebnich tabulek udalosti <-> osoby, udalosti <-> kontakty
CREATE TABLE akon_audalost (
    idudalost integer NOT NULL,
    idkontakt integer NOT NULL,
    CONSTRAINT akon_audalost_pkey PRIMARY KEY (idudalost, idkontakt),
    CONSTRAINT akon_audalost_idudalost FOREIGN KEY (idudalost)
        REFERENCES audalosti(idudalost) MATCH SIMPLE
        ON UPDATE NO ACTION ON DELETE NO ACTION,
    CONSTRAINT akon_audalost_idkontakt FOREIGN KEY (idkontakt)
        REFERENCES akontakty(idkontakt) MATCH SIMPLE
        ON UPDATE NO ACTION ON DELETE NO ACTION
);
ALTER TABLE public.akon_audalost OWNER TO %firmaRole%;

CREATE TABLE wuziv_audalost (
    idudalost integer NOT NULL,
    iduzivatel integer NOT NULL,
    CONSTRAINT wuziv_audalost_pkey PRIMARY KEY (idudalost, iduzivatel),
    CONSTRAINT wuziv_audalost_idudalost FOREIGN KEY (idudalost)
        REFERENCES audalosti(idudalost) MATCH SIMPLE
        ON UPDATE NO ACTION ON DELETE NO ACTION,
    CONSTRAINT wuziv_audalost_iduzivatel FOREIGN KEY (iduzivatel)
        REFERENCES wuzivatel(iduzivatel) MATCH SIMPLE
        ON UPDATE NO ACTION ON DELETE NO ACTION
);
ALTER TABLE public.wuziv_audalost OWNER TO %firmaRole%;

---- Uprava stavajicich tabulek ----

-- pridani sloupcu do tabulky udalosti
ALTER TABLE audalosti
    ADD COLUMN celodenni boolean DEFAULT true,
    ADD COLUMN dokonceni timestamp without time zone,
    ADD COLUMN druhudalk character varying(50),
    ADD COLUMN pocetpriloh integer NOT NULL DEFAULT 0,
    ADD COLUMN predmet character varying(255),
    ADD COLUMN prioritak character varying(50),
    ADD COLUMN stavudalk character varying(50),
    ADD COLUMN termin timestamp without time zone,
    ADD COLUMN umisteni character varying(255),
    ADD COLUMN volno boolean DEFAULT true,
    ADD COLUMN zahajeni timestamp without time zone,
    ADD COLUMN idtypakt integer,
    ADD COLUMN iduzivatel integer,
    ADD COLUMN idzakazky integer,
    ADD CONSTRAINT audalosti_idtypakt FOREIGN KEY (idtypakt)
        REFERENCES atypakt(idtypakt) MATCH SIMPLE
        ON UPDATE NO ACTION ON DELETE NO ACTION,
    ADD CONSTRAINT audalosti_iduzivatel FOREIGN KEY (iduzivatel)
        REFERENCES wuzivatel(iduzivatel) MATCH SIMPLE
        ON UPDATE NO ACTION ON DELETE NO ACTION,
    ADD CONSTRAINT audalosti_idzakazky FOREIGN KEY (idzakazky)
        REFERENCES uzakazky(idzakazky) MATCH SIMPLE
        ON UPDATE NO ACTION ON DELETE NO ACTION;

CREATE INDEX audalosti_idtypakt_index ON audalosti USING btree (idtypakt);
CREATE INDEX audalosti_iduzivatel_index ON audalosti USING btree (iduzivatel);
CREATE INDEX audalosti_idzakazky_index ON audalosti USING btree (idzakazky);
CREATE INDEX audalosti_zahajeni_index ON audalosti USING btree (zahajeni);

ALTER TABLE uzakazky
    ADD COLUMN procVyh numeric(6,2) NOT NULL DEFAULT 0.0,
    ADD COLUMN termin timestamp without time zone,
    ADD COLUMN idstavzakazky integer,
    ADD COLUMN idtypzakazky integer,
    ADD COLUMN iduzivatel integer,
    ADD COLUMN idvyhzakazky integer,
    ADD CONSTRAINT uzakazky_idtypzakazky FOREIGN KEY (idtypzakazky)
        REFERENCES utypzakazky(idtypzakazky) MATCH SIMPLE
        ON UPDATE NO ACTION ON DELETE NO ACTION,
    ADD CONSTRAINT uzakazky_iduzivatel FOREIGN KEY (iduzivatel)
        REFERENCES wuzivatel(iduzivatel) MATCH SIMPLE
        ON UPDATE NO ACTION ON DELETE NO ACTION,
    ADD CONSTRAINT uzakazky_idstavzakazky FOREIGN KEY (idstavzakazky)
        REFERENCES ustavzakazky(idstavzakazky) MATCH SIMPLE
        ON UPDATE NO ACTION ON DELETE NO ACTION,
    ADD CONSTRAINT uzakazky_idvyhzakazky FOREIGN KEY (idvyhzakazky)
        REFERENCES uvyhzakazky(idvyhzakazky) MATCH SIMPLE
        ON UPDATE NO ACTION ON DELETE NO ACTION;

---- migrace dat do noveho schematu ----

-- prekopirujeme datzahaj a datkonec do odpovidajicich timestampu
UPDATE audalosti set zahajeni = datzahaj, dokonceni = datkonec;

-- vychozi druh udalosti bude udalost
UPDATE audalosti set druhudalk = 'druhUdal.udal';

create or replace function prevedUdalosti()
    returns void as $$
declare
    _udalost record;
    _idkontakt integer;
    _iduzivatel integer;
begin
    for _udalost in (select * from audalosti) loop
        if (_udalost.prijmeni is null) then
            continue;
        end if;

        select coalesce (
            -- podivame se do firmy jestli u ni neni clovek se stejnym primenim a jmenem
            (select idkontakt
                from akontakty ak
                where ak.idfirmy = _udalost.idfirmy and ak.prijmeni = _udalost.prijmeni
                and (_udalost.jmeno is null or ak.jmeno = _udalost.jmeno) limit 1),
            -- kdyz neni hledame kontakt se stejnym jmenem a prijmenim
            (select idkontakt
                from akontakty ak
                where ak.prijmeni = _udalost.prijmeni
                and (_udalost.jmeno is null or ak.jmeno = _udalost.jmeno) limit 1)
        ) into _idkontakt;

        -- nalezen kontakt propojime si ho s udalosti
        if (_idkontakt is not null) then
            insert into akon_audalost (idudalost, idkontakt)
                values (_udalost.idudalost, _idkontakt);
        -- podivame se jestli neexistuje uzivatel s timto jmenem
        else
            select iduzivatel into _iduzivatel
                from wuzivatel wu
                where wu.prijmeni = _udalost.prijmeni
                and (_udalost.jmeno is null or wu.jmeno = _udalost.jmeno) limit 1;
            -- existuje udelame z nej zodpovednou osobu
            if (_iduzivatel is not null) then
                update audalosti set iduzivatel = _iduzivatel
                    where idudalost = _udalost.idudalost;
            -- nenasli jsme ho nikde -> udelame z nej kontakt a propojime ho
            else
                insert into akontakty (prijmeni, jmeno, idfirmy)
                    values (_udalost.prijmeni, _udalost.jmeno, _udalost.idfirmy);
                insert into akon_audalost (idudalost, idkontakt)
                    values (_udalost.idudalost, (select max(idkontakt) from akontakty));
            end if;
        end if;

    end loop;
end;
$$ LANGUAGE plpgsql;
ALTER FUNCTION prevedUdalosti() OWNER TO %firmaRole%;

select prevedUdalosti();

drop function prevedUdalosti();

create or replace function inserttypakt()
    returns void as $$
begin
    -- vychozi typy aktivit pro cechy
    if ((select max(idstatuleg) from nnastav) = (select idstatu from astaty where uuid = 'CZ_CZ')) then
        insert into atypakt (druhudalk, kod, nazev)
                values ('druhUdal.udal', 'UDÁLOST', 'Událost');
        insert into atypakt (druhudalk, kod, nazev)
                    values ('druhUdal.ukol', 'ÚKOL', 'Úkol');
        insert into atypakt (druhudalk, kod, nazev)
            values ('druhUdal.pozn', 'POZNÁMKA', 'Poznámka');
    end if;

    -- vychozi typy aktivit pro slovaky
    if ((select max(idstatuleg) from nnastav) = (select idstatu from astaty where uuid = 'SK_SK')) then
        insert into atypakt (druhudalk, kod, nazev)
                values ('druhUdal.udal', 'UDÁLOSŤ', 'Událosť');
        insert into atypakt (druhudalk, kod, nazev)
                    values ('druhUdal.ukol', 'ÚKOL', 'Úkol');
        insert into atypakt (druhudalk, kod, nazev)
            values ('druhUdal.pozn', 'POZNÁMKA', 'Poznámka');
    end if;
end;
$$ LANGUAGE plpgsql;
ALTER FUNCTION inserttypakt() OWNER TO %firmaRole%;

select inserttypakt();

drop function inserttypakt();

-- odstraneni jiz nepouzivanych sloupcu
ALTER TABLE audalosti
        DROP COLUMN prijmeni,
        DROP COLUMN jmeno,
        DROP COLUMN datzahaj,
        DROP COLUMN datkonec;

-- vlozeni vychozich stavu zakazky
create or replace function insertstavatypzakazky()
    returns void as $$
begin
    -- pro cechy
    if ((select max(idstatuleg) from nnastav) = (select idstatu from astaty where uuid = 'CZ_CZ')) then
        -- vychozi stavy zakazky
        insert into ustavzakazky (druhstavuzakk, kod, nazev)
            values ('druhStavuZakazky.ziskavame', 'ZÍSKÁVÁME', 'O zakázce se jedná');
        insert into ustavzakazky (druhstavuzakk, kod, nazev)
            values ('druhStavuZakazky.mame', 'MÁME', 'Zakázku jsme získali');
        insert into ustavzakazky (druhstavuzakk, kod, nazev)
            values ('druhStavuZakazky.hotovo', 'HOTOVO', 'Zakázku jsme realizovali');
        insert into ustavzakazky (druhstavuzakk, kod, nazev)
            values ('druhStavuZakazky.storno', 'STORNO', 'Zakázka byla stornována');

        -- vychozi typy zakazky
        insert into utypzakazky (druhzakk, kod, nazev)
            values ('typZak.nevyrob', 'NEVÝROBNÍ', 'Nevýrobní typ zakázky');
        insert into utypzakazky (druhzakk, kod, nazev)
            values ('typZak.vyrob', 'VÝROBNÍ', 'Výrobní typ zakázky');
    end if;

    -- pro slovaky
    if ((select max(idstatuleg) from nnastav) = (select idstatu from astaty where uuid = 'SK_SK')) then
        -- vychozi stavy zakazky
        insert into ustavzakazky (druhstavuzakk, kod, nazev)
            values ('druhStavuZakazky.ziskavame', 'ZÍSKAVANIE', 'O objednávce sa jedná');
        insert into ustavzakazky (druhstavuzakk, kod, nazev)
            values ('druhStavuZakazky.mame', 'MÁME', 'Objednávku sme získali');
        insert into ustavzakazky (druhstavuzakk, kod, nazev)
            values ('druhStavuZakazky.hotovo', 'HOTOVO', 'Objednávku sme realizovali');
        insert into ustavzakazky (druhstavuzakk, kod, nazev)
            values ('druhStavuZakazky.storno', 'STORNO', 'Objednávka bola stornovaná');

        -- vychozi typy zakazky
        insert into utypzakazky (druhzakk, kod, nazev)
            values ('typZak.nevyrob', 'NEVÝROBNÁ', 'Nevýrobný typ objednávky');
        insert into utypzakazky (druhzakk, kod, nazev)
            values ('typZak.vyrob', 'VÝROBNÁ', 'Výrobný typ objednávky');
    end if;
end;
$$ LANGUAGE plpgsql;
ALTER FUNCTION insertstavatypzakazky() OWNER TO %firmaRole%;

select insertstavatypzakazky();

drop function insertstavatypzakazky();

create or replace function prevedzakazky()
    returns void as $$
declare
    _zakazka record;
    _iduzivatel integer;
begin
    for _zakazka in (select * from uzakazky) loop
        if (_zakazka.stavzakk = 'stavZak.vBehu') then
            update uzakazky
                set idstavzakazky = (select idstavzakazky from ustavzakazky sz where sz.druhstavuzakk = 'druhStavuZakazky.ziskavame')
                where idzakazky = _zakazka.idzakazky;
        elsif (_zakazka.stavzakk = 'stavZak.castHot') then
            update uzakazky
                set idstavzakazky = (select idstavzakazky from ustavzakazky sz where sz.druhstavuzakk = 'druhStavuZakazky.mame')
                where idzakazky = _zakazka.idzakazky;
        elsif (_zakazka.stavzakk = 'stavZak.hotovo') then
            update uzakazky
                set idstavzakazky = (select idstavzakazky from ustavzakazky sz where sz.druhstavuzakk = 'druhStavuZakazky.hotovo')
                where idzakazky = _zakazka.idzakazky;
        end if;

        if (_zakazka.typzakk = 'typZak.nevyrob') then
            update uzakazky
                set idtypzakazky = (select idtypzakazky from utypzakazky tz where tz.druhzakk = 'typZak.nevyrob')
                where idzakazky = _zakazka.idzakazky;
        elsif (_zakazka.typzakk = 'typZak.vyrob') then
            update uzakazky
                set idtypzakazky = (select idtypzakazky from utypzakazky tz where tz.druhzakk = 'typZak.vyrob')
                where idzakazky = _zakazka.idzakazky;
        end if;

        -- migrace vedouciho na zodpovedneho pracovnika
        if (_zakazka.vedouci is not null) then
            select iduzivatel into _iduzivatel
                    from wuzivatel wu
                    where wu.prijmeni = _zakazka.vedouci;
            if (_iduzivatel is not null) then
                update uzakazky set iduzivatel = _iduzivatel
                    where idzakazky = _zakazka.idzakazky;
            else
                update uzakazky set nazev = nazev || ' [' || vedouci || ']'
                    where idzakazky = _zakazka.idzakazky;
            end if;
        end if;
    end loop;
end;
$$ LANGUAGE plpgsql;
ALTER FUNCTION prevedzakazky() OWNER TO %firmaRole%;

select prevedzakazky();

drop function prevedzakazky();

-- odstraneni jiz nepouzivanych sloupcu
ALTER TABLE uzakazky
    DROP COLUMN vedouci,
    DROP COLUMN typZakK,
    DROP COLUMN stavZakK;

---- pridani priloh na udalost ----

-- pridani odkazu do wpriloh na udalosti
ALTER TABLE wpriloha
    ADD COLUMN idadrudalost integer,
    ADD CONSTRAINT wpriloha_idadrudalost FOREIGN KEY (idadrudalost)
      REFERENCES audalosti (idudalost) MATCH SIMPLE
      ON UPDATE NO ACTION ON DELETE NO ACTION;

CREATE INDEX wpriloha_idadrudalost_index ON wpriloha USING btree (idadrudalost);

CREATE OR REPLACE FUNCTION updatepocetpriloh(prilohaid integer, addon integer)
  RETURNS void AS
$BODY$
declare
    priloha record;
begin
    select * into priloha from wpriloha p where p.idpriloha = prilohaid;

    if (priloha.idreport is not null) then
        update wreport
        set pocetPriloh =   (select count(1)+addon
                            from wpriloha p
                            where p.idreport = priloha.idreport)
        where idreport = priloha.idreport;
    end if;


    if (priloha.idadresar is not null) then
        update aadresar
        set pocetPriloh =   (select count(1)+addon
                            from wpriloha p
                            where p.idadresar = priloha.idadresar)
        where idfirmy = priloha.idadresar;
    end if;

    if (priloha.idkontakt is not null) then
        update akontakty set pocetPriloh =  (select count(1)+addon
                                            from wpriloha p
                                            where p.idkontakt = priloha.idkontakt)
        where idkontakt = priloha.idkontakt;
    end if;

    if (priloha.idcenik is not null) then
        update ccenik set pocetPriloh = (select count(1)+addon
                                        from wpriloha p
                                        where p.idcenik = priloha.idcenik)
        where idcenik = priloha.idcenik;
    end if;

    if (priloha.iddoklfak is not null) then
        update ddoklfak set pocetPriloh =   (select count(1)+addon
                                            from wpriloha p
                                            where p.iddoklfak = priloha.iddoklfak)
        where iddoklfak = priloha.iddoklfak;
    end if;

    if (priloha.iddoklint is not null) then
        update ddoklint set pocetPriloh =   (select count(1)+addon
                                            from wpriloha p
                                            where p.iddoklint = priloha.iddoklint)
        where iddoklint = priloha.iddoklint;
    end if;

    if (priloha.iddoklobch is not null) then
        update ddoklobch set pocetPriloh =  (select count(1)+addon
                                            from wpriloha p
                                            where p.iddoklobch = priloha.iddoklobch)
        where iddoklobch = priloha.iddoklobch;
    end if;

    if (priloha.iddoklsklad is not null) then
        update ddoklsklad set pocetPriloh = (select count(1)+addon
                                            from wpriloha p
                                            where p.iddoklsklad = priloha.iddoklsklad)
        where iddoklsklad = priloha.iddoklsklad;
    end if;

    if (priloha.idosoba is not null) then
        update posoba set pocetPriloh = (select count(1)+addon
                                            from wpriloha p
                                            where p.idosoba = priloha.idosoba)
        where idosobahlav = priloha.idosoba;
    end if;

    if (priloha.idadrudalost is not null) then
        update audalosti set pocetPriloh = (select count(1)+addon
                                            from wpriloha p
                                            where p.idadrudalost = priloha.idadrudalost)
        where idudalost = priloha.idadrudalost;
    end if;
end;
$BODY$
  LANGUAGE 'plpgsql' VOLATILE
  COST 100;
ALTER FUNCTION updatepocetpriloh(integer, integer) OWNER TO %firmaRole%;

-- zvyseni verze databaze
insert into dbversion(iddbversion, dbversion, needreimport)
  values (nextval('seq_dbversion'), 360, 't');
