--------------------------------------------------------------------------------
-- Bug 21220 – [WinStrom] Přidat funkci objednání pro výrobu
--------------------------------------------------------------------------------

create type cenikmnoz as (idcenik integer, mnoz numeric, idcenikpuv integer, idsklad integer, idstred integer);

CREATE OR REPLACE FUNCTION findmnozzkusovniku(_idoteccenik integer, _idstred integer, _objmnoz numeric, leavesonly boolean)
  RETURNS SETOF cenikmnoz AS
$BODY$
declare
  _cenikmnoz cenikmnoz%rowtype;
  _subcenikmnoz cenikmnoz%rowtype;
  _kus record;
  _idsklad integer;
  _hasleaves boolean;
begin
  -- pro vsechny kusovniky majici za otce _idoteccenik
  for _kus in (select * from ckusovnik where idoteccenik = _idoteccenik) loop
    -- pokud nemuze byt uzel samostatnym korenem pridame ho mezi vysledky
    if (_kus.idcenik != _kus.idoteccenik) then
      _hasleaves = false;
      -- a projdeme jeho kusovniky, ze kterych se sklada, pridame si je mezi vysledky
      for _subcenikmnoz in select * from findmnozzkusovniku(_kus.idcenik, _idstred, _kus.mnoz * _objmnoz, leavesonly) loop
        _hasleaves = true;
        _subcenikmnoz.idcenikpuv := _idoteccenik;
        return next _subcenikmnoz;
      end loop;

      -- jedna se o surovinu a chceme jenom suroviny
      -- nebo chceme vsechny
      if ((_hasleaves = false and leavesonly)
          or leavesonly = false) then
        _cenikmnoz.idcenik := _kus.idcenik;
        _cenikmnoz.mnoz := _kus.mnoz * _objmnoz;
        _cenikmnoz.idcenikpuv := _idoteccenik;
        _cenikmnoz.idstred := _idstred;
        -- nalezneme preferovany sklad
        _idsklad = getprefidskladu(_idstred, (select idskupzboz from ccenik c where c.idcenik = _kus.idcenik), _kus.idcenik);
        _cenikmnoz.idsklad = _idsklad;
        return next _cenikmnoz;
      end if;
    end if;
  end loop;
  return;
end;
$BODY$
  LANGUAGE 'plpgsql' VOLATILE
  COST 100
  ROWS 1000;
ALTER FUNCTION findmnozzkusovniku(integer, integer, numeric, boolean) OWNER TO %firmaRole%;

CREATE OR REPLACE FUNCTION findmnozzkusovniku(_idoteccenik integer[], _idstred integer[], _objmnoz numeric[], leavesonly boolean)
  RETURNS SETOF cenikmnoz AS
$BODY$
declare
  _indx integer;
  _cenikmnoz cenikmnoz%rowtype;
begin
  if (array_upper(_idoteccenik, 1) != array_upper(_objmnoz, 1) or
	array_upper(_objmnoz, 1) != array_upper(_idstred, 1)) then
    raise exception 'Velikost poli musi byt shodna';
  end if;
  
  _indx = 0;
  while (_indx < array_upper(_idoteccenik, 1)) loop
    _indx = _indx + 1;
    for _cenikmnoz in select * from findmnozzkusovniku(_idoteccenik[_indx], _idstred[_indx], _objmnoz[_indx], leavesonly) loop
      return next _cenikmnoz;
    end loop;
  end loop;
end;
$BODY$
  LANGUAGE 'plpgsql' VOLATILE
  COST 100
  ROWS 1000;
ALTER FUNCTION findmnozzkusovniku(integer[], integer[], numeric[], boolean) OWNER TO %firmaRole%;

-- u existujicich polozek OBV nastavime neobjednavat
update dpolobch set objednatk = 'objednat.neobjednat' where modul = 'OBV';

-- oprava generovanych skladovych dokladu z obchodniho kolecka
update ddoklsklad set idsazbydphzakl = null, idsazbydphsniz = null, idsazbydphosv = null, 
    szbdphsniz = 0, szbdphzakl = 0, idclendph = null, idstatdph = null 
  where idsazbydphzakl is not null or idsazbydphsniz is not null or idsazbydphosv is not null 
    or szbdphsniz <> 0 or szbdphzakl <> 0 or idclendph is not null or idstatdph is not null;

-- zvyseni verze databaze
insert into dbversion(iddbversion, dbversion, needreimport)
  values (nextval('seq_dbversion'), 399, 't');
