--------------------------------------------------------------------------------
-- Bug 21164 – [WinStrom] Mozne prepsani castek obratu
--------------------------------------------------------------------------------

create or replace function navazobratynaucetobdobi(_platioddata date, _platidodata date, _iducetobdobi integer)
  returns void as $$
declare
  _obdobiod integer;
  _obdobido integer;
  _rokod integer;
  _rokdo integer;
begin
  select extract(month from _platioddata) into _obdobiod;
  select extract(year from _platioddata) into _rokod;
  select extract(month from _platidodata) into _obdobido;
  select extract(year from _platidodata) into _rokdo;

  update uobraty set
    lastupdate = now(),
    iducetobdobi = _iducetobdobi
  where (rok > _rokod or (rok = _rokod and obdobi >= _obdobiod))
    and (rok < _rokdo or (rok = _rokdo and obdobi <= _obdobido))
    -- logicka obdobi pouzivana pri uzaverce se navazujou pri uzaverce - tu jsme si nazacatku vynutili
    -- proto jim nebudeme aktualizovat obraty
    and obdobi not in (0, 99);
end;
$$
  LANGUAGE 'plpgsql' VOLATILE
  COST 100;
ALTER FUNCTION navazobratynaucetobdobi(date, date, integer) OWNER TO %firmaRole%;

-- uprava funkce na prepocet obratu aby umela opravit nenavazane obraty
CREATE OR REPLACE FUNCTION prepoctiobraty(iducetobd integer) RETURNS void AS $$ 
declare 
  platiOd date; 
  platiDo date; 
begin 
  select platioddata, platidodata into platiOd, platiDo from nucetobdobi where iducetobdobi = idUcetObd; 
 
  if (platiOd is not null and platiDo is not null) then 
    delete from uobraty where obdobi > 0 and (iducetobdobi = idUcetObd or 
      (obdobi < 99 and date(rok || '-' || obdobi || '-02') between platiOd and platiDo)); 
     
    -- zauctovani polozek faktur 
     perform createOrUpdateObraty(P.ucetni, P.storno, P.datucto, P.idZklMdUcet, P.idZklDalUcet, P.idMeny, P.idStred, 
         P.idCinnost, P.sumZkl, P.sumZklMen) from dpolfak as P  
         where P.ucetni = true and P.storno = false and P.datUcto between platiOd and platiDo; 
      
     perform createOrUpdateObraty(P.ucetni, P.storno, P.datucto, P.idDphMdUcet, P.idDphDalUcet, P.idMeny, P.idStred, 
         P.idCinnost, P.sumDph, P.sumDphMen) from dpolfak as P  
         where P.ucetni = true and P.storno = false and P.datUcto between platiOd and platiDo; 
        
     -- zauctovani polozek internich dokladu 
     perform createOrUpdateObraty(P.ucetni, P.storno, P.datucto, P.idZklMdUcet, P.idZklDalUcet, P.idMeny, P.idStred, 
         P.idCinnost, P.sumZkl, P.sumZklMen) from dpolint as P  
         where P.ucetni = true and P.storno = false and P.datUcto between platiOd and platiDo; 
        
     perform createOrUpdateObraty(P.ucetni, P.storno, P.datucto, P.idDphMdUcet, P.idDphDalUcet, P.idMeny, P.idStred, 
         P.idCinnost, P.sumDph, P.sumDphMen) from dpolint as P  
         where P.ucetni = true and P.storno = false and P.datUcto between platiOd and platiDo; 
                                 
     -- zauctovani polozek skladovych dokladu                      
     perform createOrUpdateObraty(P.ucetni, P.storno, P.datucto, P.idZklMdUcet, P.idZklDalUcet, P.idMeny, P.idStred, 
         P.idCinnost, P.sumZkl, P.sumZklMen) from dpolsklad as P  
         where P.ucetni = true and P.storno = false and P.datUcto between platiOd and platiDo; 
        
     perform createOrUpdateObraty(P.ucetni, P.storno, P.datucto, P.idDphMdUcet, P.idDphDalUcet, P.idMeny, P.idStred, 
         P.idCinnost, P.sumDph, P.sumDphMen) from dpolsklad as P  
         where P.ucetni = true and P.storno = false and P.datUcto between platiOd and platiDo; 
     
     -- danove naklady 
     perform createOrUpdateObraty(P.ucetni, false, P.datUcto, P.idZklMdUcet, P.idZklDalUcet, L.idMeny, P.idStred,  
          P.sumZkl, P.sumZklMen) from mdannakl as P left outer join mleasing as L on L.idleasing = P.idleasing  
          where P.ucetni = true and P.datUcto between platiOd and platiDo; 
           
     -- danove odpisy 
     -- odpisy majetku jsou vedeny vzdy v mistni mene, proto za menu dosazujeme -1 a za sumZklMen 0.0 
     perform createOrUpdateObraty(P.ucetni, false, P.datUcto, P.idZklMdUcet, P.idZklDalUcet, -1, P.idStred,  
          P.sumZkl, 0.0) from mdanodpisy as P 
          where P.ucetni = true and P.datUcto between platiOd and platiDo; 
      
     -- splatkovy kalendar 
     perform createOrUpdateObraty(P.ucetni, false, P.datUcto, P.idZklMdUcet, P.idZklDalUcet, L.idMeny, P.idStred,  
          P.sumZkl, P.sumZklMen) from msplatkal as P left outer join mleasing as L on L.idleasing = P.idleasing  
          where P.ucetni = true and P.datUcto between platiOd and platiDo; 
           
     perform createOrUpdateObraty(P.ucetni, false, P.datUcto, P.idDphMdUcet, P.idDphDalUcet, L.idMeny, P.idStred,  
          P.sumDph, P.sumDphMen) from msplatkal as P left outer join mleasing as L on L.idleasing = P.idleasing  
          where P.ucetni = true and P.datUcto between platiOd and platiDo; 
      
     -- ucetni odpisy 
     -- odpisy majetku jsou vedeny vzdy v mistni mene, proto za menu dosazujeme -1 a za sumZklMen 0.0 
     perform createOrUpdateObraty(P.ucetni, false, P.datUcto, P.idZklMdUcet, P.idZklDalUcet, -1, P.idStred,  
          P.sumZkl, 0.0) from mucetodpisy as P 
          where P.ucetni = true and P.datUcto between platiOd and platiDo; 
      
     -- udalosti 
     -- odpisy majetku jsou vedeny vzdy v mistni mene, proto za menu dosazujeme -1 a za sumZklMen 0.0 
     perform createOrUpdateObraty(P.ucetni, false, P.datUcto, P.idZklMdUcet, P.idZklDalUcet, -1, P.idStred,  
          P.sumZkl, 0.0) from mudalosti as P 
          where P.ucetni = true and P.datUcto between platiOd and platiDo; 
        
  end if; 
end;  
$$ LANGUAGE plpgsql;
ALTER FUNCTION public.prepoctiobraty(iducetobd integer) OWNER TO %firmaRole%;

create or replace function runupgrade() returns void as $$
declare
  _idUoPrep integer[];
begin
  -- vyber ucetnich obdobi k prepoctu
  select array(select distinct uo.iducetobdobi from uobraty obr
    inner join nucetobdobi uo on 1=1
    where uo.platioddata <= cast(obr.rok || '-' || obr.obdobi || '-01' as date)
      -- zde je opravdu den 01 platidodata je totiz vzdy poslednim dnem v mesici (28, 29, 30 nebo 31)
      -- podminka je stejne splnena a vyhneme se tak pretypovanim na unknown pokud den z mesice neexistuje
      and uo.platidodata >= cast(obr.rok || '-' || obr.obdobi || '-01' as date)
      and obr.iducetobdobi is null
      -- obraty s logickym obdobim pro uzaverku nenavazujeme
      and obr.obdobi not in (0, 99))
  into _idUoPrep;

  -- nenavazane obraty navazeme do jejich ucetniho obdobi
  update uobraty obr
  set iducetobdobi = uo.iducetobdobi
  from nucetobdobi uo
  where uo.platioddata <= cast(obr.rok || '-' || obr.obdobi || '-01' as date)
    -- zde je opravdu den 01 platidodata je totiz vzdy poslednim dnem v mesici (28, 29, 30 nebo 31)
    -- podminka je stejne splnena a vyhneme se tak pretypovanim na unknown pokud den z mesice neexistuje
    and uo.platidodata >= cast(obr.rok || '-' || obr.obdobi || '-01' as date)
    and obr.iducetobdobi is null
    -- obraty s logickym obdobim pro uzaverku nenavazujeme
    and obr.obdobi not in (0, 99);

  -- mel nenavazane, coalesce vraci prvni nenulovou hodnotu, zde protoze 1 prvek prazdneho pole je null -> ERROR: lower bound of FOR loop cannot be null
  for i in coalesce(array_lower(_idUoPrep,1), 0) .. coalesce(array_upper(_idUoPrep,1), -1) loop
    -- nenavazane protoze se nemaji k cemu navazat nebudeme prepocitavat
    if (_idUoPrep[i] is not null) then
      perform prepoctiobraty(_idUoPrep[i]);
    end if;
  end loop;
end;
$$ LANGUAGE plpgsql;
ALTER FUNCTION runupgrade() OWNER TO %firmaRole%;

select runupgrade();

drop function runupgrade();

-- osetreni, aby se navazobratynaucetobdobi nevolalo vicekrat nez je nutne
CREATE OR REPLACE FUNCTION nucetniobdobiupdate()
  RETURNS trigger AS
$BODY$
declare
  minod date;
  maxod date;
  mindo date;
  maxdo date;
  iduo integer;
begin
  if (OLD.platioddata < NEW.platioddata) then
    minod = OLD.platioddata;
    maxod = NEW.platioddata;
    -- u obratu prvniho mesice zmeneneho obdobi nechceme nastavit iduo na null
    maxod = maxod - interval '1 month';
    iduo = null;
  else
    maxod = OLD.platioddata;
    minod = NEW.platioddata;
    iduo = NEW.iducetobdobi;
  end if;

  if (OLD.platioddata != NEW.platioddata) then
    execute navazobratynaucetobdobi(minod, maxod, iduo);
  end if;

  if (OLD.platidodata < NEW.platidodata) then
    mindo = OLD.platidodata;
    maxdo = NEW.platidodata;
    iduo = NEW.iducetobdobi;
  else
    maxdo = OLD.platidodata;
    mindo = NEW.platidodata;
    -- u obratu posledniho mesice zmeneneho obdobi nechceme nastavit iduo na null
    mindo = mindo + interval '1 month';
    iduo = null;
  end if;

  if (OLD.platidodata != NEW.platidodata) then
    execute navazobratynaucetobdobi(mindo, maxdo, iduo);
  end if;

  return NEW;
end;
$BODY$
  LANGUAGE 'plpgsql' VOLATILE
  COST 100;
ALTER FUNCTION nucetniobdobiupdate() OWNER TO %firmaRole%;

-- zvyseni verze databaze
insert into dbversion(iddbversion, dbversion, needreimport)
  values (nextval('seq_dbversion'), 403, 't');
