--------------------------------------------------------------------------------
-- Bug 21732 – [WinStrom] činnost a zakázka se z faktury nepřenáší do skladového dokladu
--------------------------------------------------------------------------------

CREATE OR REPLACE FUNCTION createPolSkladSada(_idPolDokl integer, _modul character varying)
  RETURNS void AS
$BODY$
declare
  _polDokl record;
  _doklad record;
  _polozkaSady record;
  _skladKarta record;
  _cenik record;
  _sklad record;
  _idDoklSklad integer;
  _idDoklad integer;
  _idMeny integer;
  _idUcetObdobi integer;
  _idKarty integer;
  _idPolSklad integer;
  _idZklMdUcet integer;
  _idZklDalUcet integer;
  _i integer;
  _cisRad integer;
  _typDoklK character varying;
  _mnozMj numeric(19,6);
  _sumZkl numeric(19,6);
  _sumZklMen numeric(19,6);
  _cenaMj numeric(19,6);
  _kurz numeric(19,6);
  _isCiziMena boolean;
  _nasobic integer;
  _idSkladu integer;
  _kopZklMdUcet boolean;
  _kopZklDalUcet boolean;
begin

  if(_modul like 'FAV' or _modul like 'FAP') then
    select * into _polDokl from dPolFak where idPolFak = _idPolDokl;
    _idDoklad = _polDokl.idDoklFak;
    select * into _doklad from dDoklFak where idDoklFak = _idDoklad;
    select typDoklK into _typDoklK from dTypDokl where idTypDokl = (select idTypDokl from dDoklFak where idDoklFak = _polDokl.idDoklFak);
  else
    select * into _polDokl from dPolInt where idPolInt = _idPolDokl;
    _idDoklad = _polDokl.idDoklInt;
    select * into _doklad from dDoklInt where idDoklInt = _idDoklad;
    _typDoklK = null; -- pro pokladnu dobropisy nejsou
  end if;

  select findParovanyDoklSklad(_idDoklad, _modul) into _idDoklSklad;

  if(_idDoklSklad is null) then
    select createDoklSklad(_idPolDokl, _idDoklad, _modul) into _idDoklSklad;
    if(_idDoklSklad is null) then
      return;
    end if;
  end if;

  select getkurzfromdoklad(_idDoklSklad, _polDokl.datVyst) into _kurz;
  select isDoklSkladMenaCizi(_idDoklSklad, _polDokl.datVyst) into _isCiziMena;
  select idMeny into _idMeny from dDoklSklad where idDoklSklad = _idDoklSklad;

  select * into _sklad from dBsp where idBsp = _polDokl.idBspSkl;

  _i = 0;

  FOR _polozkaSady IN
    select * from cSady where idCenikSada = _polDokl.idCenik
  LOOP
    select * into _cenik from cCenik where idCenik = _polozkaSady.idCenik;

    if(_cenik.skladove) then
      _i = _i + 1;
      _cisRad = 100 * _polDokl.cisRad + _i;

      select findIdUcetniObdobiKDatu(_polDokl.datVyst) into _iducetobdobi;

      _idSkladu = _polDokl.idBspSkl;

      select * into _skladKarta from skarty
        where idSkladu = _idSkladu and idCenik = _polozkaSady.idCenik and idUcetObdobi = _idUcetObdobi;

      -- pokud neexistuje skladova karta pokusim se ji najit pomoci mapovani skladu
      if(_skladKarta is null) then
        select getPrefIdSkladu(_polDokl.idStred, _cenik.idSkupZboz, _cenik.idCenik) into _idSkladu;
        select * into _skladKarta from skarty
          where idSkladu = _idSkladu and idCenik = _cenik.idCenik and idUcetObdobi = _idUcetObdobi;
        if(_skladKarta is null) then
          -- pokud ani podle mapovani skladu se skladova karta nenajde tak vyhazuju vyjimku
          raise exception '0007:Skladová karta neexistuje pro ceník % a sklad % a nebo je špatně nastaveno mapovaní skladu.', _cenik.nazev, _sklad.nazev;
        end if;
      else
        _idSkladu = _skladKarta.idSkladu;
      end if;
      _nasobic = 1;
      if(_typDoklK like 'typDokladu.dobropis') then
        _nasobic = -1;
      end if;
      _mnozMj = _polozkaSady.mnozMj * _polDokl.mnozMj * _nasobic;
      _cenaMj = _skladKarta.prumCenaTuz / _kurz;
      _sumZkl = _mnozMj * _skladKarta.prumCenaTuz;
      if(_isCiziMena) then
        _sumZklMen = _mnozMj * _cenaMj;
      else
        _sumZklMen = 0.0;
      end if;

      -- nastaveni uctu pro polSklad vydej
      _kopZklMdUcet = false;
      _kopZklDalUcet = false;
      if(_typDoklK like 'typDokladu.dobropis') then
        -- dal ucet je ze skupiny zbozi nebo z hlavicky pokud neni ve skupine zbozi nastaven
        select idProtiUcetSklV into _idZklDalUcet from cSkupZboz where idSkupZboz = _cenik.idSkupZboz;
        if(_idZklDalUcet is null) then
          select idProtiUcet into _idZklDalUcet from dDoklSklad where idDoklSklad = _idDoklSklad;
          --pokud je z hlavicky tak se bude kopirovat
          _kopZklDalUcet = true;
        end if;
        -- md ucet musi byt primarni ucet skladu ze ktereho se kousky sady vydavaji
        select idPrimUcet into _idZklMdUcet from dBsp where idBsp = _idSkladu;
      else
        -- md ucet je ze skupiny zbozi nebo z hlavicky pokud neni ve skupine zbozi nastaven
        select idProtiUcetSklV into _idZklMdUcet from cSkupZboz where idSkupZboz = _cenik.idSkupZboz;
        if(_idZklMdUcet is null) then
          select idProtiUcet into _idZklMdUcet from dDoklSklad where idDoklSklad = _idDoklSklad;
          --pokud je z hlavicky tak se bude kopirovat
          _kopZklMdUcet = true;
        end if;
        -- dal ucet musi byt primarni ucet skladu ze ktereho se kousky sady vydavaji
        select idPrimUcet into _idZklDalUcet from dBsp where idBsp = _idSkladu;
      end if;

      select nextval('seq_dpolsklad') into _idPolSklad;
      insert into dpolsklad(idPolSklad, lastUpdate, idcenik, idBspSkl, idkarty, datVyst, datUcto, modul, typPolozkyK, idDoklSklad,
          mnozMj, cenaMj, cenaMjPoriz, sumZkl, sumZklMen, idZklMdUcet, idZklDalUcet, kopZklMdUcet, kopZklDalUcet, baleniid, mnozBaleni, idmj,
          nazev, nazevA, nazevB, nazevC, sarze, expirace, cisRad, ucetni,
          idStred, idZakazky, idMeny, zdrojProFak, idCinnost)
        values (_idPolSklad, LOCALTIMESTAMP, _polozkaSady.idcenik, _idSkladu, _skladKarta.idKarty, _polDokl.datVyst, _polDokl.datVyst, 'SKL', 'typPolozky.katalog', _idDoklSklad,
          _mnozMj, _cenaMj, _cenaMj, _sumZkl, _sumZklMen, _idZklMdUcet, _idZklDalUcet, _kopZklMdUcet, _kopZklDalUcet, _polDokl.baleniid, 0.0, _cenik.idmj1,
          _cenik.nazev, _cenik.nazevA, _cenik.nazevB, _cenik.nazevC, null, null, _cisRad, true,
          _polDokl.idStred, _polDokl.idZakazky, _idMeny, false, _polDokl.idCinnost);

      insert into dvazebtab (castka, idtabA, idtabB, modulA, modulB, storno, typVazbyK)
        values (0.0, _idPolDokl, _idPolSklad, _modul, 'SKL', false, 'typVazbyDokl.polozkaSklSada');
    end if;
  END LOOP;

  execute sprepoctiSumyDokladu(_idDoklSklad);
end;
$BODY$
  LANGUAGE 'plpgsql' VOLATILE
  COST 100;
ALTER FUNCTION createPolSkladSada(integer, character varying) OWNER TO %firmaRole%;

--drop function createPolSklad(integer, character varying);
CREATE OR REPLACE FUNCTION createPolSklad(_idPolDokl integer, _modul character varying) RETURNS integer AS
$BODY$
declare
  _idDoklSklad integer;
  _idDoklad integer;
  _polDokl record;
  _iducetobdobi integer;
  _idkarty integer;
  _idPolSklad integer;
  _cenaMj numeric(19,6);
  _cenaMjTuz numeric(19,6);
  _typPohybuK character varying;
  _prumCenaTuz numeric(19,6);
  _sumZkl numeric(19,6);
  _sumZklMen numeric(19,6);
  _kurz numeric(19,6);
  _idZklMdUcet integer;
  _idZklDalUcet integer;
  _kopZklMdUcet boolean;
  _kopZklDalUcet boolean;
  _sklad record;
  _typDoklK character varying;
  _mnozMj numeric(19,6);
  _isCiziMena boolean;
  _idMeny integer;
  _skupZboz record;
begin
  _kopZklMdUcet =false;
  _kopZklDalUcet = false;
  if(_modul like 'FAV' or _modul like 'FAP') then
    select * into _polDokl from dPolFak where idPolFak = _idPolDokl;
    _idDoklad = _polDokl.idDoklFak;
    select typDoklK into _typDoklK from dTypDokl where idTypDokl = (select idTypDokl from dDoklFak where idDoklFak = _polDokl.idDoklFak);
  else
    select * into _polDokl from dPolInt where idPolInt = _idPolDokl;
    _idDoklad = _polDokl.idDoklInt;
    _typDoklK = null; -- pro pokladnu dobropisy nejsou
  end if;

  select findParovanyDoklSklad(_idDoklad, _modul) into _idDoklSklad;

  if(_idDoklSklad is null) then
    select createDoklSklad(_idPolDokl, _idDoklad, _modul) into _idDoklSklad;
    if(_idDoklSklad is null) then
      return null;
    end if;
  end if;

  select findIdUcetniObdobiKDatu(_polDokl.datVyst) into _iducetobdobi;
  select idkarty into _idkarty from skarty where idcenik = _polDokl.idcenik and idskladu = _polDokl.idbspskl and iducetobdobi = _iducetobdobi;
  select * into _sklad from dBsp where idBsp = _polDokl.idBspSkl;

  select typPohybuK into _typPohybuK from dDoklSklad where idDoklSklad = _idDoklSklad;
  select getkurzfromdoklad(_idDoklSklad, _polDokl.datVyst) into _kurz;
  select prumCenaTuz into _prumCenaTuz from sKarty where idKarty = _idKarty;
  select isDoklSkladMenaCizi(_idDoklSklad, _polDokl.datVyst) into _isCiziMena;
  select idMeny into _idMeny from dDoklSklad where idDoklSklad = _idDoklSklad;

  if(_typPohybuK like 'typPohybu.vydej') then
    -- vsechny dobropisy generuji prijemky
    --if(_typDoklK like 'typDokladu.dobropis') then
    --  _mnozMj = -_polDokl.mnozMj;
    --else
      _mnozMj = _polDokl.mnozMj;
    --end if;
    -- pokud vydavam ze skladu tak ma byt pouzita prumerna cena
    _cenaMj = _prumCenaTuz / _kurz;
    _sumZkl = _prumCenaTuz * _mnozMj;
    if(_isCiziMena) then
      _sumZklMen = _mnozMj * _cenaMj;
    else
      _sumZklMen = 0.0;
    end if;
    -- nastaveni uctu pro polSklad vydej
    select * into _skupZboz from cskupzboz where idskupzboz = (select idskupzboz from ccenik where idcenik = _polDokl.idCenik);
    if(_skupZboz.idProtiUcetSklV is not null) then
      -- pokud je ve skupine zbozi nastaven protiucet pro vydej ze skladu pouzije se ten a kopirovani se necha na false;
      _idZklMdUcet = _skupZboz.idProtiUcetSklV;
    else
      -- pokud neni ve skupine zbozi nastaven protiucet tak vemu protiucet z hlavicky a dam kopirovat na true
      select idProtiUcet into _idZklMdUcet from dDoklSklad where idDoklSklad = _idDoklSklad;
      _kopZklMdUcet = true;
    end if;
    _idZklDalUcet = _sklad.idPrimUcet;
  else
    if(_typDoklK like 'typDokladu.dobropis' and _modul like 'FAV') then
      -- pokud se jedna o dobropis tak doplnuji nakupni ceny;
      _mnozMj = -_polDokl.mnozMj;
      --_cenaMjTuz = _prumCenaTuz;
      _cenaMjTuz = _polDokl.cenaMjNakup;
      if(_cenaMjTuz = 0.0) then
        _cenaMjTuz = (select nakupCena from ccenik where idCenik = _polDokl.idCenik);
        if(_cenaMj = 0.0) then
          _cenaMj = _polDokl.cenaMj * _kurz;
        end if;
      end if;
      _cenaMj = _cenaMjTuz / _kurz;
      _sumZkl = _mnozMj * _cenaMjTuz;
      if(_isCiziMena) then
        _sumZklMen = _mnozMj * _cenaMj;
      else
        _sumZklMen = 0.0;
      end if;
      select * into _skupZboz from cskupzboz where idskupzboz = (select idskupzboz from ccenik where idcenik = _polDokl.idCenik);
      if(_skupZboz.idProtiUcetSklV is not null) then
        -- pokud je ve skupine zbozi nastaven protiucet pro vydej ze skladu pouzije se ten a kopirovani se necha na false;
        _idZklDalUcet = _skupZboz.idProtiUcetSklV;
      else
        -- pokud neni ve skupine zbozi nastaven protiucet tak vemu protiucet z hlavicky a dam kopirovat na true
        select idProtiUcet into _idZklDalUcet from dDoklSklad where idDoklSklad = _idDoklSklad;
        _kopZklDalUcet = true;
      end if;
    else
      -- pokud na sklad prijmam zkopiruji se ceny z faktury
      _mnozMj = _polDokl.mnozMj;
      _sumZkl = _polDokl.sumZkl;
      if(_mnozMj <> 0.0) then
        _cenaMj = _sumZkl / _mnozMj;
      else
        _cenaMj = _sumZkl;
      end if;
      _cenaMj = _cenaMj / _kurz;
      if(_isCiziMena) then
        if(_mnozMj <> 0.0) then
          _sumZklMen = _cenaMj * _mnozMj;
        else
          _sumZklMen = _cenaMj;
        end if;
      else
        _sumZklMen = 0.0;
      end if;
      _idZklDalUcet = _polDokl.idZklMdUcet;
    end if;
    -- nastaveni uctu pro polSklad prijem
    _idZklMdUcet = _sklad.idPrimUcet;
  end if;

  select nextval('seq_dpolsklad') into _idPolSklad;
  insert into dpolsklad(idPolSklad, lastUpdate, idcenik, idbspskl, idkarty, datVyst, datUcto, modul, typPolozkyK, idDoklSklad,
        mnozMj, cenaMj, cenaMjPoriz, sumZkl, sumZklMen, idZklMdUcet, idZklDalUcet, kopZklMdUcet, kopZklDalUcet, baleniid, mnozBaleni, idmj,
        nazev, nazevA, nazevB, nazevC, sarze, expirace, cisRad, ucetni,
        idStred, idZakazky, idMeny, zdrojProFak, idCinnost)
      values (_idPolSklad, LOCALTIMESTAMP, _polDokl.idcenik, _polDokl.idbspskl, _idkarty, _polDokl.datVyst, _polDokl.datVyst, 'SKL', 'typPolozky.katalog', _idDoklSklad,
        _mnozMj, _cenaMj, _cenaMj, _sumZkl, _sumZklMen, _idZklMdUcet, _idZklDalUcet, _kopZklMdUcet, _kopZklDalUcet, _polDokl.baleniid, _polDokl.mnozBaleni, _polDokl.idmj,
        _polDokl.nazev, _polDokl.nazevA, _polDokl.nazevB, _polDokl.nazevC, _polDokl.sarze, _polDokl.expirace, _polDokl.cisRad, true,
        _polDokl.idStred, _polDokl.idZakazky, _idMeny, false, _polDokl.idCinnost);

  insert into dvazebtab (castka, idtabA, idtabB, modulA, modulB, storno, typVazbyK)
       values (0.0, _idPolDokl, _idPolSklad, _modul, 'SKL', false, 'typVazbyDokl.polozkaSkl');

  -- prepocteni sum na dokladu
  execute sprepoctiSumyDokladu(_idDoklSklad);

  return _idPolSklad;
end;
$BODY$
  LANGUAGE 'plpgsql' VOLATILE
  COST 100;
ALTER FUNCTION createPolSklad(integer, character varying) OWNER TO %firmaRole%;

--DROP FUNCTION createDoklSklad(integer, integer);
CREATE OR REPLACE FUNCTION createdoklsklad(_idpoldokl integer, _iddoklad integer, _modul character varying)
  RETURNS integer AS
$BODY$
declare
  _doklad record;
  _polDokl record;
  _doklSklad record;
  _idDoklSklad integer;
  _cisDoklad character varying;
  _idrada integer;
  _rada record;
  _sklad record;
  _idUcetObd integer;
  _radaRok record;
  _typPohybuSklad character varying;
  _idTypUcOp integer;
  _cisloDokladuCursor refCursor;
  _typDoklFak record;
  _typDoklSkl record;
  _idUcetSklad integer;
  _idProtiUcet integer;
  _typUcOp record;
  _kurz numeric(19,6);
  _idMenyTuzemsko integer;
  _idMenySklad integer;
  _kurzMnozstvi integer;
  _typVazby character varying;
  _stavSklad character varying;
  _typPohybuSkladK character varying;
  _isDobropis boolean;
begin
  if(_modul like 'POK') then
    select * into _doklad from dDoklInt where idDoklInt = _idDoklad;
    select * into _polDokl from dPolInt where idPolInt = _idPolDokl;
    _stavSklad = 'stavSklad.vygenPokl';
  elsif (_modul like 'FAV' or _modul like 'FAP') then
    select * into _doklad from dDoklFak where idDoklFak = _idDoklad;
    select * into _polDokl from dPolFak where idPolFak = _idPolDokl;
    _stavSklad = 'stavSklad.vygenFakt';
  else
    select * into _doklad from dDoklSklad where idDoklSklad = _idDoklad;
    select * into _polDokl from dPolSklad where idPolSklad = _idPolDokl;
    _stavSklad = 'stavSklad.vyroba';
  end if;

  --select * into _sklad from dTypDokl where idTypDokl = _polDokl.idskladu;
  select * into _sklad from dBsp where idBsp = _polDokl.idBspSkl;
  select * into _typDoklFak from dTypDokl where idTypDokl = _doklad.idTypDokl;

  if(_doklad.idTypDoklSkl is null) then
    if(_typDoklFak.typDoklK like 'typDokladu.dobropis') then
      _isDobropis = true;
    else
      _isDobropis = false;
    end if;
    if((_modul like 'FAV' and _isDobropis) or (_modul like 'FAP')) then
      --pro dobropisy vydanych faktur a prijate faktury budeme mit prijemku
      select * into _typDoklSkl from dTypDokl where idTypDokl = _typDoklFak.idtypprotidokladuprijem;
    elsif((_modul like 'FAV' and not _isDobropis)) then
      --pro dobropisy prijatych faktur a vydane faktury budeme mit vydejku
      select * into _typDoklSkl from dTypDokl where idTypDokl = _typDoklFak.idtypprotidokladuvydej;
    elsif(_modul like 'POK' and _doklad.typPohybuK like 'typPohybu.prijem') then
      --pro prijmovy pokladni doklad budeme mit vydejku
      select * into _typDoklSkl from dTypDokl where idTypDokl = _typDoklFak.idtypprotidokladuvydej;
    elsif(_modul like 'POK' and _doklad.typPohybuK like 'typPohybu.vydej') then
      --pro vydajovy pokladni doklad budeme mit prijemku
      select * into _typDoklSkl from dTypDokl where idTypDokl = _typDoklFak.idtypprotidokladuprijem;
    elsif(_modul like 'SKL') then
      -- vyroba
      select * into _typDoklSkl from dTypDokl where idTypDokl = _typDoklFak.idtypprotidokladuvydej;
    else
      raise exception 'createdoklsklad:Nespravny modul %', _modul;
    end if;
    if(_typDoklSkl is null) then
      return null;
    end if;
  else
    select * into _typDoklSkl from dTypDokl where idTypDokl = _doklad.idTypDoklSkl;
  end if;

  -- urceni spravneho typu pohybu
  if((_typDoklFak.modul like 'FAV') and (_typDoklFak.typDoklK like 'typDokladu.dobropis')) then
    _typPohybuSklad = _doklad.typPohybuK;
  else
    if(_doklad.typPohybuK like 'typPohybu.prijem') then
      _typPohybuSklad = 'typPohybu.vydej';
    else
      _typPohybuSklad = 'typPohybu.prijem';
    end if;
  end if;

  -- doplneni zbyvajicich veci zavislich na typu pohybu
  if(_typPohybuSklad like 'typPohybu.prijem') then
    _idrada = _typDoklSkl.idRadyPrijem;
    if(_idrada is null) then
      _idRada = _sklad.idRadyPrijem;
    end if;
    if(_idRada is null) then
      raise exception 'Neni odkud natahnout dokladovou radu.';
    end if;
    _idTypUcOp = _typDoklSkl.idTypUcOpP;
    if(_modul = 'FAV' or _modul = 'FAP' or _modul = 'POK') then
      _typPohybuSkladK = 'typPohybuSklad.prijemPoFak';
    else
      _typPohybuSkladK = 'typPohybuSklad.prijemVyrob';
    end if;
  else
    _idrada = _typDoklSkl.idRadyVydej;
    if(_idrada is null) then
      _idRada = _sklad.idRadyVydej;
    end if;
    if(_idRada is null) then
      raise exception 'Neni odkud natahnout dokladovou radu.';
    end if;
    _idTypUcOp = _typDoklSkl.idTypUcOpV;
    if(_modul = 'FAV' or _modul = 'FAP' or _modul = 'POK') then
      _typPohybuSkladK = 'typPohybuSklad.vydejPoFak';
    else
      _typPohybuSkladK = 'typPohybuSklad.vydejVyrob';
    end if;
  end if;

  _idUcetSklad = _sklad.idPrimUcet;
  select * into _typUcOp from utypucop where idtypucop = _idTypUcOp;
  if(_typPohybuSklad like 'typPohybu.prijem') then
    _idProtiUcet = _typUcOp.idProtiucetP;
  else
    _idProtiUcet = _typUcOp.idProtiucetV;
  end if;
  if(_modul = 'FAP') then
    _idProtiUcet = _doklad.idProtiUcet;
  end if;

  select * into _rada from drady where idRady = _idrada;
  select findIdUcetniObdobiKDatu(_polDokl.datVyst) into _idUcetObd;
  select * into _radaRok from dRadyRok where iducetobdobi = _idUcetObd and idrady = _idrada;

  select nextval('seq_ddoklsklad') into _idDoklSklad;
  select dgetcisdokl(null, _idrada, _idUcetObd, 'SKL', _typPohybuSklad) into _cisloDokladuCursor;
  fetch _cisloDokladuCursor into _cisDoklad;

  -- ziskani kurzu pro menu skladu
  -- select getKurzFromSklad(_polDokl.idSkladu, _polDokl.datVyst) into _kurz;
  -- nejde pouzit protoze potrebuju jak kurz tak kurzove mnozstvi a ne prepocteny nna kurzove mnozstvi 1
  select idMeny into _idMenyTuzemsko from nNastav where _polDokl.datVyst >= PlatiOdData order by PlatiOdData desc limit 1;
  if (_idMenyTuzemsko is null) then
    select idMeny into _idMenyTuzemsko from nNastav where PlatiOdData is null limit 1;
  end if;
  --select idMeny into _idMenySklad from dTypDokl where idTypDokl = _polDokl.idSkladu;

  if(_sklad.idMeny is null or _sklad.idMeny = _idMenyTuzemsko) then
    _kurz = 0.0;
    _kurzMnozstvi = 1;
  else
    select nbStred, kurzMnozstvi into _kurz, _kurzMnozstvi from uKurzy
      where idMeny = _sklad.idMeny and _polDokl.datVyst >= PlatiOdData and typkurzuk = 'typKurzu.obecny'
      order by PlatiOdData desc limit 1;
    if(_kurz is null) then
      select nbStred, kurzMnozstvi into _kurz, _kurzMnozstvi from uKurzy
        where idMeny = _sklad.idMeny and PlatiOdData is null and typkurzuk = 'typKurzu.obecny'
        limit 1;
    end if;
    if(_kurz is null) then
      raise exception '0001:getKurzFromSklad : neexistuje kurz!';
    end if;
  end if;

  -- vytvoreni noveho skladoveho dokladu
  insert into dDoklSklad (idDoklSklad, lastUpdate, datVyst, datUcto, kod, modul, typPohybuK, idTypDokl, idBsp,ucetni,
         idFirmy, nazFirmy, ulice, mesto, psc, idStatu, ic, dic, vatId,
         idMur, idKrajUrc, idStatUrc, idTypUcOp, cisObj, cisDodak, doprava, idDruhDopr, popis, poznam,
         idStred, idZakazky, idMeny, kurz, kurzMnozstvi,
         eanKod, idstatpuvod, idstatodesl,
         idPrimUcet, idProtiUcet, idRady, idPolRady, zdrojProFak, stavSkladK , typPohybuSkladK,
         idCinnost
         )
       values (_idDoklSklad, LOCALTIMESTAMP, _doklad.datVyst, _doklad.datVyst, _cisDoklad, 'SKL', _typPohybuSklad, _typDoklSkl.idTypDokl, _sklad.idBsp, _typDoklSkl.ucetni,
         _doklad.idFirmy, _doklad.nazFirmy, _doklad.ulice, _doklad.mesto, _doklad.psc, _doklad.idStatu, _doklad.ic, _doklad.dic, _doklad.vatId,
         _doklad.idMur, _doklad.idKrajUrc, _doklad.idStatUrc, _idTypUcOp, _doklad.cisObj, null, _doklad.doprava, _doklad.idDruhDopr, _doklad.popis, _doklad.poznam,
         _doklad.idStred, _doklad.idZakazky, _sklad.idMeny, _kurz, _kurzMnozstvi,
         _doklad.eanKod, _doklad.idstatpuvod, _doklad.idstatodesl,
         _idUcetSklad, _idProtiUcet, _idRada, _radaRok.idRadyRok, false, _stavSklad, _typPohybuSkladK,
         _doklad.idCinnost
         );


  if (_modul = 'SKL') then
    _typVazby = 'typVazbyDokl.hlavaSklKus';
  else
    _typVazby = 'typVazbyDokl.hlavaSkl';
  end if;

  insert into dvazebtab (castka, idtabA, idtabB, modulA, modulB, storno, typVazbyK)
       values (0.0, _idDoklad, _idDoklSklad, _doklad.modul, 'SKL', false, _typVazby);

  return _idDoklSklad;
end;
$BODY$
  LANGUAGE 'plpgsql' VOLATILE
  COST 100;
ALTER FUNCTION createdoklsklad(integer, integer, character varying) OWNER TO %firmaRole%;

-- zvyseni verze databaze
insert into dbversion(iddbversion, dbversion, needreimport, appversion, upgradetime)
  values (nextval('seq_dbversion'), 427, 't', %appVersion%, clock_timestamp());
