------------------------------------------------------------------------------------------------
-- Bug 23852 – [WinStrom] Pri zmene kodu na hlavicce vyrobniho cisla se nezmeni kod na pohybech
------------------------------------------------------------------------------------------------

-- Function: svyrobnicislocreatehlav()
-- DROP FUNCTION svyrobnicislocreatehlav();
CREATE OR REPLACE FUNCTION svyrobnicislohlavupdateKod()
  RETURNS trigger AS
$BODY$ 
declare 
begin 
  if(NEW.kod != OLD.kod) then
    update svyrobnicislo set kod = NEW.kod where idvyrobnicislohlav = NEW.idvyrobnicislohlav;
  end if;
  return NEW; 
end 
$BODY$
  LANGUAGE 'plpgsql' VOLATILE
  COST 100;
ALTER FUNCTION svyrobnicislohlavupdateKod() OWNER TO %firmaRole%;

CREATE TRIGGER svyrobnicislohlavupdatekod
  AFTER UPDATE
  ON svyrobnicislohlav
  FOR EACH ROW
  EXECUTE PROCEDURE svyrobnicislohlavupdateKod();


-- Function: svyrobnicislocreatehlav()
-- DROP FUNCTION svyrobnicislocreatehlav();
CREATE OR REPLACE FUNCTION svyrobnicislocreatehlav()
  RETURNS trigger AS
$BODY$ 
declare 
  hlav record; 
begin 
  -- hlavicka neexistuje -> vytvorime ji 
  if (NEW.idvyrobnicislohlav is NULL) then 
    insert into svyrobnicislohlav(kod, idcenik) 
      values (NEW.kod, NEW.idcenik); 
    select currval('seq_svyrobnicislohlav') into NEW.idvyrobnicislohlav; 
  -- polozka se jiz vaze na hlavicku, pro jistotu zkontrolujeme, 
  -- ze mame stejny kod a idcenik 
  else 
  end if; 
 
  return NEW; 
end 
$BODY$
  LANGUAGE 'plpgsql' VOLATILE
  COST 100;
ALTER FUNCTION svyrobnicislocreatehlav() OWNER TO %firmaRole%;


CREATE OR REPLACE FUNCTION svyrobnicisloupdatehlav()
  RETURNS trigger AS
$BODY$ 
declare 
  hlav record; 
begin 
  select * into hlav from svyrobnicislohlav h where h.idvyrobnicislohlav = NEW.idvyrobnicislohlav; 
 
  -- pouzivaji se 2 vnorene if, protoze postgresql pri vyhodnocovani vyhodnocuje 
  --vsechny logicke vyrazy a OLD pri insert neni nainicializovana 
 
  -- pri aktualizaci zkontrolujeme, ze kod pohybu je stejny jako kod na hlavicce 
  if (tg_op = 'UPDATE') then 
    if (OLD.kod != hlav.kod) then 
      --raise exception 'Kod pohybu vyrobniho cisla % se neshoduje s kodem na hlavicce %.', OLD.kod, hlav.kod; 
    end if; 
 
    -- pri aktualizaci a prepsani kodu pohybu vyrobniho cisla - pozmenime kod i na hlavicce 
    if (OLD.kod != NEW.kod) then 
      if(hlav.kod != NEW.kod) then
        update svyrobnicislohlav set kod = NEW.kod where idvyrobnicislohlav = NEW.idvyrobnicislohlav; 
      end if;
    end if; 
  end if; 
 
  if (NEW.idcenik != hlav.idcenik) then 
    if (NEW.idpolskladvydej != null or NEW.idpolfakvydej != null or NEW.idpolintvydej != null) then 
      raise exception 'Menite cenik % u jiz vydaneho vyrobniho cisla %', NEW.idcenik, NEW.idvyrobnicislo; 
    end if; 
 
    update svyrobnicislohlav set idcenik = NEW.idcenik where idvyrobnicislohlav = NEW.idvyrobnicislohlav; 
  end if; 

  return NEW;
end 
$BODY$
  LANGUAGE 'plpgsql' VOLATILE
  COST 100;
ALTER FUNCTION svyrobnicisloupdatehlav() OWNER TO %firmaRole%;

CREATE TRIGGER svyrobnicisloupdatehlav
  AFTER UPDATE
  ON svyrobnicislo
  FOR EACH ROW
  EXECUTE PROCEDURE svyrobnicisloupdatehlav();


--------------------------------------------------------------------------------
-- Bug 23851 – [WinStrom] Duplikace vyrobnich cisel na prevodce
--------------------------------------------------------------------------------

-- Function: svyrobnicisloupdateprevodka()
-- DROP FUNCTION svyrobnicisloupdateprevodka();
CREATE OR REPLACE FUNCTION svyrobnicisloupdateprevodka()
  RETURNS trigger AS
$BODY$ 
declare 
  _vazbaPrevod record; 
  _polSkladPrijemPrevod record; 
  _probihaPrepocetSkladu boolean;
begin 

 select getglotxvariable('probihaPrepocetSkladu') into _probihaPrepocetSkladu;
 if(_probihaPrepocetSkladu = 'true') then
 else 
  /* Vytvoreni novych vyrobnich cisel pro vyrobni cisla, ktera byla privazana */ 
  if(NEW.idPolSkladVydej is not null and OLD.idPolSkladVydej is null) then 
    select * into _vazbaPrevod from dvazebtab where idtabA = NEW.idPolSkladVydej and modulA = 'SKL' and modulB = 'SKL' and typVazbyK = 'typVazbyDokl.polozkaSklPrevod'; 
    if(_vazbaPrevod is not null) then 
      select * into _polSkladPrijemPrevod from dpolsklad where idPolSklad = _vazbaPrevod.idTabB; 
      insert into svyrobnicislo(kod, idcenik, idskladu, 
          idpolskladprijem, idpolskladvydej, idpolfakprijem, idpolfakvydej, idpolintprijem, idpolintvydej, idvyrobnicislohlav) 
        values(NEW.kod, NEW.idcenik, _polSkladPrijemPrevod.idBspSkl, 
          _polSkladPrijemPrevod.idPolSklad, null, null, null, null, null, NEW.idvyrobnicislohlav); 
    end if; 
  end if; 
 
  /* Smazani vyrobnich cisel, ktere byli vygenerovany pro odvazana vyrobni cisla */ 
  if(NEW.idPolSkladVydej is null and OLD.idPolSkladVydej is not null) then 
    select * into _vazbaPrevod from dvazebtab where idtabA = OLD.idPolSkladVydej and modulA = 'SKL' and modulB = 'SKL' and typVazbyK = 'typVazbyDokl.polozkaSklPrevod'; 
    if(_vazbaPrevod is not null) then 
      delete from svyrobnicislo where idvyrobnicislohlav = NEW.idVyrobniCisloHlav and idpolskladprijem = _vazbaPrevod.idTabB 
        and idPolSkladVydej is null and idPolFakVydej is null and idPolIntVydej is null; 
    end if; 
  end if; 
 end if;
 return new; 
end; 
$BODY$
  LANGUAGE 'plpgsql' VOLATILE
  COST 100;
ALTER FUNCTION svyrobnicisloupdateprevodka() OWNER TO %firmaRole%;

-- Function: dgetstavnasklade(integer, date, integer, character varying, date)
-- DROP FUNCTION dgetstavnasklade(integer, date, integer, character varying, date);
CREATE OR REPLACE FUNCTION dgetstavnasklade(i_skladkartaid integer, i_datum date, i_polskladid integer, i_sarze character varying, i_expirace date)
  RETURNS numeric AS
$BODY$      
declare     
  _pocMj numeric(19,6);     
  _obratMj numeric(19,6);     
  _stavMj numeric(19,6);     
  _idcenik integer;   
  _idskladu integer;   
  _typPohybuK character varying(50);   
  _sadaItemCount integer; 
  _polozkaSady record; 
  _idSkladKarty integer; 
  _iducetniobdobi integer; 
  _idPolSklad integer; 
  _stavSkladuSady numeric(19,6); 
  _mnozMj numeric(19,6); 
  _stavMjKarty numeric(19,6); 
  _rozpadlaPolSklad record;
begin     
  SELECT pocatMj INTO _pocMj     
  FROM sKarty     
  WHERE idKarty = i_skladkartaid;     
     
  if (i_datum is null) then     
    raise exception '0005:dGetStavNaSklade : Vstupni parametr datum je null.';      
  end if;   
   
  _typPohybuK := null;   
  if (i_polskladid >= 0) then   
    select typPohybuK into _typPohybuK from ddoklsklad    
      where iddoklsklad = (select iddoklsklad from dpolsklad where idpolsklad = i_polskladid);   
  end if;   
   
  select idcenik, idskladu, iducetobdobi into _idcenik, _idskladu, _iducetniobdobi from skarty where idkarty = i_skladkartaid; 
 
  -- zjistim jestli nahodu dany cenik neni sada nebo komplet 
  select count(*) into _sadaItemCount from csady where idceniksada = _idCenik; 
  -- pokud se nejedna o sadu tak funguju normalne 
  if(_sadaItemCount = 0) then 
    if ((i_sarze is null or i_sarze = '') and i_expirace is null) then    
      -- zkusime zjistit stav skladu k datu 
      if (i_polSkladId < 0) then       
        SELECT sum(CASE ds.typPohybuK WHEN 'typPohybu.prijem' THEN ps.mnozMj ELSE -ps.mnozMj END) INTO _obratMj       
        FROM dPolSklad ps LEFT OUTER JOIN dDoklSklad ds ON ps.idDoklSklad = ds.idDoklSklad       
        WHERE idkarty = i_skladkartaid AND ps.storno <> true AND ps.datUcto <= i_datum;       
      else       
        SELECT sum(CASE ds.typPohybuK WHEN 'typPohybu.prijem' THEN ps.mnozMj ELSE -ps.mnozMj END) INTO _obratMj       
        FROM dPolSklad ps LEFT OUTER JOIN dDoklSklad ds ON ps.idDoklSklad = ds.idDoklSklad       
        WHERE idkarty = i_skladkartaid AND ps.storno <> true AND (ps.datUcto < i_datum OR (ps.datUcto = i_datum AND ps.idPolSklad < i_polskladid));       
      end if;    
      if(_obratMj is null) then _obratMj = 0.0; end if;       
      if(_pocMj is null) then _pocMj = 0.0; end if;        
      _stavMj = _pocMj + _obratMj;    
      -- zkusime vypocitat stav na vrcholu 
      select stavMj into _stavMjKarty from skarty where idkarty = i_skladkartaid; 
      _mnozMj = 0.0; 
      if(i_polSkladId >= 0) then 
        select mnozMj into _mnozMj from dPolSklad where idPolSklad = i_polSkladId; 
        for _rozpadlaPolSklad in (select * from dPolSklad where idPolSklad in 
          (select idpolskladcil from svazbypolsklad where idpolskladzdroj = i_polSkladId)) 
        loop 
          _mnozMj = _mnozMj + _rozpadlaPolSklad.mnozMj;
        end loop;
        if(_typPohybuK = 'typPohybu.vydej') then 
          _stavMjKarty = _stavMjKarty + _mnozMj; 
        else 
          _stavMjKarty = _stavMjKarty - _mnozMj; 
        end if; 
      end if; 
 
      -- spravna hodnota dle bugu 21375 minimum z obou hodnot 
      if(_typPohybuK = 'typPohybu.vydej') then 
        select minValue(_stavMj, _stavMjKarty) into _stavMj; 
      end if; 
    else    
      if ((i_sarze is not null and i_sarze <> '') and (i_expirace is null)) then    
        --raise exception 'idCenik = % and idSkladu = %', _idCenik, _idSkladu;   
       
        select sum(pocet) into _stavMj from dpolskladf where idcenik = _idcenik and idskladu = _idskladu and sarze = i_sarze;    
      elsif ((i_sarze is null or i_sarze = '') and (i_expirace is not null)) then    
        select sum(pocet) into _stavMj from dpolskladf where idcenik = _idcenik and idskladu = _idskladu and expirace = i_expirace;    
      else    
        select sum(pocet) into _stavMj from dpolskladf where idcenik = _idcenik and idskladu = _idskladu and sarze = i_sarze and expirace = i_expirace;    
      end if;    
      if (_typPohybuK is not null and _typPohybuK = 'typPohybu.prijem') then    
        select mnozMj into _obratMj from dpolsklad where idpolsklad = i_polskladid;    
        if(_stavMj is null) then    
          _stavMj := -_obratMj;    
        else    
          _stavMj := _stavMj - _obratMj;    
        end if;    
      elsif (_typPohybuK is not null and _typPohybuK = 'typPohybu.vydej') then    
        select mnozMj into _obratMj from dpolsklad where idpolsklad = i_polskladid;    
        if(_stavMj is null) then    
          _stavMj := _obratMj;    
        else    
          _stavMj := _stavMj + _obratMj;    
        end if;    
      end if;    
    end if;   
  else 
    for _polozkaSady in 
      select * from cSady where idCenikSada = _idCenik 
    loop 
      select idKarty into _idSkladKarty from sKarty where idCenik = _polozkaSady.idCenik and idSkladu = _idskladu and idUcetObdobi = _iducetniobdobi; 
      if(_idSkladKarty is not null) then  
        if(_stavMj is null) then 
          _stavMj = 9999999999999.0; -- vic uz pretejka 
        end if; 
        _idPolSklad = -1; 
        --TODO nejak doresit _idPolSklad, sarze a expirace; 
        select dgetstavnasklade(_idSkladKarty, i_datum, -1, i_sarze, i_expirace) into _stavSkladuSady; 
        _stavSkladuSady = _stavSkladuSady / _polozkaSady.mnozMj; 
	if(i_polskladid > 0) then 
	  select mnozMj into _mnozMj from dPolSklad where idPolSklad = i_polSkladId; 
          _stavSkladuSady = _stavSkladuSady + (_mnozMj/_polozkaSady.mnozMj); 
        end if; 
        select minValue(_stavMj, _stavSkladuSady) into _stavMj; 
      end if; 
    end loop; 
  end if;  
  
  -- muze se stat ze _stavMj je null a to znamena ze na sklade nic neni  
  if(_stavMj is null) then  
    _stavMj := 0;  
  end if;  
    
  RETURN _stavMj;      
end      
$BODY$
  LANGUAGE 'plpgsql' VOLATILE
  COST 100;
ALTER FUNCTION dgetstavnasklade(integer, date, integer, character varying, date) OWNER TO %firmaRole%;

-- zvyseni verze databaze
insert into dbversion(iddbversion, dbversion, needreimport, appversion, upgradetime)
  values (nextval('seq_dbversion'), 485, 't', %appVersion%, clock_timestamp());
