--------------------------------------------------------------------------------
-- Bug 23645 – [WinStrom] Problém s účtováním vydaných faktur
--------------------------------------------------------------------------------

CREATE OR REPLACE FUNCTION dpolfakupdatezauctovano()
  RETURNS trigger AS
$BODY$
declare
  ZAZNAM record;
  _idPolDokl integer;
  _zuctovano boolean;
begin
  if (tg_op = 'DELETE') then
    ZAZNAM := OLD;
  else
    ZAZNAM := NEW;
  end if;
  
  select isDokladZuctovan(ZAZNAM.idDoklFak, 'Fak') into _zuctovano;
  
  update ddoklfak set zuctovano = _zuctovano where idDoklFak = ZAZNAM.idDoklFak and zuctovano <> _zuctovano;
  
  return NEW;
end;
$BODY$
  LANGUAGE 'plpgsql' VOLATILE
  COST 100;

ALTER FUNCTION dpolfakupdatezauctovano() OWNER TO %firmaRole%;



CREATE OR REPLACE FUNCTION dpolintupdatezauctovano()
  RETURNS trigger AS
$BODY$
declare
  ZAZNAM record;
  _idPolDokl integer;
  _zuctovano boolean;
begin
  if (tg_op = 'DELETE') then
    ZAZNAM := OLD;
  else
    ZAZNAM := NEW;
  end if;
  
  select isDokladZuctovan(ZAZNAM.idDoklInt, 'Int') into _zuctovano;
    
  update ddoklint set zuctovano = _zuctovano where idDoklInt = ZAZNAM.idDoklInt and zuctovano <> _zuctovano;
    
  return NEW;  
end;  
$BODY$
  LANGUAGE 'plpgsql' VOLATILE
  COST 100;

ALTER FUNCTION dpolintupdatezauctovano() OWNER TO %firmaRole%;



CREATE OR REPLACE FUNCTION dpolskladupdatezauctovano()
  RETURNS trigger AS
$BODY$
declare
  ZAZNAM record;
  _idPolDokl integer;
  _zuctovano boolean;
begin
  if (tg_op = 'DELETE') then
    ZAZNAM := OLD;
  else
    ZAZNAM := NEW;
  end if;
  
  select isDokladZuctovan(ZAZNAM.idDoklSklad, 'Sklad') into _zuctovano;
    
  update ddoklsklad set zuctovano = _zuctovano where idDoklSklad = ZAZNAM.idDoklSklad and zuctovano <> _zuctovano;
    
  return NEW;  
end;  
$BODY$
  LANGUAGE 'plpgsql' VOLATILE
  COST 100;

ALTER FUNCTION dpolskladupdatezauctovano() OWNER TO %firmaRole%;


--------------------------------------------------------------------------------


DROP TRIGGER dpolfakupdatezauctovano ON dpolfak;
DROP TRIGGER dpolintupdatezauctovano ON dpolint;
DROP TRIGGER dpolskladupdatezauctovano ON dpolsklad;

CREATE TRIGGER dpolfakupdatezauctovano
    AFTER INSERT OR UPDATE OR DELETE
    ON dpolfak
    FOR EACH ROW
    EXECUTE PROCEDURE dpolfakupdatezauctovano();


CREATE TRIGGER dpolintupdatezauctovano
    AFTER INSERT OR UPDATE OR DELETE
    ON dpolint
    FOR EACH ROW
    EXECUTE PROCEDURE dpolintupdatezauctovano();


CREATE TRIGGER dpolskladupdatezauctovano
    AFTER INSERT OR UPDATE OR DELETE
    ON dpolsklad
    FOR EACH ROW
    EXECUTE PROCEDURE dpolskladupdatezauctovano();


-- zvyseni verze databaze
insert into dbversion(iddbversion, dbversion, needreimport, appversion, upgradetime)
  values (nextval('seq_dbversion'), 506, 't', %appVersion%, clock_timestamp());
