--------------------------------------------------------------------------------
-- Bug 27022 – [WinStrom] Smlouvy - Datové schema smluv + B a E okna
--------------------------------------------------------------------------------

CREATE SEQUENCE seq_dstavsml
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;
ALTER TABLE seq_dstavsml OWNER TO %firmaRole%;

CREATE TABLE dstavsml (
    idstavsml integer NOT NULL DEFAULT nextval('seq_dstavsml'::regclass),
    lastupdate timestamp without time zone NOT NULL DEFAULT '2008-01-01 00:00:00'::timestamp without time zone,
    kod character varying(20) NOT NULL,
    nazev character varying(255) NOT NULL,
    nazeva character varying(255),
    nazevb character varying(255),
    nazevc character varying(255),
    popis text,
    poznam text,
    show boolean NOT NULL DEFAULT true,
    platido integer NOT NULL DEFAULT 9999,
    platiod integer NOT NULL DEFAULT 0,
    uuid character varying(50),

    CONSTRAINT dstavsml_pkey PRIMARY KEY (idstavsml)
);
ALTER TABLE dstavsml OWNER TO %firmaRole%;

--------------------------------------------------------------------------------

CREATE SEQUENCE seq_dtypsml
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;

ALTER TABLE public.seq_dtypsml OWNER TO %firmaRole%;

CREATE TABLE dtypsml (
    idtypsml integer DEFAULT nextval('seq_dtypsml'::regclass) NOT NULL,
    lastupdate timestamp without time zone DEFAULT '2008-01-01 00:00:00'::timestamp without time zone NOT NULL,
    kod character varying(20) NOT NULL,
    nazev character varying(255) NOT NULL,
    nazeva character varying(255),
    nazevb character varying(255),
    nazevc character varying(255),
    popis text,
    poznam text,
    show boolean DEFAULT true NOT NULL,
    platido integer NOT NULL DEFAULT 9999,
    platiod integer NOT NULL DEFAULT 0,

    autoprodl boolean NOT NULL DEFAULT false,
    autozakazka boolean NOT NULL DEFAULT false,
    autogen boolean NOT NULL DEFAULT false,
    den integer,
    mesic integer,
    posupom boolean NOT NULL DEFAULT false,
    dnyupom integer not null default 0,
    genpenale boolean NOT NULL DEFAULT false,
    procpenale numeric(6,2) DEFAULT 0.0 NOT NULL,
    preplatky boolean NOT NULL DEFAULT false,
    zpusfaktk character varying(50),
    dnyFakt integer not null default 0,
    automail boolean NOT NULL DEFAULT false,
    varsymfakt boolean NOT NULL DEFAULT false,

    idtypdokl integer,

    CONSTRAINT dtypsml_pkey PRIMARY KEY (idtypsml),
    CONSTRAINT dtypsml_idtypdokl FOREIGN KEY (idtypdokl)
        REFERENCES dtypdokl (idtypdokl) MATCH SIMPLE
        ON UPDATE NO ACTION ON DELETE NO ACTION
);

ALTER TABLE public.dtypsml OWNER TO %firmaRole%;

CREATE INDEX dtypsml_idtypdokl_index ON dtypsml USING btree (idtypdokl);

--------------------------------------------------------------------------------

CREATE SEQUENCE seq_dsmlouvy
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;

ALTER TABLE public.seq_dsmlouvy OWNER TO %firmaRole%;

CREATE TABLE dsmlouvy (
    idsmlouvy integer DEFAULT nextval('seq_dsmlouvy'::regclass) NOT NULL,
    lastupdate timestamp without time zone DEFAULT '2008-01-01 00:00:00'::timestamp without time zone NOT NULL,
    kod character varying(20) NOT NULL,
    nazev character varying(255) NOT NULL,
    nazeva character varying(255),
    nazevb character varying(255),
    nazevc character varying(255),
    popis text,
    poznam text,
    show boolean DEFAULT true NOT NULL,

    sablona boolean NOT NULL DEFAULT false,
    smlouvaod date,
    smlouvado date,
    frekfakt integer not null default 0,
    den integer,
    mesic integer,
    varsym character varying(30),
    pocetPriloh integer not null default 0,

    idtypsml integer NOT NULL,
    idfirmy integer NOT NULL,
    idzakazky integer,
    idstred integer,
    idtypdokl integer,
    idkonsym integer,
    idstavsml integer,

    CONSTRAINT dsmlouvy_pkey PRIMARY KEY (idsmlouvy),
    CONSTRAINT dsmlouvy_idtypsml FOREIGN KEY (idtypsml)
        REFERENCES dtypsml (idtypsml) MATCH SIMPLE
        ON UPDATE NO ACTION ON DELETE NO ACTION,
    CONSTRAINT dsmlouvy_idfirmy FOREIGN KEY (idfirmy)
        REFERENCES aadresar (idfirmy) MATCH SIMPLE
        ON UPDATE NO ACTION ON DELETE NO ACTION,
    CONSTRAINT dsmlouvy_idzakazky FOREIGN KEY (idzakazky)
        REFERENCES uzakazky (idzakazky) MATCH SIMPLE
        ON UPDATE NO ACTION ON DELETE NO ACTION,
    CONSTRAINT dsmlouvy_idstred FOREIGN KEY (idstred)
        REFERENCES ustrediska (idstred) MATCH SIMPLE
        ON UPDATE NO ACTION ON DELETE NO ACTION,
    CONSTRAINT dsmlouvy_idtypdokl FOREIGN KEY (idtypdokl)
        REFERENCES dtypdokl (idtypdokl) MATCH SIMPLE
        ON UPDATE NO ACTION ON DELETE NO ACTION,
    CONSTRAINT dsmlouvy_idkonsym FOREIGN KEY (idkonsym)
        REFERENCES dkonsym (idkonsym) MATCH SIMPLE
        ON UPDATE NO ACTION ON DELETE NO ACTION,
    CONSTRAINT dsmlouvy_idstavsml FOREIGN KEY (idstavsml)
        REFERENCES dstavsml (idstavsml) MATCH SIMPLE
        ON UPDATE NO ACTION ON DELETE NO ACTION
);

ALTER TABLE public.dsmlouvy OWNER TO %firmaRole%;

CREATE INDEX dsmlouvy_idtypsml_index ON dsmlouvy USING btree (idtypsml);
CREATE INDEX dsmlouvy_idfirmy_index ON dsmlouvy USING btree (idfirmy);
CREATE INDEX dsmlouvy_idzakazky_index ON dsmlouvy USING btree (idzakazky);
CREATE INDEX dsmlouvy_idstred_index ON dsmlouvy USING btree (idstred);
CREATE INDEX dsmlouvy_idtypdokl_index ON dsmlouvy USING btree (idtypdokl);
CREATE INDEX dsmlouvy_idkonsym_index ON dsmlouvy USING btree (idkonsym);
CREATE INDEX dsmlouvy_idstavsml_index ON dsmlouvy USING btree (idstavsml);

--------------------------------------------------------------------------------

CREATE SEQUENCE seq_dpolsml
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;

ALTER TABLE public.seq_dpolsml OWNER TO %firmaRole%;

CREATE TABLE dpolsml (
    idpolsml integer DEFAULT nextval('seq_dpolsml'::regclass) NOT NULL,
    lastupdate timestamp without time zone DEFAULT '2008-01-01 00:00:00'::timestamp without time zone NOT NULL,
    kod character varying(20) NOT NULL,
    nazev character varying(255) NOT NULL,
    nazeva character varying(255),
    nazevb character varying(255),
    nazevc character varying(255),

    cisrad integer not null default 0,
    platioddata date,
    platidodata date,
    frekfakt integer,
    cenarucne boolean NOT NULL DEFAULT false,
    cenamj numeric(19,6) DEFAULT 0.0 NOT NULL,
    mnozmj numeric(19,6) DEFAULT 0.0 NOT NULL,
    cenacelkem numeric(15,2) DEFAULT 0.0 NOT NULL,
    typcenydphk character varying(50),
    typszbdphk character varying(50),
    sleva numeric(6,2) DEFAULT 0.0 NOT NULL,
    extgener boolean NOT NULL DEFAULT false,
    datPoslFaktObd date,
    pocetPriloh integer not null default 0,

    idsmlouvy integer NOT NULL,
    idcenik integer,
    idbspskl integer,
    idstred integer,
    idpolfak integer,

    CONSTRAINT dpolsml_pkey PRIMARY KEY (idpolsml),
    CONSTRAINT dpolsml_idsmlouvy FOREIGN KEY (idsmlouvy)
        REFERENCES dsmlouvy (idsmlouvy) MATCH SIMPLE
        ON UPDATE NO ACTION ON DELETE NO ACTION,
    CONSTRAINT dpolsml_idcenik FOREIGN KEY (idcenik)
        REFERENCES ccenik (idcenik) MATCH SIMPLE
        ON UPDATE NO ACTION ON DELETE NO ACTION,
    CONSTRAINT dpolsml_idbspskl FOREIGN KEY (idbspskl)
        REFERENCES dbsp (idbsp) MATCH SIMPLE
        ON UPDATE NO ACTION ON DELETE NO ACTION,
    CONSTRAINT dpolsml_idstred FOREIGN KEY (idstred)
        REFERENCES ustrediska (idstred) MATCH SIMPLE
        ON UPDATE NO ACTION ON DELETE NO ACTION,
    CONSTRAINT dpolsml_idpolfak FOREIGN KEY (idpolfak)
        REFERENCES dpolfak (idpolfak) MATCH SIMPLE
        ON UPDATE NO ACTION ON DELETE NO ACTION
);

ALTER TABLE public.dpolsml OWNER TO %firmaRole%;

CREATE INDEX dpolsml_idsmlouvy_index ON dpolsml USING btree (idsmlouvy);
CREATE INDEX dpolsml_idcenik_index ON dpolsml USING btree (idcenik);
CREATE INDEX dpolsml_idbspskl_index ON dpolsml USING btree (idbspskl);
CREATE INDEX dpolsml_idstred_index ON dpolsml USING btree (idstred);
CREATE INDEX dpolsml_idpolfak_index ON dpolsml USING btree (idpolfak);

--------------------------------------------------------------------------------

ALTER TABLE dtypdokl
    ADD COLUMN faktauto boolean NOT NULL DEFAULT false;

ALTER TABLE ddoklfak
    ADD COLUMN stavmailk character varying(50);

ALTER TABLE ddoklfak
    ADD COLUMN idsmlouvy integer,
    ADD CONSTRAINT ddoklfak_idsmlouvy FOREIGN KEY (idsmlouvy)
        REFERENCES dsmlouvy (idsmlouvy) MATCH SIMPLE
        ON UPDATE NO ACTION ON DELETE NO ACTION;

ALTER TABLE ddoklint
    ADD COLUMN idsmlouvy integer,
    ADD CONSTRAINT ddoklint_idsmlouvy FOREIGN KEY (idsmlouvy)
        REFERENCES dsmlouvy (idsmlouvy) MATCH SIMPLE
        ON UPDATE NO ACTION ON DELETE NO ACTION;

ALTER TABLE ddoklsklad
    ADD COLUMN idsmlouvy integer,
    ADD CONSTRAINT ddoklsklad_idsmlouvy FOREIGN KEY (idsmlouvy)
        REFERENCES dsmlouvy (idsmlouvy) MATCH SIMPLE
        ON UPDATE NO ACTION ON DELETE NO ACTION;

ALTER TABLE ddoklobch
    ADD COLUMN idsmlouvy integer,
    ADD CONSTRAINT ddoklobch_idsmlouvy FOREIGN KEY (idsmlouvy)
        REFERENCES dsmlouvy (idsmlouvy) MATCH SIMPLE
        ON UPDATE NO ACTION ON DELETE NO ACTION;

CREATE INDEX ddoklfak_idsmlouvy_index ON ddoklfak USING btree (idsmlouvy);
CREATE INDEX ddoklint_idsmlouvy_index ON ddoklint USING btree (idsmlouvy);
CREATE INDEX ddoklobch_idsmlouvy_index ON ddoklobch USING btree (idsmlouvy);
CREATE INDEX ddoklsklad_idsmlouvy_index ON ddoklsklad USING btree (idsmlouvy);

--------------------------------------------------------------------------------

ALTER TABLE wpriloha
  ADD COLUMN idsmlouvy integer,
  ADD COLUMN idpolsml integer,
  ADD CONSTRAINT wpriloha_idsmlouvy FOREIGN KEY (idsmlouvy)
      REFERENCES dsmlouvy (idsmlouvy) MATCH SIMPLE
      ON UPDATE NO ACTION ON DELETE NO ACTION,
  ADD CONSTRAINT wpriloha_idpolsml FOREIGN KEY (idpolsml)
      REFERENCES dpolsml (idpolsml) MATCH SIMPLE
      ON UPDATE NO ACTION ON DELETE NO ACTION;

CREATE INDEX wpriloha_idsmlouvy_index ON wpriloha USING btree (idsmlouvy);
CREATE INDEX wpriloha_idpolsml_index ON wpriloha USING btree (idpolsml);


create or replace function updatePocetPriloh(prilohaid integer, addon integer) returns void as
$BODY$
declare
    priloha record;
begin
    select * into priloha from wpriloha p where p.idpriloha = prilohaid;

    if (priloha.idreport is not null) then
        update wreport
        set pocetPriloh =   (select count(1)+addon
                            from wpriloha p
                            where p.idreport = priloha.idreport)
        where idreport = priloha.idreport;
    end if;


    if (priloha.idadresar is not null) then
        update aadresar
        set pocetPriloh =   (select count(1)+addon
                            from wpriloha p
                            where p.idadresar = priloha.idadresar)
        where idfirmy = priloha.idadresar;
    end if;

    if (priloha.idkontakt is not null) then
        update akontakty set pocetPriloh =  (select count(1)+addon
                                            from wpriloha p
                                            where p.idkontakt = priloha.idkontakt)
        where idkontakt = priloha.idkontakt;
    end if;

    if (priloha.idcenik is not null) then
        update ccenik set pocetPriloh = (select count(1)+addon
                                        from wpriloha p
                                        where p.idcenik = priloha.idcenik)
        where idcenik = priloha.idcenik;
    end if;

    if (priloha.iddoklfak is not null) then
        update ddoklfak set pocetPriloh =   (select count(1)+addon
                                            from wpriloha p
                                            where p.iddoklfak = priloha.iddoklfak)
        where iddoklfak = priloha.iddoklfak;
    end if;

    if (priloha.iddoklint is not null) then
        update ddoklint set pocetPriloh =   (select count(1)+addon
                                            from wpriloha p
                                            where p.iddoklint = priloha.iddoklint)
        where iddoklint = priloha.iddoklint;
    end if;

    if (priloha.iddoklobch is not null) then
        update ddoklobch set pocetPriloh =  (select count(1)+addon
                                            from wpriloha p
                                            where p.iddoklobch = priloha.iddoklobch)
        where iddoklobch = priloha.iddoklobch;
    end if;

    if (priloha.iddoklsklad is not null) then
        update ddoklsklad set pocetPriloh = (select count(1)+addon
                                            from wpriloha p
                                            where p.iddoklsklad = priloha.iddoklsklad)
        where iddoklsklad = priloha.iddoklsklad;
    end if;

    if (priloha.idosoba is not null) then
        update posoba set pocetPriloh = (select count(1)+addon
                                            from wpriloha p
                                            where p.idosoba = priloha.idosoba)
        where idosobahlav = priloha.idosoba;
    end if;

    if (priloha.idadrudalost is not null) then
        update audalosti set pocetPriloh = (select count(1)+addon
                                            from wpriloha p
                                            where p.idadrudalost = priloha.idadrudalost)
        where idudalost = priloha.idadrudalost;
    end if;

    if (priloha.idzakazky is not null) then
        update uzakazky set pocetPriloh = (select count(1)+addon
                                            from wpriloha p
                                            where p.idzakazky = priloha.idzakazky)
        where idzakazky = priloha.idzakazky;
    end if;

    if (priloha.idkonektor is not null) then
        update wkonektor set pocetPriloh = (select count(1)+addon
                                            from wpriloha p
                                            where p.idkonektor = priloha.idkonektor)
        where idkonektor = priloha.idkonektor;
    end if;

    if (priloha.idsmlouvy is not null) then
        update dsmlouvy set pocetPriloh = (select count(1)+addon
                                            from wpriloha p
                                            where p.idsmlouvy = priloha.idsmlouvy)
        where idsmlouvy = priloha.idsmlouvy;
    end if;

    if (priloha.idpolsml is not null) then
        update dpolsml set pocetPriloh = (select count(1)+addon
                                            from wpriloha p
                                            where p.idpolsml = priloha.idpolsml)
        where idpolsml = priloha.idpolsml;
    end if;
end;
$BODY$
  LANGUAGE 'plpgsql' VOLATILE;
ALTER FUNCTION updatePocetPriloh(integer, integer) OWNER TO %firmaRole%;

--------------------------------------------------------------------------------

CREATE TABLE wkli_smlouva (
    idsmlouvy integer NOT NULL,
    idvybklic integer NOT NULL,

    CONSTRAINT wkli_smlouva_pkey PRIMARY KEY (idsmlouvy, idvybklic),
    CONSTRAINT wkli_smlouva_idsmlouvy FOREIGN KEY (idsmlouvy)
      REFERENCES dsmlouvy (idsmlouvy) MATCH SIMPLE
      ON UPDATE NO ACTION ON DELETE NO ACTION,
    CONSTRAINT wkli_smlouva_idvybklic FOREIGN KEY (idvybklic)
        REFERENCES wvybklice(idvybklice) MATCH SIMPLE
        ON UPDATE NO ACTION ON DELETE NO ACTION
);
ALTER TABLE public.wkli_smlouva OWNER TO %firmaRole%;

CREATE INDEX wkli_smlouva_idsmlouvy_index ON wkli_smlouva USING btree (idsmlouvy);
CREATE INDEX wkli_smlouva_idvybklic_index ON wkli_smlouva USING btree (idvybklic);


CREATE TABLE wkli_polsml (
    idpolsml integer NOT NULL,
    idvybklic integer NOT NULL,

    CONSTRAINT wkli_polsml_pkey PRIMARY KEY (idpolsml, idvybklic),
    CONSTRAINT wkli_polsml_idpolsml FOREIGN KEY (idpolsml)
      REFERENCES dpolsml (idpolsml) MATCH SIMPLE
      ON UPDATE NO ACTION ON DELETE NO ACTION,
    CONSTRAINT wkli_polsml_idvybklic FOREIGN KEY (idvybklic)
        REFERENCES wvybklice(idvybklice) MATCH SIMPLE
        ON UPDATE NO ACTION ON DELETE NO ACTION
);
ALTER TABLE public.wkli_polsml OWNER TO %firmaRole%;

CREATE INDEX wkli_polsml_idpolsml_index ON wkli_polsml USING btree (idpolsml);
CREATE INDEX wkli_polsml_idvybklic_index ON wkli_polsml USING btree (idvybklic);

--------------------------------------------------------------------------------


-- zvyseni verze databaze
insert into dbversion(iddbversion, dbversion, needreimport, appversion, upgradetime)
  values (nextval('seq_dbversion'), 565, 't', %appVersion%, clock_timestamp());
