----------------------------------------------------------------------------------------------
-- Bug 29847 – [WinStrom] Fixopia při ukončení rodičovské dovolené
----------------------------------------------------------------------------------------------

UPDATE nnastav SET postmigrace = true;
INSERT INTO wsetting (klic, hodnota, iduzivatel) VALUES ('opravaNep624', 'true', 0);

----------------------------------------------------------------------------------------------
-- Bug 29770 – [winstrom] mzdy - výpočet mzdy při ukončení PP
----------------------------------------------------------------------------------------------

-- trigger generujici mzdove slozky pri uprave pracovnich pomeru
CREATE OR REPLACE FUNCTION ppracpomhlavupdatemzdsloz()
  RETURNS trigger AS $$
declare
  nullcount integer;
  _rok integer;
  _mesic integer;
  _idSmena integer;
  pregenSmenuRok integer;
  pregenSmenuMesic integer;
  pregenSmenuOd date;
  pregenSmenuDo date;
begin
  -- vymazeme nadbytecne slozky pro pracovni pomer
  if (NEW.aktivniod is not null) then
    _rok = date_part('year', NEW.aktivniod);
    _mesic = date_part('month', NEW.aktivniod);
    
    delete from pmzdsloz where idpracpom = NEW.idpracpomhlav and (rok < _rok or (rok = _rok and mesic < _mesic));
    delete from psmeny where idpracpom = NEW.idpracpomhlav and (rok < _rok or (rok = _rok and mesic < _mesic));
  end if;
  
  if (NEW.aktivnido is not null) then
    _rok = date_part('year', NEW.aktivnido);
    _mesic = date_part('month', NEW.aktivnido);
    
    delete from pmzdsloz where idpracpom = NEW.idpracpomhlav and (rok > _rok or (rok = _rok and mesic > _mesic));
    delete from psmeny where idpracpom = NEW.idpracpomhlav and (rok > _rok or (rok = _rok and mesic > _mesic));
  end if;
  
  -- vymazeme nadbytecne spolecne slozky
  select count(*) into nullcount from ppracpomhlav where idosoba = NEW.idosoba and aktivniod is null;
  if nullcount = 0 then
    select date_part('year', min(aktivniod)), date_part('month', min(aktivniod)) into _rok, _mesic from ppracpom where idosoba = NEW.idosoba;
    
    delete from pmzdsloz where idosoba = NEW.idosoba and (rok < _rok or (rok = _rok and mesic < _mesic));
  end if;
  
  select count(*) into nullcount from ppracpomhlav where idosoba = NEW.idosoba and aktivnido is null;
  if nullcount = 0 then
    select date_part('year', max(aktivnido)), date_part('month', max(aktivnido)) into _rok, _mesic from ppracpom where idosoba = NEW.idosoba;
    
    delete from pmzdsloz where idosoba = NEW.idosoba and (rok > _rok or (rok = _rok and mesic > _mesic));
  end if;
  
  -- vygenerujeme chybejici
  perform pgeneratemzdslozforpracpom(NEW.idpracpomhlav);
  
  if (tg_op = 'UPDATE') then       -- pri zmene aktivni od/do zjistime jesteli neni potreba pregenerovat nejaky mesic, kvuli dobe kdy nepracuje
    -- zkontrolujeme zda se nezmenil zacatek pomeru a neni treba neco pregenerovat
    if (comparedate(NEW.zacatek, OLD.zacatek) = false) then
      pregenSmenuRok = 0;
      pregenSmenuMesic = 0;
      
      if (NEW.zacatek is null) then
        if extract(day from OLD.zacatek) > 1 then
          pregenSmenuRok = extract(year from OLD.zacatek);
          pregenSmenuMesic = extract(month from OLD.zacatek);
        end if;
      end if;
      if (OLD.zacatek is null) then
        if extract(day from NEW.zacatek) > 1 then
          pregenSmenuRok = extract(year from NEW.zacatek);
          pregenSmenuMesic = extract(month from NEW.zacatek);
        end if;
      end if;
      if NEW.zacatek is not null and OLD.zacatek is not null then
        if (extract(month from NEW.zacatek) = extract(month from OLD.zacatek) and extract(year from NEW.zacatek) = extract(year from OLD.zacatek)) then
          pregenSmenuRok = extract(year from NEW.zacatek);       -- zmenil se den pocatku ale mesic zustal
          pregenSmenuMesic = extract(month from NEW.zacatek);
        else
          if (extract(day from OLD.zacatek) > 1 and (extract(month from NEW.zacatek) < extract(month from OLD.zacatek) or extract(year from NEW.zacatek) < extract(year from OLD.zacatek))) then
            pregenSmenuRok = extract(year from NEW.zacatek);    -- stary den pocatku neni 1 a mesic se posunul v case dozadu - tj mesic existule ale uz je tam celej
            pregenSmenuMesic = extract(month from NEW.zacatek);
          end if;
          if (extract(day from NEW.zacatek) > 1 and (extract(month from NEW.zacatek) > extract(month from OLD.zacatek) or extract(year from NEW.zacatek) > extract(year from OLD.zacatek))) then
            pregenSmenuRok = extract(year from NEW.zacatek);    -- novy den pocatku neni 1 a mesic se posunul v case dopredu - tj mesic existuje ale meni se
            pregenSmenuMesic = extract(month from NEW.zacatek);
          end if;
        end if;
      end if;
      
      if pregenSmenuMesic > 0 then
        select idsmeny into _idSmena from psmeny where idpracpom = NEW.idpracpomhlav and rok = pregenSmenuRok and mesic = pregenSmenuMesic;
        if _idSmena is not null then
          perform pcreatemzdslozforsmeny(_idSmena);
        end if;
      end if;
    end if;
    
    -- zkontrolujeme zda se nezmenil konec pomeru a neni treba neco pregenerovat
    if (comparedate(NEW.konecpomeru, OLD.konecpomeru) = false) then
      pregenSmenuOd = null;
      pregenSmenuDo = null;
      if (NEW.aktivnido is not null and (OLD.aktivnido is null or NEW.aktivnido <= OLD.aktivnido)) then
        pregenSmenuDo = NEW.aktivnido;
      end if;
      if (OLD.aktivnido is not null and (NEW.aktivnido is null or OLD.aktivnido < NEW.aktivnido)) then
        pregenSmenuDo = OLD.aktivnido;
      end if;

      if (NEW.konecpomeru is null) then
        pregenSmenuOd = date_trunc('month', OLD.konecpomeru);
        if iskonecmesice(OLD.konecpomeru) = true then
          pregenSmenuOd = pregenSmenuOd + interval '1 month';
        end if;
      end if;
      if (OLD.konecpomeru is null) then
        pregenSmenuOd = date_trunc('month', NEW.konecpomeru);
        if iskonecmesice(NEW.konecpomeru) = true then
          pregenSmenuOd = pregenSmenuOd + interval '1 month';
        end if;
      end if;
      if NEW.konecpomeru is not null and OLD.konecpomeru is not null then
        if (extract(month from NEW.konecpomeru) = extract(month from OLD.konecpomeru) and extract(year from NEW.konecpomeru) = extract(year from OLD.konecpomeru)) then
          pregenSmenuOd = date_trunc('month', NEW.konecpomeru);                 -- zmenil se den konce ale mesic zustal
          pregenSmenuDo = pregenSmenuOd;
        else
          if (NEW.konecpomeru < OLD.konecpomeru) then
            pregenSmenuOd = NEW.konecpomeru;
            pregenSmenuDo = date_trunc('month', OLD.konecpomeru);
          else
            pregenSmenuOd = OLD.konecpomeru;
            pregenSmenuDo = date_trunc('month', NEW.konecpomeru);
          end if;
          
          if iskonecmesice(pregenSmenuOd) then
            pregenSmenuOd = pregenSmenuOd + interval '1 day';
          else
            pregenSmenuOd = date_trunc('month', pregenSmenuOd);
          end if;
        end if;
      end if;
      
      if pregenSmenuOd is not null then
        perform pcreatemzdslozforsmeny(idsmeny) from psmeny where idpracpom = NEW.idpracpomhlav and 
          pregenSmenuOd <= date('' || rok || '-' || mesic || '-01') and
          (pregenSmenuDo is null or pregenSmenuDo >= date('' || rok || '-' || mesic || '-01'));
      end if;
    end if;
  end if;
  
  return NEW;
end;
$$ LANGUAGE plpgsql;
ALTER FUNCTION public.ppracpomhlavupdatemzdsloz() OWNER TO %firmaRole%;

-- zvyseni verze databaze
insert into dbversion(iddbversion, dbversion, needreimport, appversion, upgradetime)
  values (nextval('seq_dbversion'), 618, 't', %appVersion%, clock_timestamp());

