-- Bug 11396 – [WinStrom] Funkce Rezervovat
-- přidání tabulky pro rezervace
CREATE SEQUENCE seq_srezervace
  INCREMENT 1
  MINVALUE 1
  MAXVALUE 9223372036854775807
  START 1
  CACHE 1;
ALTER TABLE seq_srezervace OWNER TO %firmaRole%;

CREATE TABLE srezervace
(
  idrezervace integer NOT NULL DEFAULT nextval('seq_srezervace'::regclass),
  datumdo date,
  datumod date,
  mnozstvi numeric(19,6) NOT NULL DEFAULT 0.0,
  poznamka text,
  idpolobch integer,
  iduziv integer NOT NULL,
  idcenik integer NOT NULL,
  idfirmy integer NOT NULL,
  idsklad integer NOT NULL,
  CONSTRAINT srezervace_pkey PRIMARY KEY (idrezervace),
  CONSTRAINT fk1f4f5aae2ec2f001 FOREIGN KEY (iduziv)
      REFERENCES wuzivatel (iduzivatel) MATCH SIMPLE
      ON UPDATE NO ACTION ON DELETE NO ACTION,
  CONSTRAINT fk1f4f5aae39262734 FOREIGN KEY (idsklad)
      REFERENCES dtypdokl (idtypdokl) MATCH SIMPLE
      ON UPDATE NO ACTION ON DELETE NO ACTION,
  CONSTRAINT fk1f4f5aae3d003810 FOREIGN KEY (idpolobch)
      REFERENCES dpolobch (idpolobch) MATCH SIMPLE
      ON UPDATE NO ACTION ON DELETE NO ACTION,
  CONSTRAINT fk1f4f5aae451651ce FOREIGN KEY (idcenik)
      REFERENCES ccenik (idcenik) MATCH SIMPLE
      ON UPDATE NO ACTION ON DELETE NO ACTION,
  CONSTRAINT fk1f4f5aaef7b64b9e FOREIGN KEY (idfirmy)
      REFERENCES aadresar (idfirmy) MATCH SIMPLE
      ON UPDATE NO ACTION ON DELETE NO ACTION
)
WITH (OIDS=FALSE);
ALTER TABLE srezervace OWNER TO %firmaRole%;


-- Funkce aktualizujici rezervaci pro obchodni polozku
CREATE OR REPLACE FUNCTION rezervujPolObch(_idPolObch integer)
  RETURNS void AS $$
declare
  polObch record;
  doklObch record;
  mnoz numeric(19,6);
  _idUcetObd integer;
  _idRezer integer;
begin
  select * into polObch from dPolObch where idPolObch = _idPolObch;
  
  if (polObch.rezervovat = true and polObch.modul = 'OBP') then
    mnoz = polObch.mnozMj - polObch.mnozMjReal;
    
    if (mnoz > 0 and polObch.typPolozkyK = 'typPolozky.katalog' and polObch.idSkladu is not null) then            -- jestlize zbyva jeste neco realizovat a je to skladova pol
      select * into doklObch from dDoklObch where idDoklObch = polObch.idDoklObch;
      
      select idrezervace into _idRezer from srezervace where idPolObch = polObch.idPolObch;
      
      if _idRezer is null then    -- zaznam neexistuje, vytvorime ho
        insert into srezervace (idpolobch, iduziv, idfirmy, idcenik, idsklad, mnozstvi, datumod)
          values (polObch.idPolObch, doklObch.iduziv, doklObch.idfirmy, polObch.idCenik, polObch.idSkladu, mnoz, polObch.datVyst);
      else                        -- zaznam jiz existuje, upravime ho
        update srezervace set idfirmy = doklObch.idfirmy, idCenik = polObch.idCenik, idSklad = polObch.idSkladu,
          mnozstvi = mnoz, datumod = polObch.datVyst where idrezervace = _idRezer;
      end if;
    else                          -- pokud je polozka jiz realizovana
      delete from srezervace where idPolObch = polObch.idPolObch;
    end if;
  end if;
end; 
$$ LANGUAGE plpgsql;
ALTER FUNCTION public.rezervujPolObch(integer) OWNER TO %firmaRole%;


-- trigger volany pri vytvoreni nebo zmene obchodni polozky
CREATE OR REPLACE FUNCTION dpolobchupdaterezer()
  RETURNS trigger AS $$
begin
  if (tg_op = 'UPDATE') then
    -- pokud se jedna o update a meni se realizovane mnozstvi, postara se o to trigger na skladu
    if (OLD.mnozMjReal <> NEW.mnozMjReal) then
      return NEW;
    end if;
  end if;

  perform rezervujPolObch(NEW.idPolObch);
  
  return NEW;
end;
$$ LANGUAGE plpgsql;
ALTER FUNCTION public.dpolobchupdaterezer() OWNER TO %firmaRole%;

-- navěšení triggeru na tabulku položek obchodnich dokladů
CREATE TRIGGER dpolobchupdaterezer AFTER INSERT OR UPDATE ON dpolobch
  FOR EACH ROW EXECUTE PROCEDURE dpolobchupdaterezer();


-- trigger volany pri smazeni obchodni polozky
CREATE OR REPLACE FUNCTION dpolobchdeleterezer()
  RETURNS trigger AS $$
begin
  delete from srezervace where idpolobch = OLD.idpolobch;

  return OLD;
end;
$$ LANGUAGE plpgsql;
ALTER FUNCTION public.dpolobchdeleterezer() OWNER TO %firmaRole%;

-- navěšení triggeru na tabulku položek obchodnich dokladů
CREATE TRIGGER dpolobchdeleterezer BEFORE DELETE ON dpolobch
  FOR EACH ROW EXECUTE PROCEDURE dpolobchdeleterezer();



-- Funkce odrezervujici dane mnozstvi z pro danou cenikou polozku na sklade
CREATE OR REPLACE FUNCTION odrezervuj(_idFirmy integer, _idCenik integer, _idSkladu integer, vyskladni numeric(19,6))
  RETURNS void AS $$
declare
  _idRezer integer;
  _mnozstvi numeric(19,6);
  _vyskl numeric(19,6);
begin
  _vyskl = vyskladni;

  for _idRezer, _mnozstvi in select idRezervace, mnozstvi from sRezervace where idfirmy = _idFirmy and idCenik = _idCenik and idSklad = _idSkladu loop
    if _vyskl < _mnozstvi then
      update sRezervace set mnozstvi = (mnozstvi-_vyskl) where idRezervace = _idRezer;
      
      return;
    else
      delete from sRezervace where idRezervace = _idRezer;
      
      _vyskl = _vyskl - _mnozstvi;
    end if;
   
  end loop;
end; 
$$ LANGUAGE plpgsql;
ALTER FUNCTION public.odrezervuj(integer, integer, integer, numeric) OWNER TO %firmaRole%;


-- trigger rusici rezervaci pri vyskladneni
CREATE OR REPLACE FUNCTION dpolskladupdaterezer()
  RETURNS trigger AS $$
declare
  zmena numeric(19,6);
  _idFirmy integer;
begin
  zmena = 0;
  if (tg_op = 'UPDATE') then
    zmena = OLD.mnozMj;
  end if;
  
  zmena = NEW.mnozMj - zmena;

  if (zmena > 0) then
    select idfirmy into _idFirmy from ddoklsklad where idDoklSklad = NEW.idDoklSklad;
    
    perform odrezervuj(_idFirmy, NEW.idCenik, NEW.idSkladu, zmena);
  end if;

  return NEW;
end;
$$ LANGUAGE plpgsql;
ALTER FUNCTION public.dpolskladupdaterezer() OWNER TO %firmaRole%;

-- navěšení triggeru na tabulku položek skladovych dokladů
CREATE TRIGGER dpolskladupdaterezer AFTER INSERT OR UPDATE ON dpolsklad
  FOR EACH ROW EXECUTE PROCEDURE dpolskladupdaterezer();

-- zvyseni verze databaze
insert into dbversion(iddbversion, dbversion, needreimport) values (nextval('seq_dbversion'), 64, 't');