--------------------------------------------------------------------------------
-- Bug 35564 – [FlexiBee] Doplnit funkce pro řešení skladu (profizoo)
--------------------------------------------------------------------------------

CREATE OR REPLACE FUNCTION skladgenerujrozsirenepolozky(i_datum timestamp, i_idpolsklad integer, i_idskladkarty integer)
  RETURNS boolean AS
$BODY$
declare
  tmp_skladovekarty_exists boolean;
  tmp_rozsirenepolozky_exists boolean;
  _polSklad record;
  _skladKarta record;
  _pocetSkladKarty integer;
  _result boolean;
  _idUcetObd integer;
begin
  -- pokud se zde nebyli tabulky vytvoreny tak se vrati false a pokud ano tak se true
  _result = false;

  -- vytvoreni temporary tabulky pro ulozeni seznamu skladovych karet na kterych dochazi k prepoctu
  tmp_skladovekarty_exists := (select istableexist('tmp_SkladoveKarty'));
  if(tmp_skladovekarty_exists <> true) then
    --raise exception 'tmp_skladoveKarty se vytvari';
    CREATE TEMPORARY TABLE tmp_SkladoveKarty (
      idskladkarty integer
    ) WITH (OIDS=TRUE) ON COMMIT DROP;

    CREATE INDEX tmp_SkladoveKarty_idskladkarty_index ON tmp_SkladoveKarty USING btree (idskladkarty );
  end if;

  -- vytvoreni temporary tabulky pro ulozeni seznamu polozek, ktere budou prepocitavat skladove karty
  tmp_rozsirenepolozky_exists := (select istableexist('tmp_RozsirenePolozky'));
  if(tmp_rozsirenepolozky_exists <> true) then
    CREATE TEMPORARY TABLE tmp_RozsirenePolozky (
      idpolsklad integer,
      iddoklskladMaster integer,
      iddoklskladpolozky integer,
      datum timestamp, 
      datVyst date,
      skupina integer,
      typPohybuK character varying,
      idskladkarty integer,
      storno boolean,
      cenyRucne boolean,
      evidVyrCis boolean,
      idcenik integer,
      idbspskl integer,
      mnozMj numeric(19,6)
      --cenaMj numeric(19,6),
      --sumZkl numeric(19,6)
    ) WITH (OIDS=TRUE) ON COMMIT DROP;

    CREATE INDEX tmp_RozsirenePolozky_idpolsklad_index ON tmp_RozsirenePolozky USING btree (idpolsklad );
    
    -- tabulky zde byli vytvoreny tim padem se musi vracet true  
    _result = true;
  end if;
  
  if(i_idSkladKarty is not null and i_idSkladKarty <> -1) then
    -- pokud se jedna o prepocet po vlozeni jedna polozky nebo o rekurzi tak se vse dela jen s jednou skladovou kartou
    --select count(*) into _pocetSkladKarty from tmp_SkladoveKarty where idskladkarty = i_idSkladKarty;
    --if(_pocetSkladKarty = 0) then
    select 1 into _pocetSkladKarty from tmp_SkladoveKarty where idskladkarty = i_idSkladKarty limit 1;
    if (_pocetSkladKarty is null) then
      insert into tmp_SkladoveKarty (idskladkarty) values (i_idSkladKarty);
      for _polSklad in (select * from dpolsklad where datSklad >= i_datum and idkarty = i_idSkladKarty) loop
        execute skladInsertRozsirenouPolozku(_polSklad.idPolSklad);
      end loop;
    else
      -- do something stupid - kdyz je skladkarta v seznamu
    end if;
  else
    -- pokud se jedna o celkovy prepocet skladu tak do tmp tabulky skladovych karet dam vsechny pro dane obdobi
    select iducetobdobi into _idUcetObd from nucetobdobi where platioddata <= i_datum and platidodata >= i_datum;
    for _skladKarta in (select * from skarty where iducetobdobi = _idUcetObd) loop
      select 1 into _pocetSkladKarty from tmp_SkladoveKarty where idskladkarty = _skladKarta.idKarty limit 1;
      if (_pocetSkladKarty is null) then
        insert into tmp_SkladoveKarty(idskladkarty) values (_skladKarta.idKarty);
      end if;
      --for _polSklad in (select * from dpolsklad where datSklad >= i_datum and idkarty = _skladKarta.idKarty) loop
      -- tento for cyklus je spatne protoze pokud jsou spatna data (dat vyst je 2010 a karta 2011) tak k prepoctu nedojde
      for _polSklad in (select * from dpolsklad where idkarty = _skladKarta.idKarty) loop
        execute skladInsertRozsirenouPolozku(_polSklad.idPolSklad);
      end loop;
    end loop;
  end if;

  return _result;
end;
$BODY$
  LANGUAGE 'plpgsql' VOLATILE
  COST 100;
ALTER FUNCTION skladGenerujRozsirenePolozky(timestamp, integer, integer ) OWNER TO %firmaRole%;


create or replace function skladInsertRozsirenouPolozku(i_idPolSklad integer) returns void as
$BODY$
declare
  _polSklad record;
  _navazanaPolSklad record;
  _pocetRozsirenePolSklad integer;
  _typPohybuK character varying;
  _pocetProSkupinu integer;
  _skupina integer;
  _idDoklSkladMaster integer;
  _idDoklSkladMasterPom integer;
  _mazatTmpTable boolean;
  _evidVyrCis boolean;
begin
  --select count(*) into _pocetRozsirenePolSklad from tmp_RozsirenePolozky where idPolSklad = i_idPolSklad;
  --if(_pocetRozsirenePolSklad = 0) then
  select 1 into _pocetRozsirenePolSklad from tmp_RozsirenePolozky where idPolSklad = i_idPolSklad limit 1;
  if(_pocetRozsirenePolSklad is null) then
    select * into _polSklad from dPolSklad where idPolSklad = i_idPolSklad;
    select typPohybuK into _typPohybuK from dDoklSklad where idDoklSklad = _polSklad.idDoklSklad;

    -- urceni skupiny do ktere dana polozka patri
    _skupina = 2;
    select 1 into _pocetProSkupinu from dvazebtab
         where (idtabA = _polSklad.idPolSklad and modulA = 'SKL' and (typVazbyK = 'typVazbyDokl.polozkaSklPrevod' or typVazbyK = 'typVazbyDokl.polozkaSklKus'))
             or (idtabB = _polSklad.idPolSklad and modulB = 'SKL' and (typVazbyK = 'typVazbyDokl.polozkaSklPrevod' or typVazbyK = 'typVazbyDokl.polozkaSklKus'))
         limit 1;
    if(_pocetProSkupinu is null and _polSklad.idPolDobropis is null) then
      if(_typPohybuK = 'typPohybu.prijem') then
        _skupina = 1;
      else
        _skupina = 3;
      end if;
    end if;

    -- urceni id skladoveho doklad ktery je master
    _idDoklSkladMaster = _polSklad.idDoklSklad;
    if(_skupina = 2) then
      -- _idDoklSkladMasterPom
      select idtabA into _idDoklSkladMasterPom from dvazebtab where idtabB = _polSklad.idDoklSklad and modulB = 'SKL' and modulA = 'SKL' and 
          (typVazbyK = 'typVazbyDokl.hlavaSklPrevod' or typVazbyK = 'typVazbyDokl.hlavaSklKus');
      if(_idDoklSkladMasterPom is not null) then
        _idDoklSkladMaster = _idDoklSkladMasterPom;
      end if;
    end if;

    -- rekurzi spoustim pro prevodku a kusovnik
    if(_skupina = 2 and _typPohybuK = 'typPohybu.vydej') then
      select * into _navazanaPolSklad from dPolSklad 
        where idPolSklad = (select idTabB from dvazebtab where idTabA = _polSklad.idPolSklad and modulA = 'SKL' and typVazbyK = 'typVazbyDokl.polozkaSklPrevod');

      -- pokud neni soucasti prevodky tak se jedna o kusovnik
      if(_navazanaPolSklad is null) then
        select * into _navazanaPolSklad from dPolSklad
          where idPolSklad = (select idTabA from dVazebTab where idTabB = _polSklad.idPolSklad and modulB = 'SKL' and typVazbyK = 'typVazbyDokl.polozkaSklKus');
      end if;
      
      if(_navazanaPolSklad is null) then
        --raise notice 'navazana je null';
      else
        --raise notice 'volam rekurzi';
        select skladGenerujRozsirenePolozky(_navazanaPolSklad.datSklad, _navazanaPolSklad.idPolSklad, _navazanaPolSklad.idKarty ) into _mazatTmpTable;
      end if;
    end if;

    select evidVyrCis into _evidVyrCis from ccenik where idcenik = _polSklad.idcenik;
    -- vlozeni zaznamu o polozce do tmp tabulky
    insert into tmp_RozsirenePolozky (idpolsklad, iddoklskladMaster, datum, datVyst, skupina, typPohybuK, idskladkarty, mnozMj, /*cenaMj, sumZkl,*/ iddoklskladpolozky, storno, cenyRucne, evidVyrCis, idcenik, idbspskl)
            values (_polSklad.idPolSklad, _idDoklSkladMaster, _polSklad.datSklad, _polSklad.datVyst, _skupina, _typPohybuK, _polSklad.idkarty, _polSklad.mnozMj, /*_polSklad.cenaMj, _polSklad.sumZkl,*/ _polSklad.idDoklSklad, _polSklad.storno, _polSklad.cenyRucne, _evidVyrCis, _polSklad.idcenik, _polSklad.idbspskl);
  else
    -- do something stupid - kdyz je polozka v seznamu
  end if;
end;
$BODY$
  LANGUAGE 'plpgsql' VOLATILE
  COST 100;
ALTER FUNCTION skladInsertRozsirenouPolozku(integer) OWNER TO %firmaRole%;


DROP FUNCTION IF EXISTS szkontrolujsklad(integer);

CREATE OR REPLACE FUNCTION szkontrolujsklad(i_ucetobd integer, i_typVypisu integer)
  RETURNS void AS
$BODY$ 
declare 
  _pocatek_ucetobd date;
  _konec_ucetobd date; 
  _mazatTmpTable boolean;
  _pocetPred integer; 
  _skladKartaId integer;
  _rozsirenaSkladKarta record;
  _stavMj numeric(19,6);
  _rozsirenaPolozka record;
  _index integer;
  tmp_result_exists boolean; 
begin 
  -- 1 = vypis skladovych karet, ktere jsou spatne
  -- 2 = vypis skladovych karet, ktere jsou spatne i s polozkama, ktere je do tohoto spatne privadeji
  -- 3 = do tabulky
  -- else = exception
  
  if(i_typVypisu = 3) then
    tmp_result_exists := (select istableexist('tmp_result')); 
    if(tmp_result_exists <> true) then 
      CREATE TEMPORARY TABLE tmp_result ( 
        idpolsklad integer, 
        idskladkarty integer,
        sklad character varying,
        cenik character varying
        --iddoklskladMaster integer, 
        --iddoklskladpolozky integer, 
        --datum timestamp,  
        --skupina integer, 
        --typPohybuK character varying, 
        --storno boolean, 
        --cenyRucne boolean, 
        --evidVyrCis boolean, 
        --idcenik integer, 
        --idbspskl integer, 
        --mnozMj numeric(19,6) 
        --cenaMj numeric(19,6), 
        --sumZkl numeric(19,6) 
      ) WITH (OIDS=TRUE) ON COMMIT DROP; 
 
      --CREATE INDEX tmp_RozsirenePolozky_idpolsklad_index ON tmp_RozsirenePolozky USING btree (idpolsklad ); 
       
      -- tabulky zde byli vytvoreny tim padem se musi vracet true   
      --_result = true; 
    else
      delete from tmp_result;
    end if; 
  end if;
   
  select platioddata, platidodata into _pocatek_ucetobd, _konec_ucetobd from nucetobdobi where iducetobdobi = i_ucetobd; 
 
  -- nagenerovani pomocnych tabulek 
  select skladGenerujRozsirenePolozky(_pocatek_ucetobd, -1, -1) into _mazatTmpTable; 
 
  select count(*) into _pocetPred from tmp_SkladoveKarty;
  raise notice 'pocet skladkaret pred %', _pocetPred;

  select count(*) into _pocetPred from tmp_RozsirenePolozky; 
  raise notice 'pocet pred %', _pocetPred;

  _index = 0;
  for _skladKartaId in (select * from tmp_SkladoveKarty) loop
    select * into _rozsirenaSkladKarta from (
      select idkarty, c.kod as cenikKod, s.kod as skladKod
        from skarty k
        left join ccenik c on k.idcenik = c.idcenik
        left join dbsp s on s.idbsp = k.idskladu
      where idkarty = _skladKartaId) as sklkar;
    _index = _index + 1;
    --raise notice 'pocitam skladovou kartu id = % a je to jiz %. karta', _skladKartaId, _index;
    select pocatMj into _stavMj from skarty where idkarty = _skladKartaId;
    for _rozsirenaPolozka in (select * from tmp_RozsirenePolozky 
        where storno = false and idskladkarty = _skladKartaId
        order by datVyst, skupina, iddoklskladMaster, typpohybuk desc, idpolsklad) loop 
      if(_rozsirenaPolozka.typPohybuK = 'typPohybu.prijem') then
        _stavMj = _stavMj + _rozsirenaPolozka.mnozMj;
      else
        _stavMj = _stavMj - _rozsirenaPolozka.mnozMj;
      end if;
      if(_stavMj < 0.0) then
        if(i_typVypisu = 1) then
          raise notice 'v zaporu je skladova karta id=%, sklad=%, cenik=%', _skladKartaId, _rozsirenaSkladKarta.skladKod, _rozsirenaSkladKarta.cenikKod;
          exit;
        elsif(i_typVypisu = 2) then
          raise notice 'skladova karta id=% je v minusu po polozce id=%', _skladKartaId, _rozsirenaPolozka.idPolSklad;
        elsif(i_typVypisu = 3) then
          insert into tmp_result(idpolsklad, idskladkarty, sklad, cenik)
            values (_rozsirenaPolozka.idPolSklad, _skladKartaId, _rozsirenaSkladKarta.skladKod, _rozsirenaSkladKarta.cenikKod);
        else
          raise exception 'Nepodporovany typ vypisu ale skladov karta s id % je v zaporu po polozce id %', _skladKartaId, _rozsirenaPolozka.idPolSklad;
        end if;
      end if;
    end loop;
  end loop;

  if(_mazatTmpTable) then 
    drop table tmp_RozsirenePolozky; 
    drop table tmp_SkladoveKarty; 
    execute removeglotxvariable('zurnalTxName'); 
    execute removeglotxvariable('probihaPrepocetSkladu'); 
  end if; 
 
end; 
$BODY$
  LANGUAGE plpgsql VOLATILE
  COST 100;
ALTER FUNCTION szkontrolujsklad(integer, integer) OWNER TO %firmaRole%;


CREATE OR REPLACE FUNCTION zapisDatVystDoDatSklad(i_ucetobd integer)
  RETURNS void AS
$BODY$  
declare  
begin
  alter table dpolsklad 
    disable trigger dcpolskladupdate,
    disable trigger dpolskladupdatecenamjnakupnafakture;
    
  update dpolsklad set datsklad = datvyst
    where idkarty in (select idkarty from skarty where iducetobdobi = i_ucetobd)
      and datSklad <> datVyst;
  
  alter table dpolsklad 
    enable trigger dcpolskladupdate,
    enable trigger dpolskladupdatecenamjnakupnafakture;


  --select sprepoctisklad(4, -1); 
end;  
$BODY$
  LANGUAGE plpgsql VOLATILE
  COST 100;
ALTER FUNCTION zapisDatVystDoDatSklad(integer) OWNER TO %firmaRole%;

-- zvyseni verze databaze
insert into dbversion(iddbversion, dbversion, needreimport, appversion, upgradetime)
  values (nextval('seq_dbversion'), 774, 't', %appVersion%, clock_timestamp());
