<?xml version="1.0" encoding="utf-8" ?>
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:java="http://xml.apache.org/xalan/java" exclude-result-prefixes="java">
  <xsl:output method="xml" indent="yes" encoding="utf-8" />
  <xsl:template match="MoneyData">
    <winstrom version="1.0">
      <xsl:apply-templates select="./SeznamFirem/Firma" />
    </winstrom>
  </xsl:template>

  <xsl:template name="NovaBanka">
    <xsl:if test="java:flexibee.MigrationTool.shouldCreate('smerKod', KodBanky)">
        <penezni-ustav update="ignore">
          <id>code:<xsl:value-of select="KodBanky"/></id>
          <kod><xsl:value-of select="KodBanky"/></kod>
          <xsl:if test="Banka">
            <nazev><xsl:value-of select="Banka"/></nazev>
          </xsl:if>
          <xsl:if test="not(Banka)">
            <nazev><xsl:value-of select="KodBanky"/></nazev>
          </xsl:if>
        </penezni-ustav>
    </xsl:if>
  </xsl:template>
  
  <xsl:template name="NovaSkupina">
    <xsl:if test="./Skupina/Zkratka != ''">
      <skupina-firem update="ignore">
        <id>ext:ms3sf:<xsl:value-of select="java:flexibee.Tool.toUpper(./Skupina/Zkratka)"/></id>
        <id>code:<xsl:value-of select="java:flexibee.Tool.toUpper(./Skupina/Zkratka)"/></id>
        <kod><xsl:value-of select="java:flexibee.Tool.toUpper(./Skupina/Zkratka)"/></kod>
        <nazev><xsl:value-of select="./Skupina/Nazev"/></nazev>
      </skupina-firem>
    </xsl:if>
  </xsl:template>
  
  <xsl:template match="Firma">
  <!-- zalozeni nove banky -->
  <xsl:call-template name="NovaBanka"/>
  <!-- skupina firem -->
  <xsl:call-template name="NovaSkupina"/>

  <!-- adresar -->
  <adresar>
    <id>ext:mS3adr:<xsl:value-of select="substring(GUID, 2, string-length(GUID)-2)"/></id>
    <xsl:if test="Nazev">
      <nazev><xsl:value-of select="Nazev"/></nazev>
    </xsl:if>
    <xsl:if test="not(Nazev)">
      <nazev><xsl:value-of select="substring(GUID, 2, 20)"/></nazev>
    </xsl:if>
    <kod><xsl:value-of select="substring(GUID, 2, 20)"/></kod>
    <ulice><xsl:value-of select="./Adresa/Ulice"/></ulice>
    <mesto><xsl:value-of select="./Adresa/Misto"/></mesto>
    <psc><xsl:value-of select="./Adresa/PSC"/></psc>
    <xsl:if test="./Adresa/KodStatu != ''">
      <stat>code:<xsl:value-of select="java:flexibee.Tool.toUpper(./Adresa/KodStatu)"/></stat>
    </xsl:if>
    
    <ic><xsl:value-of select="ICO"/></ic>
    <tel><xsl:value-of select="concat(./Tel/Pred, ./Tel/Cislo)"/></tel>
    <mobil><xsl:value-of select="concat(./Mobil/Pred, ./Mobil/Cislo)"/></mobil>
    
    <!-- fakturacni adresa -->
    <xsl:if test="FaktNazev != Nazev  or ./FaktAdresa/Ulice != ./Adresa/Ulice or ./FaktAdresa/Misto != ./Adresa/Misto or ./FaktAdresa/PSC != ./Adresa/PSC or ./FaktAdresa/KodStatu != ./Adresa/KodStatu">
      <postovniShodna>false</postovniShodna>
      <faJmenoFirmy><xsl:value-of select="FaktNazev"/></faJmenoFirmy>
      <faulice><xsl:value-of select="./FaktAdresa/Ulice"/></faulice>
      <famesto><xsl:value-of select="./FaktAdresa/Misto"/></famesto>
      <fapsc><xsl:value-of select="./FaktAdresa/PSC"/></fapsc>
      <xsl:if test="./FaktAdresa/KodStatu">
        <faStat>code:<xsl:value-of select="./FaktAdresa/KodStatu"/></faStat>
      </xsl:if>
    </xsl:if>

    <platceDph>
      <xsl:choose><xsl:when test="PlatceDPH = '0'">false</xsl:when>
      <xsl:otherwise>true</xsl:otherwise></xsl:choose>
    </platceDph>

    <xsl:if test="./Skupina/Zkratka != ''">
      <skupFir>code:<xsl:value-of select="java:flexibee.Tool.toUpper(./Skupina/Zkratka)"/></skupFir>
    </xsl:if>
    <typVztahuK>typVztahu.odberDodav</typVztahuK>
    
    <poznam><xsl:value-of select="Poznamka"/></poznam>
  </adresar>
  
  <xsl:if test="Ucet != '' and KodBanky != ''">
    <!-- bankovni spojeni -->
    <adresar-bankovni-ucet>
      <id>ext:mS3bu:<xsl:value-of select="concat(substring(GUID, 2, string-length(GUID)-2), KodBanky)"/></id>
      <buc><xsl:value-of select="Ucet"/></buc>
      <smerKod><xsl:value-of select="java:flexibee.MigrationTool.getValidOrNearestIdent('smerKod', KodBanky)"/></smerKod>
      <nazBanky><xsl:value-of select="Banka"/></nazBanky>
      <firma>ext:mS3adr:<xsl:value-of select="substring(GUID, 2, string-length(GUID)-2)"/></firma>
    </adresar-bankovni-ucet>
  </xsl:if>
  
  </xsl:template>
  
  <xsl:template name="getSmerKod">
    <xsl:param name="kodBanky"/>
    <xsl:choose>
      <xsl:when test="../../KodBankyTrans[From = $kodBanky]">
        <xsl:value-of select="../../KodBankyTrans[From = $kodBanky]/To"/>
      </xsl:when>
      <xsl:otherwise>
        <xsl:value-of select="concat('code:', java:flexibee.Tool.toUpper($kodBanky))"/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>
</xsl:stylesheet>
